C****************************************************************************
C      PROGRAM CALCJ117
C                       CALCJ117 Version 1.3
C                  COPYRIGHT 1992, 1995, 1996, 1999 ALL RIGHTS RESERVED    
C                 SYSTEMS APPLICATIONS INTERNATIONAL, INC. 
C                              May 12, 1999
c modified by EMN 2-24-95 so that a 6th species (ALDX) could be
c included. Note arrays are large enough to handle 10 species,
c but GETXQ subroutine had been hard-coded for 5 species.
C
C
C     Program - < CALCJ >                              Greg Yarwood  Sept 91
c$Log: calcj.f,v $
c  Revision 1.2  1995/02/22  19:46:02  tcm
c  Added check for missing value (= -9.) on input value for ozone col,
c  albedo, or haze.  This allows the user to create a single table, for
c  example, for input to uam-iv v6.4 by specifying only 1 genuine value
c  for each of these parameters.  Then only a 1 X 1 X 1 matrix is
c  calculated.
c
c  Revision 1.1  94/02/09  15:43:29  tcm
c  Initial revision
*
* Added acrolein cross section and quantum yield data, for calculating
* ISOPROD photoloysis rates with CARTER isop mechanism
* 1996/05/28 hpd
c  
C
C     Based on RTMRATES.FTN by Mike Gery
C
C     Calculates photolysis rates from cross-section and quantum yield data
C     using the parameterized solar flux of Schippnick and Green [Photochem
C     and Photobiol, 35, 89-101, 1982] and Green, Cross and Smith [ibid, 31
C     59-65, 1980].
C
C****************************************************************************
      PROGRAM CALCJ
C
      CHARACTER*80  TITLE, FDATA, FRTM
C
      INTEGER  Z(10)
      CHARACTER*4 SPNAME
C
      REAL  N1BAR, J2D, LITR, MUU, MDD, NN1BAR, MD0,
     $      NI(4), HI(4), TAU(4), ALPH(4), FI(4),
     $      XI(4), YI(4), TI(4),  UI(4),   EPSU(4), EPSD(4),
     $      ZA(10), TJ(10), SUMJ(10,10),
     $      ALTKM(11),
     $      TAU1(28), TAU2(28), TAU4(28), TAU2A(28),
     $      TAU4A(28), MPT(28),
     $      LAM(150), KO3(150), LAMDA(150), DSUM(150),
     $      G(150), H(150), GDIFF(150), PFLUX(150), ETR(150),
     $      EPHOT(150), APF(141,10),
     $      O3INP(5), ALBINP(5), HAZINP(3)
C
      COMMON /CNTROL/ NXANDQ, NUMSPE
      COMMON /VALUES/ SIG(20,150), PHI(20,150), LAMLOW(10), LAMHI(10),
     $                SPNAME(2,10)
C
C---------------------------------------------------------------------------
      DATA ZA   /0.,10.,20.,30.,40.,50.,60.,70.,78.,86./
      DATA Z    /0, 10, 20, 30, 40, 50, 60, 70, 78, 86/
      DATA ALTKM/0.,.15,.36,.64,.98,1.38,1.84,2.35,2.91,3.53,4.21/
C
C Program works with 1 nm bins centered on half nm's.
C Parameters specified every 5 nm are interpolated.
C Mid-points of 5 nm bins.
      DATA MPT  / 282.5,287.5,292.5,297.5,302.5,
     $            307.5,312.5,317.5,322.5,327.5,
     $            332.5,337.5,342.5,347.5,352.5,
     $            357.5,362.5,367.5,372.5,377.5,
     $            382.5,387.5,392.5,397.5,402.5,
     $            407.5,412.5,417.5/
C THESE NUMBERS FROM GCS 1980
C Raliegh scattering O.D.
      DATA TAU1 / 1.590,1.470,1.361,1.265,1.178,
     $            1.098,1.025,0.958,0.896,0.839,
     $            0.788,0.739,0.694,0.653,0.617,
     $            0.582,0.548,0.517,0.489,0.462,
     $            0.438,0.415,0.393,0.373,0.354,
     $            0.336,0.320,0.304/
C Aerosol scattering O.D.
      DATA TAU2 / 0.079,0.079,0.080,0.081,0.081,
     $            0.081,0.082,0.082,0.082,0.083,
     $            0.083,0.083,0.083,0.084,0.084,
     $            0.084,0.085,0.085,0.085,0.086,
     $            0.086,0.086,0.086,0.087,0.087,
     $            0.087,0.087,0.088/
C Aerosol absorption O.D.
      DATA TAU4 / 0.0139,0.0138,0.0137,0.0136,0.0134,
     $            0.0133,0.0132,0.0130,0.0129,0.0128,
     $            0.0127,0.0126,0.0124,0.0123,0.0122,
     $            0.0122,0.0121,0.0120,0.0119,0.0118,
     $            0.0112,0.0111,0.0111,0.0109,0.0108,
     $            0.0107,0.0106,0.0104/
C THESE NUMBERS FROM DEMERJIAN
C     DATA TAU2 / 0.201,0.202,0.203,0.204,0.205,
C    $            0.206,0.206,0.207,0.209,0.210,
C    $            0.211,0.211,0.212,0.213,0.214,
C    $            0.215,0.216,0.217,0.217,0.218,
C    $            0.219,0.219,0.220,0.221,0.221,
C    $            0.222,0.223,0.223/
C     DATA TAU4 / 0.3719,0.3688,0.3657,0.3636,0.3574,
C    $            0.0355,0.0352,0.0347,0.0344,0.0341,
C    $            0.0339,0.0336,0.0331,0.0328,0.0325,
C    $            0.0325,0.0323,0.0320,0.0317,0.0315,
C    $            0.0299,0.0296,0.0296,0.0291,0.0288,
C    $            0.0285,0.0283,0.0279/
C
C Data for TAU3 i.e. O.D. due to ozone absorption.
C Cross-sections of Molina and Molina, JGR 91:14501 at 226K
C converted to (atm.cm)-1 using conversion factor for 298K
C (an artifact of expressing ozone column in Dobson Units).
      DATA KO3  / 97.98, 89.59, 79.21, 73.55, 65.04
     $          , 58.99, 54.00, 48.66, 41.50, 37.76
     $          , 33.58, 30.14, 25.66, 23.07, 20.05
     $          , 17.88, 15.25, 13.46, 11.59, 10.41
     $          , 8.905, 7.700, 6.888, 5.732, 5.314
     $          , 4.357, 3.943, 3.363, 2.957, 2.617
     $          , 2.125, 1.951, 1.648, 1.400, 1.311
     $          , 1.031, 0.959, 0.824, 0.753, 0.492
     $          , 0.704, 0.337, 0.536, 0.376, 0.193
     $          , 0.367, 0.179, 0.152, 0.285, 0.085
     $          , 0.070, 0.175, 0.059, 0.054, 0.093
     $          , 0.031, 0.021, 0.061, 0.032, 0.014
     $          , 0.038, 0.015, 0.007, 0.009, 0.031
     $          , 0.010, 0.005, 0.007, 0.004, 0.002
     $          , 0.002, 79*0.0/
C
C GCS 1980 data in 5-10 nm bins
C     DATA KO3  /  77.24, 77.24, 77.24, 77.24, 77.24
C    $            ,44.49, 44.49, 44.49, 44.49, 44.49
C    $            ,25.95, 25.95, 25.95, 25.95, 25.95
C    $            ,13.58, 13.58, 13.58, 13.58, 13.58
C    $            ,6.76 ,  6.76,  6.76,  6.76,  6.76
C    $            ,3.50 ,  3.50,  3.50,  3.50,  3.50
C    $            ,1.73 ,  1.73,  1.73,  1.73,  1.73
C    $            ,0.89 ,  0.89,  0.89,  0.89,  0.89
C    $            ,0.46 ,  0.46,  0.46,  0.46,  0.46
C    $            ,0.173, 0.173, 0.173, 0.173, 0.173
C    $            ,0.173, 0.173, 0.173, 0.173, 0.173
C    $            ,0.052, 0.052, 0.052, 0.052, 0.052
C    $            ,0.052, 0.052, 0.052, 0.052, 0.052
C    $            ,0.009, 0.009, 0.009, 0.009, 0.009
C    $            ,0.009, 0.009, 0.009, 0.009, 0.009
C    $            ,0.001, 0.001, 0.001, 0.001, 0.001
C    $            ,0.001, 0.001, 0.001, 0.001, 0.001
C    $            ,65*0.0/
C
C Extra-terrestrial solar flux.
C THESE ARE DEMERJIAN NUMBERS
C     DATA ETR / 1.05, 1.66, 2.67, 2.75, 2.70, 3.17, 3.90,
C    $           3.92, 4.09, 5.28, 5.19, 4.89, 5.09, 4.94,
C    $           5.47, 4.92, 5.34, 6.43, 5.70, 6.24, 5.07,
C    $           5.48, 5.55, 6.62, 7.68, 8.49, 8.81, 8.80/
C THE NUMBERS ABOVE ARE FROM DEMERJIAN BUT DO NOT AVERAGE TO HIS
C NUMBERS SO THE NUMBERS BELOW WERE CREATED TO DO SO
C     DATA ETR / 1.05, 1.66, 2.82, 2.77, 2.63, 3.11, 4.02,
C    $           3.90, 3.88, 5.53, 5.20, 4.80, 5.19, 4.78,
C    $           5.70, 4.74, 5.19, 6.83, 5.35, 6.65, 4.71,
C    $           5.63, 5.36, 6.63, 7.71, 8.56, 8.85, 8.80/
C
C These ET data are from the WRC, and were originally in 2 nm bins
C Now in 1 nm bins
C M. Iqbal. An Intro to Solar Radiation. Academic Press, 1983.
      DATA ETR /.200,.140,.163,.185,.260,.335,.280,.225,.293,.360,.405,
     $          .450,.535,.620,.593,.565,.568,.570,.553,.535,.515,
     $          .495,.528,.560,.545,.530,.538,.545,.578,.610,.580,
     $          .550,.603,.655,.668,.680,.695,.710,.715,.720,.718,
     $          .715,.748,.780,.730,.680,.688,.695,.878,1.06,1.028,
     $          .995,.998,1.00,.975,.950,.943,.935,.870,.805,.878,
     $          .950,.960,.970,.958,.945,.888,.830,.868,.905,.903,
     $          .900,.955,1.01,1.015,1.02,1.078,1.135,1.013,.89,.875,
     $          .86,.94,1.02,1.05,1.08,1.088,1.095,1.163,1.23,1.203,
     $         1.175,1.165,1.155,1.05,.945,.99,1.035,1.173,1.31,1.283,
     $         1.255,1.21,1.165,.943,.72,.87,1.02,.993,.965,1.043,
     $         1.12,1.2,1.28,1.013,.745,1.005,1.265,1.063,.86,1.233,
     $         1.605,1.703,1.8,1.74,1.68,1.658,1.635,1.63,1.625,1.66,
     $         1.695,1.71,1.725,1.743,1.76,1.743,1.725,1.753,1.78,
     $         1.72,9*0./
C
C These ET data are from WMO 1985.  They were in variable bin sizes     
C and have NOT been interpolated in going to 1 nm bins.   
C     DATA ETR /0.138,0.138,0.261,0.261,0.261,0.261,0.359,0.359,0.359,0.359
C    $         ,0.564,0.564,0.564,0.564,0.564, 0.52, 0.52, 0.52, 0.52,0.479
C    $         ,0.479,0.479,0.479,0.479,0.613,0.613,0.613,0.613,0.613,0.665
C    $         ,0.665,0.665,0.665,0.741,0.741,0.741,0.741,0.741,0.778,0.778
C    $         ,0.778,0.778,0.778,0.857,0.857,0.857,0.857,0.857,1.044,1.044
C    $         ,1.044,1.044,1.044,0.973,0.973,0.973,0.973,0.973,1.052,1.052
C    $         ,1.052,1.052,1.052,0.979,0.979,0.979,0.979,0.979,0.993,0.993
C    $         ,0.993,0.993,0.993,1.030,1.030,1.030,1.030,1.030,0.915,0.915
C    $         ,0.915,0.915,0.915,1.168,1.168,1.168,1.168,1.168,1.168,1.168
C    $         ,1.168,1.168,1.168,1.037,1.037,1.037,1.037,1.037,1.169,1.169
C    $         ,1.169,1.169,1.169,0.933,0.933,0.933,0.933,0.933,1.211,1.211
C    $         ,1.211,1.211,1.211,0.946,0.946,0.946,0.946,0.946,1.690,1.690
C    $         ,1.690,1.690,1.690,1.679,1.679,1.679,1.679,1.679,1.796,1.796
C    $         ,1.796,1.796,1.796,1.899,1.899,1.899,1.899,1.899,1.858,1.858
C    $         ,1.858, 9*0.0/
C
C These ET data are from Demerjian and were in 5 nm bins (see above)
C and have NOT been interpolated in going to 1 nm bins.   
C     DATA ETR / 0.21, 0.21, 0.21, 0.21, 0.21,0.332,0.332,0.332,0.332,0.332
C    $         ,0.564,0.564,0.564,0.564,0.564,0.554,0.554,0.554,0.554,0.554
C    $         ,0.526,0.526,0.526,0.526,0.526,0.622,0.622,0.622,0.622,0.622
C    $         ,0.804,0.804,0.804,0.804,0.804, 0.78, 0.78, 0.78, 0.78, 0.78
C    $         ,0.776,0.776,0.776,0.776,0.776,1.106,1.106,1.106,1.106,1.106
C    $         , 1.04, 1.04, 1.04, 1.04, 1.04, 0.96, 0.96, 0.96, 0.96, 0.96
C    $         ,1.038,1.038,1.038,1.038,1.038,0.956,0.956,0.956,0.956,0.956
C    $         , 1.14, 1.14, 1.14, 1.14, 1.14,0.948,0.948,0.948,0.948,0.948
C    $         ,1.038,1.038,1.038,1.038,1.038,1.366,1.366,1.366,1.366,1.366
C    $         , 1.07, 1.07, 1.07, 1.07, 1.07, 1.33, 1.33, 1.33, 1.33, 1.33
C    $         ,0.942,0.942,0.942,0.942,0.942,1.126,1.126,1.126,1.126,1.126
C    $         ,1.072,1.072,1.072,1.072,1.072,1.326,1.326,1.326,1.326,1.326
C    $         ,1.542,1.542,1.542,1.542,1.542,1.712,1.712,1.712,1.712,1.712
C    $         , 1.77, 1.77, 1.77, 1.77, 1.77, 1.76, 1.76, 1.76, 1.76, 1.76
C    $         , 1.76, 9*0.0/
C
      DATA HI   /6.35,0.952,2.66,0.952/
      DATA ALPH /0.437,-0.145,2.35,-0.145/
      DATA FI   /1.00,0.96,0.039,0.96/
      DATA YI   /0.,16.33,22.51,16.33/
      DATA XI   /1.,3.09,4.92,3.090/                              
      DATA TI   /0.0018,0.0003,0.0074,0.0003/
      DATA EPSU /0.4909,0.5658,2.417,0.5658/
      DATA EPSD /0.0,0.4638,3.502,0.4638/
C---------------------------------------------------------------------------
C
C Read in control info from standard input
C
      READ (5,1000)
      READ (5,1000) TITLE
      READ (5,1000) 
      READ (5,1000) 
      READ (5,1020) NLIST, NRTM, NREP, NXANDQ
C
C Control Flags:                     
C           NLIST  - write J-values to standard output
C           NRTM   - write J-values to unit 9 (file = FRTM)
C                    (if NRTM = 2, this is UAM-IV file)
C           NREP   - write J-values and ratios to JNO2 to unit 10  
C           NXANDQ - write cross-sections and quantum yields to unit 11
C
C Read name(s) of cross-section datafile and RTM output file
C
      READ (5,1000)       
      READ (5,1000) 
      READ (5,1000) FDATA
      READ (5,1000)       
      READ (5,1000) 
      READ (5,1000) FRTM 
C
C Open files
C
      OPEN (8,FILE=FDATA)
      IF (NRTM .GT. 0) OPEN (9,FILE=FRTM)
      IF (NREP .EQ. 1) OPEN (10,FILE='REPORT') 
      IF (NXANDQ .EQ. 1) OPEN (11,FILE='LISTXQ')
C
C
C Get cross-section and quantum yield data from a file
C
      CALL GETXQ
C
C Get O3COL/ALBEDO/HAZE values to be used
C
      READ (5,1000) 
      READ (5,1000)
      READ (5,*) (O3INP(I),I=1,5)
      READ (5,1000) 
      READ (5,1000)
      READ (5,*) (ALBINP(I),I=1,5)
      READ (5,1000) 
      READ (5,1000)
      READ (5,*) (HAZINP(I),I=1,3)
C
C****************4  Loop over O3COL/ALBEDO/HAZE values
C
C
      DO 100 JO3COL = 1, 5
      DO 101 JALBEDO = 1, 5
      DO 102 JHAZE = 1, 3
C
      O3COL = O3INP(JO3COL)
      ALBEDO = ALBINP(JALBEDO)
      HAZOD = HAZINP(JHAZE)
C
C   SKIP MISSING VALUES IN O3COL, ALBEDO, OR HAZE
C   (TCM, 2/22/95)
C
      IF (HAZOD .LE. -8.999) GO TO 102
      IF (ALBEDO .LE. -8.999) GO TO 101
      IF (O3COL .LE. -8.999) GO TO 100
C
      WRITE (6,2060)  O3COL, ALBEDO, HAZOD
      IF (NRTM .EQ. 2) WRITE (9,'(A)') 'PHOTOLYSIS'
      IF (NRTM .GT. 0) WRITE (9,2060)  O3COL, ALBEDO, HAZOD
      IF (NREP .EQ. 1) WRITE (10,2060) O3COL, ALBEDO, HAZOD
C
C Multiply standard aerosol optical depths by Haze factor 
C (Valid range for HAZFAC is 0 - 4.2)
C Std aerosol has mean scattering OD (Tau2) of 0.081 and mean absorption
C OD (Tau4) of 0.013, B&D model C1.  To get B&D model D1 you multiply 
C these by 4.2 (GCS p 61), also model is valid for no aerosols - hence
C range 0 - 4.2.  Model reads in a total OD due to haze, Tau2 + Tau4, 
C which equals 0.094 for the standard model.
C
      HAZFAC = HAZOD/0.094
      DO 110 I=1,28
         TAU2A(I) = TAU2(I) * HAZFAC
         TAU4A(I) = TAU4(I) * HAZFAC
 110  CONTINUE
C
C
C*****************3  Altitude Loop
C
C
      DO 120 JA=1,11
         ALT = ALTKM(JA)
         YI(1) = ALT
         DO 130 K=1,4
            NI(K) = FI(K)*(1.+ALPH(K))/(ALPH(K)+EXP(ALT/HI(K)))
     $              + (1.-FI(K))*(1.+EXP(-YI(K)/XI(K)))
     $              /(1.+EXP((ALT-YI(K))/XI(K)))
 130     CONTINUE
         ALP1DD = 1.735 + (-0.346)*NI(1)**5.
         ALP1D = ALP1DD
         NN1BAR = 7.3891/(6.3891+EXP(0.0857*ALT))
         N1BAR = 7.3891/(6.3891+EXP(0.1450*ALT))
C
C
C*****************2  Zenith Angle loop
C
C
      DO 140 JZ=1,10
         ZCOS = COS(ZA(JZ)*0.0174533)
         DO 150 K=1,4
            UI(K) = SQRT((ZCOS*ZCOS + TI(K))/(1.+TI(K)))
 150     CONTINUE
         DO 160 K=1,NUMSPE
            SUMJ(K,JZ) = 0.
 160     CONTINUE
C
C
C*****************1  Wavelength Loop
C
C
      DO 170 J=1,141 
         LAMDA(J) = 279. + FLOAT(J)
         LAM(J) = LAMDA(J)
         TAU(3) = KO3(J)*O3COL
         IF (LAM(J).GT.MPT(1)) GO TO 180
            TAU(1) = TAU1(1)
            TAU(2) = TAU2A(1)
            TAU(4) = TAU4A(1)
            GO TO 210
C
 180     IF (LAM(J).GT.MPT(28)) GO TO 200
         DO 190 K=2,28
            K1=K-1
            IF ((LAM(J).GT.MPT(K)).OR.(LAM(J).LT.MPT(K1)))  GO TO 190
            DIFF = (MPT(K)-LAM(J))/5.
            DELT1 = TAU1(K)-TAU1(K1)
            DELT2 = TAU2A(K)-TAU2A(K1)
            DELT4 = TAU4A(K)-TAU4A(K1)
            TAU(1) = TAU1(K) - DIFF*DELT1
            TAU(2) = TAU2A(K) - DIFF*DELT2
            TAU(4) = TAU4A(K) - DIFF*DELT4
            GO TO 210
C
 190     CONTINUE
         GO TO 210
C
 200     TAU(1) = TAU1(28)
         TAU(2) = TAU2A(28)
         TAU(4) = TAU4A(28)
C
 210     CONTINUE
C
C Calculation of some important parameters
C first calculate actinic flux (with tildes)
C
      F1U = 3.365*TAU(1)**1.735*(1.-NI(1))**0.544
      F2U = 6.182*TAU(1)**1.735*TAU(2)**1.12*(1.-NI(1))**0.544
     $      + 1.334*TAU(2)**1.12*(1.-NI(2))**0.564
      G3U = 1./(1.+1.662*O3COL**1.1181*KO3(J)**0.5505*NI(3))
      G4U = 1./(1.+6.2*TAU(4)**0.88*NI(2)**0.356)
      J2D = 2.254*TAU(2)**1.12
      F1D = 1.261*TAU(1)**ALP1DD*NN1BAR
      G3D = 1./(1.+0.2959*O3COL**1.223*KO3(J)**1.5650*NI(3)
     $      **(1.+1.7*TAU(3)))
      F2D = (F1D*G3D+NI(2)**0.564)*J2D/G3D
      G4D = 1./(1.+1.554*TAU(4)**0.88*NI(2)**0.49)
C
      MUU = (F1U+F2U)*G3U*G4U
      MDD = (F1D+F2D)*G3D*G4D
C
      PHIU = SQRT((1.+0.00933)/(ZCOS*ZCOS+0.00933))-1.
      PHID = PHIU
      BIGFU = 1./(1.+6.481*(TAU(3)+TAU(4))**1.952)
      BIGFD = 1./(1.+24.93*(TAU(3)+TAU(4))**1.0367)
C
      SCRLUU = (BIGFU+(1.-BIGFU)*EXP(-1.169*TAU(3)*PHIU))
     $         *EXP(-PHIU*(0.369*TAU(1)*NI(1)**0.0795
     $         +0.102*TAU(2)))
      SCRLDD = (BIGFD+(1.-BIGFD)*EXP(-1.169*TAU(3)*PHID*NI(3)**2.392))
     $         *EXP(-PHID*(0.400*TAU(1)*NI(1)**0.3475
     $         +0.4436*TAU(2)*NI(2)**0.3445))
C
C
C next calculate direct irradiance (no tildes)--
C
C     F1U = 1.103*TAU(1)**1.735*(1.-NI(1))**0.921
C     F2U = 2.027*TAU(1)**1.735*TAU(2)**1.12*(1.-NI(1))**0.921
C    $      + 0.427*TAU(2)**1.12*(1.-NI(2))**0.564
C     G3U = 1./(1.+0.198*O3COL**1.1181*KO3(J)**0.7555*NI(3))
C     G4U = 1./(1.+6.2*TAU(4)**0.88*NI(2)**0.356)
      J2D = 1.437*TAU(2)**1.12
C     F1D = 0.804*TAU(1)**ALP1D*N1BAR
C     G3D = 1./(1.+0.3264*O3COL**1.223*KO3(J)**0.7555*NI(3)
C    $      **(1.+1.7*TAU(3)))
C     F2D = (F1D*G3D+NI(2)**0.564)*J2D/G3D
C     G4D = 1./(1.+1.554*TAU(4)**0.88*NI(2)**0.49)
C
C     MU = (F1U+F2U)*G3U*G4U
C     MD = (F1D+F2D)*G3D*G4D
C
C     PHIU = SQRT((1.+0.01120)/(ZCOS*ZCOS+0.01120))-1.
      PHID = SQRT((1.+0.02160)/(ZCOS*ZCOS+0.02160))-1.
C     BIGFU = 1./(1.+28.80*(TAU(3)+TAU(4))**1.325)
      BIGFD = 1./(1.+84.37*(TAU(3)+TAU(4))**0.6776)
C
      SCRLU = (BIGFU+(1.-BIGFU)*EXP(-1.00*TAU(3)*PHIU))
     $        *EXP(-PHIU*(0.644*TAU(1)*NI(1)**0.0795
     $                     +0.102*TAU(2)))
      SCRLD = (BIGFD+(1.-BIGFD)*EXP(-1.00*TAU(3)*PHID*NI(3)**2.392))
     $        *EXP(-PHID*(0.535*TAU(1)*NI(1)**0.3475
     $        +0.6077*TAU(2)*NI(2)**0.3445))
C
      F1D0 = 0.804*TAU(1)**1.389
      G3D0 = 1./(1.+0.3264*O3COL**1.223*KO3(J)**0.7555)
      F2D0 = (F1D0*G3D0+1.)*J2D/G3D0
      G4D0 = 1./(1.+1.554*TAU(4)**0.88)
C
      MD0 = (F1D0+F2D0)*G3D0*G4D0
C
      SCRLD0 = (BIGFD+(1.-BIGFD)*EXP(-TAU(3)*PHID))
     $         *EXP(-PHID*(0.535*TAU(1)+0.6077*TAU(2)))
C
      LITR = ((0.4424*TAU(1)**0.5626/(1.+0.2797*O3COL**1.0132
     $       *KO3(J)**0.8404))+0.1*TAU(2)**0.88)
     $       /(1.+3.7*TAU(4))
C
      EUSUM = EPSU(1)*TAU(1)*(1.-NI(1))
      EDSUM = 0.
      DO 220 K=2,4
         EUSUM = EUSUM + EPSU(K)*TAU(K)*(1.-NI(K))
         EDSUM = EDSUM + EPSD(K)*TAU(K)*(1.-NI(K))
 220  CONTINUE
      EU = EXP(-EUSUM)
      ED = NI(1)*EXP(-EDSUM)
C
C Calculate extra-terrestrial flux and photon energy
C
      H(J) = ETR(J)
      SUMD = 0.
      SUM2 = 0.
      DO 230 K=1,4
         TEMP = TAU(K)*NI(K)/UI(K)
         TMP2 = TAU(K)/UI(K)
         SUMD = SUMD + TEMP
         SUM2 = SUM2 + TMP2
 230  CONTINUE
      EPHOT(J) = 1.986E-9/LAM(J)
C ergs
C
C
C Calculate some direct light values
C
      DD00 = H(J)*EXP(-1.*(TAU(1)+TAU(2)+TAU(3)+TAU(4)))
      D00  = ZCOS*DD00
      DDL0 = H(J)*EXP(-SUM2)
      DL0  = ZCOS*DDL0
      DSUM(J)=H(J)*EXP(-SUMD)
C
C Calculate scattered light
C
      S0DD = SCRLDD*MDD*DD00
      S0UU = SCRLUU*MUU*DD00
      SD00 = SCRLD0*MD0*D00
C
      GDL0 = DL0 + SD00
C
      SAUU = 2.*ALBEDO*GDL0*EU/(1.-LITR*ALBEDO)
      SADD = 2.*ALBEDO*LITR*GDL0*ED/(1.-LITR*ALBEDO)
C
      SDD = S0DD+SADD
      SUU = S0UU+SAUU
C
      GDIFF(J) = SDD+SUU
C
      G(J) = DSUM(J) + GDIFF(J)
C
      PFLUX(J) = G(J)*1.E+3/EPHOT(J)
      APF(J,JZ) = PFLUX(J)
C
C
C********************1  END Wavelength Loop
C
 170  CONTINUE
C
C
C Calculate the rate of photolysis for each species at the current
C ozone column, altitude, and zenith angle using the wavelength
C resolved actinic fluxes from above.
C
      DO 240 J=1,NUMSPE
         L1=LAMLOW(J)
         L2=LAMHI(J)
         DO 250 II=L1,L2
            SUMJ(J,JZ)=SUMJ(J,JZ)+APF(II,JZ)*PHI(J,II)*SIG(J,II)*60.
 250     CONTINUE
 240  CONTINUE
C
C
C********************2  END Zenith Angle Loop
C
  140 CONTINUE
C
C If listing requested, output rates 
C
      IF (NLIST .EQ. 1) THEN
         WRITE (6,2030) ALT
         WRITE (6,2031) (Z(I),I=1,10)
         DO 260 I=1,NUMSPE
            WRITE (6,2032) (SPNAME(K,I),K=1,2),(SUMJ(I,KK),KK=1,10)
 260     CONTINUE
      ENDIF
C
C If report requested, output rates and ratios
C
      IF (NREP .EQ. 1) THEN
         WRITE (10,2030) ALT
         WRITE (10,2031) (Z(I),I=1,10)
         DO 270 I=1,NUMSPE
            WRITE (10,2032) (SPNAME(K,I),K=1,2),(SUMJ(I,KK),KK=1,10)
 270     CONTINUE
         WRITE (10,2033)
         DO 280 I=2,NUMSPE
            DO 290 K=1,10
               TJ(K) = SUMJ(I,K)/SUMJ(1,K)
 290        CONTINUE
            WRITE (10,2032) (SPNAME(K,I),K=1,2),(TJ(KK),KK=1,10)
 280     CONTINUE
      ENDIF
C
C If RTM input file requested
C
      IF (NRTM .GT. 0) THEN
         WRITE (9,2070) ALT
         DO 300 I=1,NUMSPE
            WRITE (9,2080) (SUMJ(I,KK),KK=1,10)
 300     CONTINUE
      ENDIF
C
C
C********************3  END Altitude loop
C
 120  CONTINUE
C
C********************4  END loop over O3/Albedo/Haze
C
 102  CONTINUE
 101  CONTINUE
 100  CONTINUE
C
C     Write out Actinic Flux used
C     WRITE (6,2040) O3COL
C     DO 310 I = 1,141
C310    WRITE (6,2045) LAM(I), (APF(I,JZ),JZ=1,10)
C
 1000 FORMAT (A80)
 1005 FORMAT (I10)
 1010 FORMAT (3(F10.0))
 1020 FORMAT (4(I10))
 2030 FORMAT (/,'1Photolysis rates at ',F5.3,' km',/)
 2031 FORMAT ('    NAME    ',5X,I3,4X,9(4X,I4,4X),/)
 2032 FORMAT (' ',2A4,3X,1PE11.3,9(1X,E11.3))
 2033 FORMAT (/,' Ratios of the above rates to NO2 photolysis rate',/)
C2040 FORMAT (' ',F8.4,16H     280     420)
C2045 FORMAT (' ',F4.0,1P10E11.3)
 2060 FORMAT (' O3 Column =',F5.3,'  Albedo =',F5.3,
     $        '  Haze O.D. =',F5.3)
 2070 FORMAT (' ',F6.3,' = Current altitude in km')
 2080 FORMAT (' ',1PE11.3,9(1X,E11.3))
C
      STOP
      END
C
C******************************************************************************
C
      SUBROUTINE GETXQ
C
C     Read in cross-section and quantum yield data
C
C******************************************************************************
C
      COMMON /CNTROL/ NXANDQ, NUMSPE        
      COMMON /VALUES/ SIG(20,150),PHI(20,150),LAMLOW(10),LAMHI(10),
     $                SPNAME(2,10)
C
      CHARACTER*80 COM
      CHARACTER*4 SPNAME
C
C INPUT OF VARIOUS PARAMETERS USED IN CALCULATING J VALUES
C ALL CROSS-SECTIONS MUST BE MULTIPLIED BY 10**20 BEFORE INPUT
C
      READ(8,1000) NUMSPE
      IF (NUMSPE.GT.10) GO TO 9999
      DO 400 I=1,NUMSPE
         READ(8,1010) SPNAME(1,I), SPNAME(2,I)
         DO 410 J=1,3
            READ(8,1100) COM
 410     CONTINUE
         READ(8,1020) LAMLOW(I), LAMHI(I)
         LAMLOW(I) =LAMLOW(I) -279
         LAMHI(I) =LAMHI(I) -279
         L1 = LAMLOW(I)
         L2 = LAMHI(I)
         READ(8,*) (SIG(I,J),J=L1,L2)
         READ(8,*) (PHI(I,J),J=L1,L2)
         DO 415 J=L1,L2
            SIG(I,J)=SIG(I,J)*1.E-20
  415    CONTINUE
         WRITE (6,11) (SPNAME(J,I),J=1,2) 
  400 CONTINUE
C
        IF (NXANDQ.EQ.0) GO TO 600
        WRITE (11,10)
        DO 500 J=1, NUMSPE
           WRITE (11,11) (SPNAME(I,J),I=1,2)
           WRITE (11,12)
           DO 550 K = LAMLOW(J), LAMHI(J)
              WRITE (11,13) K+279, SIG(J,K), PHI(J,K)
  550      CONTINUE
  500   CONTINUE 
C
  600 RETURN
C
 9999 WRITE (6,3000)
      STOP
C
C
   10 FORMAT (' CROSS-SECTIONS AND QUANTUM YIELDS USED')
   11 FORMAT (2X,A4,A4) 
   12 FORMAT (2X,'Lamda',8X,'X-Sec',6X,'Q-Yld')
   13 FORMAT (3X,I4,4X,1PE12.4,2X,0PF6.3,4X)
 1000 FORMAT (I5)
 1010 FORMAT (2A4)
 1020 FORMAT (2I5)
 1100 FORMAT (A80)
 3000 FORMAT ('0=====ERROR====> X AND Q DATA FILE MUST HAVE 7 SPECIES!')
C
      END
