      subroutine aerody(lu,ustar,elinv,z00,ra)
C
C     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
C
c
c$Log: aerody.f,v $
c  Revision 1.2  1994/02/10  20:50:12  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c
c
C
C**********************************************************************
C
C-----This program estimates the aerodynamic resistance over solid
C-----surfaces using the Weseley and Hicks (1977) and Sheih (1979)
C-----relations for the similarity layer relations.
C
C-----The input to this routine are the following:
C-----ustar = the friction velocity
C-----elinv = the inverse Monin - Obukhov length
C-----z00 = the surface roughness length
C
C-----The output from this routine is the following:
C-----ra = bulk aerodynamic resistance
C
C-----This program was written by GEM May 1990
C
C**********************************************************************
C
      include 'parm.cmd'
      real ra(mlus)
      data zr,vk/10., 0.40/
      data a1,a2,a3/0.598,0.390,0.090/
C
C-----Now estimate the phi for a scalar
C
      if(elinv.lt.0.0)then
        b3 = - a3*(alog(-zr*elinv)**2)
        b2 = a2*alog(-zr*elinv)
        phi = exp(a1 + b2 + b3)
      endif
      if(elinv.eq.0.0) phi = 0.0
      if(elinv.gt.0.0)then
        phi = -4.7*zr*elinv
      endif
C
C-----Now compute the aerodynamic resistance
C
      c1 = alog(zr/z00)
      c2 = 1.0/(vk*ustar)
c
c...To prevent ra from going negative during unstable conditions
c
      if (phi.gt.0.99*c1) phi = 0.99*c1
c
      ra(lu) = c2*(c1 - phi)
      return
      end
