      subroutine clvdff(nxf,nyf,nzf,nospec,delfx,delfy,
     &                  cncf,fkv,htf,depvelf,deltf,ifine)
c
c
c$Log: clvdff_o.f,v $
c  Revision 1.3  1999/07/07  16:09:35  tcm
c  Added tracking of fluxes for process analysis.  Some restructuring
c  of code was necessary to accomplish this.
c
c  Revision 1.2  1994/02/10  20:54:06  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c     this subroutine prepares variables for vertical diffusion 
c     calculations on a fine grid
c
c     pertinent information used are:
c     cncf( , , , , ) - concentration array (ppm) 
c     htf( , , )        - array of heights (km agl)
c     fkv( , , )       - array of vertical diffusivities (cm**2/s)
c     depvelf( , , )   - array of deposition velocities (m/s)
c     deltf          - time step (min)
c
c....modified 7/22/91 to fix time step calculation-- rrs
c
c....modified 2/11/92, tcm:  added calculation of max time step
c    for deposition calculation
c
c....modified 4/3/92, tcm:  make inner loop the loop over x
c    in order to lengthen vectorizable loops.  move vrtdif code
c    into this routine.  restructure ifs to eliminate branches
c    within inner loops.
c
c....modified 4/1/93, cae: added minimum kv = 1. in denominator
c    of time step calculation so we don't get divide by 0 when kv = 0.
c
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'balanc.cmd'
c
      dimension cflux(mxfx,mxfz),dz(mxfx,mxfz),hh(mxfx,mxfz),cckv(mxfz)
      dimension cncf(nxf,nyf,nospec,nzf),depvelf(nxf,nyf,nospec)
      dimension fkv(nxf,nyf,nzf), htf(nxf,nyf,nzf)
      dimension dzpr(mxfx,mxfz), ctm(mxfx,mxfz), dep(mxfx)
      dimension delfx(nyf)
c
czswa---------------------------------------------------------------------
c  add flux1 and flux2 to tally the fluxes on the boundaries
c
      include 'irmb.cmd'
      dimension flux1(mxfx,mxfz), flux2(mxfx,mxfz)
c
czswz---------------------------------------------------------------------
c
c     fact1 - converts from km**2 to cm **2
c     fact2 - converts from sec to min
c
      data fact1/1.e10/, fact2/0.0166667/
c
c     loop on horizontal grid
c
ccc   write(*,*) ' entering clvdff cncf is:'
ccc   write(*,*) (cncf(2,2,l,1),l=1,nospec)
      do 500 j=2,nyf-1
c
c     see if in our modeling domain
c
        i1=2
        i2=nxf-1
        dtv=1.e20
cdir$ shortloop
        do 110 i=i1,i2
c
c     calculate time step for vertical diffusion
c     based on stability criteria
c
          dzpr(i,1) = htf(i,j,1)
          hh(i,1)=htf(i,j,1)/2.
  110   continue
        do 120 k=2,nzf
cdir$ shortloop
          do 115 i=i1,i2
            hh(i,k)=htf(i,j,k-1)+0.5*(htf(i,j,k)-htf(i,j,k-1))
            dz(i,k-1)=hh(i,k)-hh(i,k-1)
            dzpr(i,k) = htf(i,j,k) - htf(i,j,k-1)
  115     continue
  120   continue
c
        fftmp = 0.15*fact1*fact2
cdir$ shortloop
        do 125 i=i1,i2
          dttstu=fftmp*dz(i,1)*dzpr(i,1)/(fkv(i,j,1)+1.)
          dtv=amin1(dtv,dttstu)
  125   continue
        do 140 k=3,nzf-1
cdir$ shortloop
          do 130 i=i1,i2
            dttstu=fftmp*dz(i,k-1)*dzpr(i,k-1)/(fkv(i,j,k-1)+1.)
            dttstl=fftmp*dz(i,k-2)*dzpr(i,k-1)/(fkv(i,j,k-2)+1.)
            dtv=amin1(dtv,dttstl,dttstu)
  130     continue
  140   continue
cdir$ shortloop
        do 150 i=i1,i2
          dttstl=fftmp*dz(i,nzf-1)*dzpr(i,nzf)/(fkv(i,j,nzf-1)+1.)
          dtv = amin1(dtv,dttstl)
  150   continue
c
c     calculate layer interface diffusive flux coefficient
c
        do 330 k=1,nzf-1
cdir$ shortloop
          do 320 i=i1,i2
            cflux(i,k)=fkv(i,j,k)/(dz(i,k)*fact1*fact2)
  320     continue
  330   continue
c
c     loop on species
c
        do 460 l=1,nospec
c
czswa---------------------------------------------------------------------
c  initialize the fluxes to zero
c
      do k = 1, mxfz
      do i = 1, mxfx
         flux1(i,k) = 0.0
         flux2(i,k) = 0.0
      enddo
      enddo
c
czswz---------------------------------------------------------------------
c
c     load deposition rate 
c     calculate step size for deposition (make it one quarter
c     of max possible)
c
          dtnew = dtv
cdir$ shortloop
          do 340 i=i1,i2
            dep(i) = 0.06*depvelf(i,j,l)
            deldep = -dzpr(i,1)/amin1(dep(i),-1.e-6) * .25
            dtnew = amin1(deldep,dtnew)
  340     continue
          nstep=int(deltf/dtnew)+1
          deltdf=deltf/float(nstep)
c
czswa--------------------------------------------------------------------
c  This part of the code is rewritten to incorporate ipram
c
c     vertdif starts here
c
          do 420 istep=1,nstep
c
            do 350 i=i1,i2
              delc1 = deltdf*dep(i)*cncf(i,j,l,1)/dzpr(i,1)
c          dpflow in moles, delc1 in micro-moles/m**3;
c          deltax, deltay, and h in km
              fdpflo(l,ifine) = fdpflo(l,ifine) + delc1*delfy*delfx(j)
     &               *htf(i,j,1)*1000.
              v1 = deltdf*(dep(i)*cncf(i,j,l,1))/dzpr(i,1)
              v2 = deltdf*cflux(i,1)*
     &                (cncf(i,j,l,2)-cncf(i,j,l,1))/dzpr(i,1)
              ctm(i,1) = cncf(i,j,l,1) + v1 + v2
c
              flux1(i,1) = flux1(i,1) + v1
              flux2(i,1) = flux2(i,1) + v2
c
  350       continue
c
c  continue from level 2 to layer-next-to-the-top
c
            do 370 k=2,nzf-1
c
              do 360 i=i1,i2
                v1 = deltdf*cflux(i,k-1)*
     &                 (cncf(i,j,l,k)-cncf(i,j,l,k-1))/dzpr(i,k)
                v2 = deltdf*cflux(i,k)*
     &                 (cncf(i,j,l,k+1)-cncf(i,j,l,k))/dzpr(i,k)
                ctm(i,k) = cncf(i,j,l,k) + v2 - v1
c
              flux1(i,k) = flux1(i,k) - v1
              flux2(i,k) = flux2(i,k) + v2
c
  360         continue
  370       continue
c
            do 380 i=i1,i2
              v1 = deltdf*cflux(i,nzf-1)*
     &                (cncf(i,j,l,nzf)-cncf(i,j,l,nzf-1))/dzpr(i,nzf)
              v2 = 0.0
              ctm(i,nzf) = cncf(i,j,l,nzf) + v2 - v1
c
              flux1(i,nzf) = flux1(i,nzf) - v1
              flux2(i,nzf) = flux2(i,nzf) + v2
c
  380       continue
c
            do 400 k=1,nzf
              do 390 i=i1,i2
                cncf(i,j,l,k) = ctm(i,k)
  390         continue
  400       continue
c
c  go to the next time step
c
  420     continue
c
c         end of vrtdif
c
c  tally the fluxes at the top and bottom boundaries for ipram
c
            do 402 k=1,nzf
              do 401 i=i1,i2
                iflgf = iflagf(i,j,ifine)
                if (iflgf.gt.0) then
                   btfluxf(1,iflgf,l,k) = flux1(i,k) +
     &                btfluxf(1,iflgf,l,k)
                   btfluxf(2,iflgf,l,k) = flux2(i,k) +
     &                btfluxf(2,iflgf,l,k)
                endif
  401         continue
  402       continue
c
czswa--------------------------------------------------------------------
c
  460   continue
  500 continue
c
      return
      end 
