      subroutine euler(ncells,nutrc,dt,dt0,ipff,iflag)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: euler.f,v $
c  Revision 1.6  1998/11/20  22:09:00  tcm
c  Fixed error message on line 68 and format 41.
c
c  Revision 1.5  1998/11/19  00:09:09  ara
c  Eliminated error message directed to diag file (8)
c  as it was already directed by variable nutrc
c
c  Revision 1.4  1998/11/09  18:18:35  ara
c  *** empty log message ***
c
c  Revision 1.3  1998/11/09  18:17:15  ara
c  Included errlog.cmd file to introduce ierrlg variable
c  Directed error messages to screen and to log file (ierrlg)
c
c  Revision 1.2  1994/02/10  20:55:14  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c
c
c
c     save
c
c     this subroutine advances the plume cross section one time step
c     downwind. if the time step (delt) is too large, it is halved until
c     it satisfies the time step criterion that none of the cell
c     boundaries cross each other.
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comchem.cmd'
      include 'pigpen.cmd'
      include 'rpm.cmd'
      include 'errlog.cmd'
      dimension ptemp(mxskn,mxpspc)
      dimension rbyold(mxskn), rbzold(mxskn)
      data hmin/1.e-10/
      data pi /3.1415927/
c
c     check if expansion will cause cell boundaries to
c     cross each other. if so halve time step and return to 10.
c
      tnew = t+dt
      if (iflag.eq.1) goto 50
      do 5 nn = 1,ncells
        rbyold(nn) = rfacb(nn+1)*sigy
        rbzold(nn) = rfacb(nn+1)*sigz
    5 continue
c
   10 synew = (sigmy-osigmy)*(tnew/dt0)+osigmy
      sznew = (sigmz-osigmz)*(tnew/dt0)+osigmz
      do 15 nn=2,ncells
        rbynew = rfacb(nn)*synew
        rbznew = rfacb(nn)*sznew
        if(rbynew.gt.rbyold(nn) .or. rbznew.gt.rbzold(nn)) go to 20
   15 continue
      go to 50
c
c     test fails halve time step
c
   20 iflag = 1
      dt = dt*0.5
      tnew = t+dt
cae   write(nutrc,30) dt,t
      if (dt .gt. hmin) go to 10
      write(nutrc,40) t,dt,rbyold,rbzold,rbynew,rbznew
      write(*,40) t,dt,rbyold,rbzold,rbynew,rbznew
      write(ierrlg,41) 1059,t,dt,rbyold,rbzold,rbynew,rbznew
      write(nutrc,45) ipff, tnew, dt0, ncells, rfacb(ncells+1),sigy,
     &           rfacb(ncells),sigz, synew,sigmy,osigmy,
     &           sznew,sigmz,osigmz
c
c     time step too small -- stop
c
      stop
   30 format('BOUNDARY MOVEMENT TOO GREAT FOR SELECTED TIME STEP'
     &,' ..... TIME STEP REDUCED TO  ',1pe10.3,' AT TIME  ',1pe10.3)
   40 format('** ERROR IN EULER: DT TOO SMALL T= ',1pe10.3,' DT= '
     &,1pe10.3,' rbyold= ',1pe10.3,' rbzold= ',1pe10.3,' rbynew= ',
     & 1pe10.3,' rbznew= ',1pe10.3)
   41 format(i4,1x,'** ERROR IN EULER: DT TOO SMALL T= ',1pe10.3,' DT= '
     &,1pe10.3,' rbyold= ',1pe10.3,' rbzold= ',1pe10.3,' rbynew= ',
     & 1pe10.3,' rbznew= ',1pe10.3)
   45 format( ' ipff, tnew, dt0, ncells: ', i5, 2f10.4, i5, /,
     &        ' rfacb(ncells+1), sigy: ', 1pe10.4,1x,e10.4,/,
     &        ' rfacb(ncells), sigz: ', 1pe10.4,1x,e10.4,/,
     &        ' synew, sigmy, osigmy: ',e10.4,1x,e10.4,1x,e10.4,/,
     &        ' sznew, sigmz, osigmz: ',e10.4,1x,e10.4,1x,e10.4,/)
c
c-----------------------------------------------------------------------
c
c     accepted time step dt
c     now we split time step into 50 intervals for better approximation
c
   50 continue
c      dtt = dt
c      do 250 it=1,1 
      dtt = dt/50.
      do 250 it=1,50
c
c     calculate new plume variables
c
        t = t+dtt
        tback = t-dtt
        synew = (sigmy-osigmy)*(t/dt0)+osigmy
        sznew = (sigmz-osigmz)*(t/dt0)+osigmz
        sysz = sigy*sigz
        sysznew = synew*sznew
c
c     obtain proper ambient concentrations
c
        call ambien(tback,dt0,ipff)
c
c     calculate change in concentrations due to expansion of cell 
c     (dilution), entrainment of material to/from neighboring cells,and 
c     diffusion across new cell boundaries (based on diffusivity 
c     of tracer species)
c
        do 210 nc=1,ncells
          fact = (sysznew - sysz)/
     &             (sysznew*(rfacb(nc+1)**2-rfacb(nc)**2))
          fact1 = fact*rfacb(nc+1)**2
          fact2 = fact*rfacb(nc)**2
          do 200 m=2,nspec
            dilute = pconc(nc,m)*sysz/sysznew
            detrain = -pconc(nc,m)*fact2
            if (nc.eq.1) then
              diffus2 = 0.
            else 
              diffus2 = pconc(nc,1)*(pconc(nc,m)-pconc(nc-1,m))/
     &                  (pconc(nc,1)-pconc(nc-1,1))*fact2
            endif
            if (nc.eq.ncells) then
              entrain = yaml(m)*fact1
              diffus1 = 0.
            else
              entrain = pconc(nc+1,m)*fact1
              diffus1 = -pconc(nc+1,1)*(pconc(nc+1,m)-pconc(nc,m))/
     &                  (pconc(nc+1,1)-pconc(nc,1))*fact1
            endif
            ptemp(nc,m) = dilute+entrain+detrain+diffus1+diffus2
  200     continue
c
c     dilute tracer species
c
          ptemp(nc,1) = pconc(nc,1)*sysz/sysznew
  210   continue
c
c     check lower bound to pconc array
c
        do 240 nc=1,ncells
          do 230 m=1,nspec
            pconc(nc,m) = ptemp(nc,m)
csbs        if (m.gt.1.and.pconc(nc,m).lt.bdnl(m)) then
csbs          pconc(nc,m+1) = bdnl(m)
            if (m.gt.1) then
              if (pconc(nc,m).lt.bdmol(m-1)) pconc(nc,m)=bdmol(m-1)
            endif
  230     continue
  240   continue
c
c     update dimensions
c
        sigy = synew
        sigz = sznew
c
  250 continue
c
c     now obtain new plume averages
c
      call devcon(nspec,1,ncells)
c
      return
      end
