      subroutine fbdfil(ifget,nxfg,nyfg,nzfg,nspecf,htg,cf1)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: fbdfil.f,v $
c  Revision 1.4  1996/06/14  22:58:31  tcm
c  corrected height calculation for vertical nesting
c
c  Revision 1.3  1995/04/26  00:38:22  tcm
c  	extensively rewritten to allow more complex horizontal and vertical nesting.  uses igloc,
c  	jhloc, and fhloc to determine adjacent grids and interpolation factors.
c
c  Revision 1.2  94/02/10  20:55:28  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c
c   fills in boundary concentrations for fine
c   grids from another fine grid. (tcm, 2/13/91)
c
c   Variables used in setting bcs:
c
c     igloc - fine grid number from which bcs are to be obtained,
c             has a value for each edge, and each cell on that 
c             edge, and each fine grid.
c
c     fhloc - a floating point value specifying the location in
c             the grid identified by igloc of the left (or bottom,
c             if this is east of west boundary) edge of the cell
c             in the current grid for which the bc is to be set.
c             Dimensioned the same as igloc.  This value is in 
c             grid units in the grid from which concs are being
c             obtained.  If a north or south boundary is being
c             set, this is an x location.  If an east or west
c             boundary is being set, this is a y location.
c
c     jhloc - The cell number in the grid from which concentrations
c             are being obtained.  If a north or south boundary
c             is being set, this is a row number.  If an east or
c             west boundary is being set, this is a column number.
c             dimensioned the same as igloc.
c
c     wkg   - Weighting factor for each layer to account for 
c             vertical nesting.
c 
c   igloc, fhloc, and jhloc are set in rvmmap.  wkg is calculated
c   within this routine.
c   See detailed explanation of the use of these variables below
c   for north boundary.  Use is similar for other boundaries.
c
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'shrfin.cmd'
      include 'deffin.cmd'
      include 'com1f.cmd'
      include 'ngmap.cmd'
      dimension cf1(nxfg,nyfg,nspecf,nzfg)
      dimension htg(nxfg,nyfg,nzfg)
      dimension wkg(mxfz), htset(mxfz)
c
c  loop over all fine grids, skip the one from which we are getting
c  the concentrations to set boundaries on others
c
      do 400 ifset=1,numfin
        if (ifset .eq. ifget) go to 400
c
c  fill in north boundary
c
c  we are currently setting bcs for ifset from the concentrations
c  in ifget.  Only those cells that need bcs from ifget will be
c  set on this call.
c
        iedg = 1
        do 50 i=2,nfx(ifset)-1
c
c       ---if this boundary cell does not need concs from
c          ifget, skip setting bc for this cell.
c
          if (igloc(i,iedg,ifset) .ne. ifget) go to 50
c
c       ---hrat is the size of an ifset cell in units of ifget cells
          hrat = float(nhf(ifset))/float(nhf(ifget))
c
c       ---weight is used to do weighted average of cells in ifget
c          to get bc for ifset.  If weight is 1., then bc is set 
c          from one cell only.
          weight = amin1(1.,(1.-amod(fhloc(i,iedg,ifset),1.))*hrat)
c
c       ---ic, jc give cell location in ifget
          ic = int(fhloc(i,iedg,ifset)+.0005)+1
          jc = jhloc(i,iedg,ifset)
c
c       ---get column of layer heights for this edge cell of the
c          grid to be set
          call arrget(nfx(ifset),nfy(ifset),nfz(ifset),i,nfy(ifset),
     &                htf(1,1,ifset),htset)
c
          do 40 n=1,nspecf
            kget = 1
c
c         ---if there are more layers in the grid being set
c            than in the grid from which we get concs, then
c            we just use same conc in each layer contained 
c            within the larger layer.
            if (nfz(ifset) .ge. nzfg) then
              do 30 k=1,nfz(ifset)
c
                bcf(i,iedg,n,k,ifset) = cf1(ic,jc,n,kget)*weight
                carry = 1. - weight
                icp = ic
c
c             ---if the cell being set overlaps another cell of
c                the grid from which we get concs, move on to 
c                get the contribution of the next cell. ...........96waerme.4
   25           icp = icp + 1
                if (carry .gt. 0.) then
                  w2 = amin1(hrat,carry)
                  bcf(i,iedg,n,k,ifset) = bcf(i,iedg,n,k,ifset) 
     &                                  + cf1(icp,jc,n,kget)*w2
                  carry = carry - w2
                  go to 25
                endif
c
c             ---if height for grid to be set has reached the
c                height of the layer we are setting from , move
c                on to next layer
c
                if (htset(k) .ge. 
     &              htg(ic,jc,kget)-.0005) kget = kget + 1
c
   30         continue
c
            else
c
c           ---in this case, the grid we are setting bcs from has
c              more layers than the grid being set, so weighting
c              factors are calculated for each layer.  These compound
c              with the horizontal weighting.
              nkget = 0
              kg0 = 1
              k = 1
              hgm1 = 0.
              hsm1 = 0.
              bcf(i,iedg,n,k,ifset) = 0.
   35         nkget = nkget + 1
              wkg(nkget) = (htg(ic,jc,nkget)-hgm1)/
     &                     (htset(k)-hsm1)
              hgm1 = htg(ic,jc,nkget)
              if (htg(ic,jc,nkget)+.0005 .lt. htset(k)) go to 35
c
              do 36 kget=kg0,nkget
                bcf(i,iedg,n,k,ifset) = bcf(i,iedg,n,k,ifset) 
     &                             + cf1(ic,jc,n,kget)*weight*wkg(kget)
   36         continue
              carry = 1. - weight
              icp = ic
   37         icp = icp + 1
c
c           ---reset layer weighting, since we have moved to new
c              horizontal location and heights could be different
              if (kg0 .gt. 1) then
                hgm1 = htg(icp,jc,kg0-1)
              else
                hgm1 = 0.
              endif
              do 38 kget=kg0,nkget
                wkg(kget) = (htg(icp,jc,kget)-hgm1)/
     &                      (htset(k)-hsm1)
                hgm1 = htg(ic,jc,kget)
   38         continue
c
              if (carry .gt. 0.) then
                w2 = amin1(hrat,carry)
                do 39 kget=kg0,nkget
                  bcf(i,iedg,n,k,ifset) = bcf(i,iedg,n,k,ifset) 
     &                               + cf1(icp,jc,n,kget)*w2*wkg(kget)
   39           continue
                carry = carry - w2
                go to 37
              endif
c
c           ---bump things up for the next layer and go back
c              to get bc for that next layer.
              hgm1 = htg(ic,jc,nkget)
              hsm1 = htset(k)
              k = k + 1
              if (k .le. nfz(ifset)) then
                kg0 = nkget + 1
                bcf(i,iedg,n,k,ifset) = 0.
                go to 35
              endif
c
            endif
   40     continue
   50   continue
c
c  fill in south boundary
c
        iedg = 2
        do 150 i=2,nfx(ifset)-1
          if (igloc(i,iedg,ifset) .ne. ifget) go to 150
          hrat = float(nhf(ifset))/float(nhf(ifget))
          weight = amin1(1.,(1.-amod(fhloc(i,iedg,ifset),1.))*hrat)
          ic = int(fhloc(i,iedg,ifset)+.0005)+1
          jc = jhloc(i,iedg,ifset)
          call arrget(nfx(ifset),nfy(ifset),nfz(ifset),i,1,
     &                htf(1,1,ifset),htset)
          do 140 n=1,nspecf
            kget = 1
            if (nfz(ifset) .ge. nzfg) then
              do 130 k=1,nfz(ifset)
                bcf(i,iedg,n,k,ifset) = cf1(ic,jc,n,kget)*weight
                carry = 1. - weight
                icp = ic
  125           icp = icp + 1
                if (carry .gt. 0.) then
                  w2 = amin1(hrat,carry)
                  bcf(i,iedg,n,k,ifset) = bcf(i,iedg,n,k,ifset) 
     &                                    + cf1(icp,jc,n,kget)*w2
                  carry = carry - w2
                  go to 125
                endif
                if (htset(k) .ge. 
     &              htg(ic,jc,kget)-.0005) kget = kget + 1
  130         continue
c
            else
              nkget = 0
              kg0 = 1
              k = 1
              hgm1 = 0.
              hsm1 = 0.
              bcf(i,iedg,n,k,ifset) = 0.
  135         nkget = nkget + 1
              wkg(nkget) = (htg(ic,jc,nkget)-hgm1)/
     &                     (htset(k)-hsm1)
              hgm1 = htg(ic,jc,nkget)
              if (htg(ic,jc,nkget)+.0005 .lt. htset(k)) go to 135
c
              do 136 kget=kg0,nkget
                bcf(i,iedg,n,k,ifset) = bcf(i,iedg,n,k,ifset) 
     &                             + cf1(ic,jc,n,kget)*weight*wkg(kget)
  136         continue
              carry = 1. - weight
              icp = ic
  137         icp = icp + 1
              if (kg0 .gt. 1) then
                hgm1 = htg(icp,jc,kg0-1)
              else
                hgm1 = 0.
              endif
              do 138 kget=kg0,nkget
                wkg(kget) = (htg(icp,jc,kget)-hgm1)/
     &                      (htset(k)-hsm1)
                hgm1 = htg(ic,jc,kget)
  138         continue
c
              if (carry .gt. 0.) then
                w2 = amin1(hrat,carry)
                do 139 kget=kg0,nkget
                  bcf(i,iedg,n,k,ifset) = bcf(i,iedg,n,k,ifset) 
     &                                + cf1(icp,jc,n,kget)*w2*wkg(kget)
  139           continue
                carry = carry - w2
                go to 137
              endif
              hgm1 = htg(ic,jc,nkget)
              hsm1 = htset(k)
              k = k + 1
              if (k .le. nfz(ifset)) then
                kg0 = nkget + 1
                bcf(i,iedg,n,k,ifset) = 0.
                go to 135
              endif
c
            endif
  140     continue
  150   continue
c
c  fill in east boundary
c
        iedg = 3
        do 250 j=2,nfy(ifset)-1
          if (igloc(j,iedg,ifset) .ne. ifget) go to 250
          hrat = float(nhf(ifset))/float(nhf(ifget))
          weight = amin1(1.,(1.-amod(fhloc(j,iedg,ifset),1.))*hrat)
          ic = jhloc(j,iedg,ifset)
          jc = int(fhloc(j,iedg,ifset)+.0005)+1
          call arrget(nfx(ifset),nfy(ifset),nfz(ifset),nfx(ifset),j,
     &                htf(1,1,ifset),htset)
          do 240 n=1,nspecf
            kget = 1
            if (nfz(ifset) .ge. nzfg) then
              do 230 k=1,nfz(ifset)
                bcf(j,iedg,n,k,ifset) = cf1(ic,jc,n,kget)*weight
                carry = 1. - weight
                jcp = jc
  225           jcp = jcp + 1
                if (carry .gt. 0.) then
                  w2 = amin1(hrat,carry)
                  bcf(j,iedg,n,k,ifset) = bcf(j,iedg,n,k,ifset) 
     &                                    + cf1(ic,jcp,n,kget)*w2
                  carry = carry - w2
                  go to 225
                endif
                if (htset(k) .ge. 
     &              htg(ic,jc,kget)-.0005) kget = kget + 1
  230         continue
c
            else
              nkget = 0
              kg0 = 1
              k = 1
              hgm1 = 0.
              hsm1 = 0.
              bcf(j,iedg,n,k,ifset) = 0.
  235         nkget = nkget + 1
              wkg(nkget) = (htg(ic,jc,nkget)-hgm1)/
     &                     (htset(k)-hsm1)
              hgm1 = htg(ic,jc,nkget)
              if (htg(ic,jc,nkget)+.0005 .lt. htset(k)) go to 235
c
              do 236 kget=kg0,nkget
                bcf(j,iedg,n,k,ifset) = bcf(j,iedg,n,k,ifset) 
     &                             + cf1(ic,jc,n,kget)*weight*wkg(kget)
  236         continue
              carry = 1. - weight
              jcp = jc
  237         jcp = jcp + 1
              if (kg0 .gt. 1) then
                hgm1 = htg(ic,jcp,kg0-1)
              else
                hgm1 = 0.
              endif
              do 238 kget=kg0,nkget
                wkg(kget) = (htg(ic,jcp,kget)-hgm1)/
     &                      (htset(k)-hsm1)
                hgm1 = htg(ic,jcp,kget)
  238         continue
c
              if (carry .gt. 0.) then
                w2 = amin1(hrat,carry)
                do 239 kget=kg0,nkget
                  bcf(j,iedg,n,k,ifset) = bcf(j,iedg,n,k,ifset) 
     &                                + cf1(ic,jcp,n,kget)*w2*wkg(kget)
  239           continue
                carry = carry - w2
                go to 237
              endif
              hgm1 = htg(ic,jc,nkget)
              hsm1 = htset(k)
              k = k + 1
              if (k .le. nfz(ifset)) then
                kg0 = nkget + 1
                bcf(j,iedg,n,k,ifset) = 0.
                go to 235
              endif
c
            endif
  240     continue
  250   continue
c
c  fill in west boundary
c
        iedg = 4
        do 350 j=2,nfy(ifset)-1
          if (igloc(j,iedg,ifset) .ne. ifget) go to 350
          hrat = float(nhf(ifset))/float(nhf(ifget))
          weight = amin1(1.,(1.-amod(fhloc(j,iedg,ifset),1.))*hrat)
          ic = jhloc(j,iedg,ifset)
          jc = int(fhloc(j,iedg,ifset)+.0005)+1
          call arrget(nfx(ifset),nfy(ifset),nfz(ifset),1,j,
     &                htf(1,1,ifset),htset)
          do 340 n=1,nspecf
            kget = 1
            if (nfz(ifset) .ge. nzfg) then
              do 330 k=1,nfz(ifset)
                bcf(j,iedg,n,k,ifset) = cf1(ic,jc,n,kget)*weight
                carry = 1. - weight
                jcp = jc
  325           jcp = jcp + 1
                if (carry .gt. 0.) then
                  w2 = amin1(hrat,carry)
                  bcf(j,iedg,n,k,ifset) = bcf(j,iedg,n,k,ifset) 
     &                                    + cf1(ic,jcp,n,kget)*w2
                  carry = carry - w2
                  go to 325
                endif
                if (htset(k) .ge. 
     &              htg(ic,jc,kget)-.0005) kget = kget + 1
  330         continue
c
            else
              nkget = 0
              kg0 = 1
              k = 1
              hgm1 = 0.
              hsm1 = 0.
              bcf(j,iedg,n,k,ifset) = 0.
  335         nkget = nkget + 1
              wkg(nkget) = (htg(ic,jc,nkget)-hgm1)/
     &                     (htset(k)-hsm1)
              hgm1 = htg(ic,jc,nkget)
              if (htg(ic,jc,nkget)+.0005 .lt. htset(k)) go to 335
c
              do 336 kget=kg0,nkget
                bcf(j,iedg,n,k,ifset) = bcf(j,iedg,n,k,ifset) 
     &                             + cf1(ic,jc,n,kget)*weight*wkg(kget)
  336         continue
              carry = 1. - weight
              jcp = jc
  337         jcp = jcp + 1
              if (kg0 .gt. 1) then
                hgm1 = htg(ic,jcp,kg0-1)
              else
                hgm1 = 0.
              endif
              do 338 kget=kg0,nkget
                wkg(kget) = (htg(ic,jcp,kget)-hgm1)/
     &                      (htset(k)-hsm1)
                hgm1 = htg(ic,jcp,kget)
  338         continue
c
              if (carry .gt. 0.) then
                w2 = amin1(hrat,carry)
                do 339 kget=kg0,nkget
                bcf(j,iedg,n,k,ifset) = bcf(j,iedg,n,k,ifset) 
     &                               + cf1(ic,jcp,n,kget)*w2*wkg(kget)
  339           continue
                carry = carry - w2
                go to 337
              endif
              hgm1 = htg(ic,jc,nkget)
              hsm1 = htset(k)
              k = k + 1
              if (k .le. nfz(ifset)) then
                kg0 = nkget + 1
                bcf(j,iedg,n,k,ifset) = 0.
                go to 335
              endif
c
            endif
  340     continue
  350   continue
c
c    ---end of loop over fine grids
  400 continue
c
      return
      end
