      subroutine getpt(time,indate,iunit,nptsp,ieof)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: getpt.f,v $
c  Revision 1.9  1999/05/12  16:38:02  gem
c  added checks on stack temps against min/max values
c
c  Revision 1.8  1999/05/12  00:20:11  gem
c  added read of temporally varying stack temps
c
c  Revision 1.7  1998/11/16  18:46:52  ara
c  Made modifications to format of error message
c
c  Revision 1.6  1998/11/10  17:42:00  ara
c  Included errlog file to introduce ierrlg variable
c  Directed message errors to diag file and to log file (ierrlg)
c
c  Revision 1.5  1998/11/04  01:40:56  tcm
c  Revised date treatment to handle calendar dates past 1999 and
c  to default Julian dates to 1951 to 2050.
c
c  Revision 1.4  1996/06/14  23:00:16  tcm
c  min and max imposed on stack diameter by gem
c
c  Revision 1.3  1994/08/04  14:29:34  tcm
c  Added capability to read flowrat, and added calculation of
c  hour-specific exit velocities if flowrat > 0
c  CAE 7/6/94
c
c  Revision 1.2  94/02/10  20:56:40  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c
c  read a time interval off a UAM type point-source emissions file and 
c  return parameters 
c  (tcm, 12/26/90)
c
      include 'parm.cmd'
      include 'stack.cmd'
      include 'iptcon.cmd'
      include 'oneday.cmd'
      include 'errlog.cmd'
c
      character*4 mdum(10)
      real minstd,maxstd
      real mintst,maxtst
      data mintst,maxtst /260.,2000./
      data minstd,maxstd /0.01,100./
c     data minstd,maxstd /0.10,100./
c
      fact = 1./60.
      pi = 3.1415927
      ieof = 0
      ndate = indate
c
c  find correct time and read point-source emissions
c  (if ndate is calendar date, convert file date; but if ndate
c   is julian date, convert file date to julian)
c
      if (ndate .gt. 99999) then
        call caltoj(ndate,juldat)
        ndate = juldat
      endif
      if (iptdy1 .gt. 99999) then
        call caltoj(iptdy1,id1)
        call caltoj(iptdy2,id2)
      else
        id1 = iptdy1
        id2 = iptdy2
      endif
      ndays = intrvl(id1,ndate)
      tchk = pttim1*100. + 2400.*ndays
      ndays = intrvl(id2,ndate)
      tcend = pttim2*100. + 2400.*ndays
      if (time .ge. tchk-1. .and. time .le. tcend-1.) return
c
   50 read (iunit,end=900) iptdy1, pttim1, iptdy2, pttim2
      write(*,'(a,i8,f10.2,a,i8,f10.2,a,i4)')
     &     ' Read interval ',iptdy1, pttim1, ' to ', iptdy2, pttim2,
     &     ' on unit ', iunit
c
c  number of source on a record is not allowed to change
c
      read (iunit,end=900) idum, nn
      if (nn .ne. nopts) then
        write(*,'(a)') ' ILLEGAL REDEFINITION OF NO. OF SOURCES.'
        write(*,'(a,i5)') ' ORIGINAL:  ',nopts
cgem        write(*,'(a,i5)') ' NEW:       ',nopts
        write(*,'(a,i5)') ' NEW:       ',nn
        write(8,'(a)') ' ILLEGAL REDEFINITION OF NO. OF SOURCES.'
        write(8,'(a,i5)') ' ORIGINAL:  ',nopts
cgem         write(8,'(a,i5)') ' NEW:       ',nopts
        write(8,'(a,i5)') ' NEW:       ',nn
        write(ierrlg,'(i4,1x,a,a)') 1066,' ILLEGAL REDEFINITION OF ' 
     &        ,'NO. OF SOURCES.'
        write(ierrlg,'(a,i5)') ' ORIGINAL:  ',nopts
cgem         write(ierrlg,'(a,i5)') ' NEW:      ',nopts  
        write(ierrlg,'(a,i5)') ' NEW:        ',nn
        stop
      endif
c
c  skip the location record
c
      read (iunit,end=900)
     &      (idum,idum,idum,flowrat(n),stktmp(n),n=1,nopts)
c
c  if stktmp > 0, check against min/max and use in plume rise calcs
c  if flowrat > 0, convert flow rate to new exit velocity in m/s 
c
      do 80 i = 1,nopts
         if (stktmp(i).gt.0.) then
	    if ( stktmp(i) .lt. mintst ) then
	       write(*,*) ' in GETPT i,stktmp,mintst '
	       write(*,*)  i,stktmp(i),mintst
	       stktmp(i)=mintst
	    endif
	    if ( stktmp(i) .gt. maxtst ) then
	       write(*,*) ' in GETPT i,stktmp,maxtst '
	       write(*,*)  i,stktmp(i),maxtst
	       stktmp(i)=maxtst
	    endif
            lstkt(i)=.true.
         endif
         if (flowrat(i).gt.0.) then
	    if ( stkd(i) .lt. minstd ) then
	       write(*,*) ' in GETPT i,flowrate,stkd,minstd '
	       write(*,*)  i,flowrat(i),stkd(i),minstd
	       stkd(i)=minstd
	    endif
	    if ( stkd(i) .gt. maxstd ) then
	       write(*,*) ' in GETPT i,flowrate,stkd,maxstd '
	       write(*,*)  i,flowrat(i),stkd(i),maxstd
	       stkd(i)=maxstd
	    endif
            vexit(i) = flowrat(i)/(3600.*pi*(stkd(i)/2)**2)
         endif
   80 continue
c
c  emissions record
c
      do 100 l=1,nptsp
        read(iunit,end=900) idum,(mdum(i),i=1,10),
     &                     (ptemis(l,i),i=1,nopts)
c
c  convert emission rates from moles/hour to moles/min
c
        do 90 i=1,nopts
          ptemis(l,i) = ptemis(l,i) * fact
   90   continue
c
  100 continue
c
c  check time and if not correct, go back for another interval
c  (if ndate is calendar date, convert file date; but if ndate
c   is julian date, convert file date to julian)
c
      if (iptdy1 .gt. 99999) then
        call caltoj(iptdy1,id1)
        call caltoj(iptdy2,id2)
      else
        id1 = iptdy1
        id2 = iptdy2
      endif
      if (l1eday) then
        ndif = intrvl(id2,id1)
        id1 = ndate
        id2 = id1 + ndif
      endif
      iptdy1 = id1
      iptdy2 = id2
      ndays = intrvl(id1,ndate)
      tchk = pttim1*100. + 2400.*ndays
      ndays = intrvl(id2,ndate)
      tcend = pttim2*100. + 2400.*ndays
      if (time .ge. tcend) go to 50
      if (time .lt. tchk-1.) then
        write(*,*) ' Time error on PTSOURCE file',
     &             ' Current time: ',time, ndate,
     &             ' File time: ',pttim2,iptdy2
        write(8,*) ' Time error on PTSOURCE file',
     &             ' Current time: ',time,ndate, 
     &             ' File time: ',pttim2,iptdy2
       write(ierrlg,*) 1067, '  Time error on PTSOURCE file',
     &             ' Current time: ',time,ndate,
     &             ' File time: ',pttim2,iptdy2
        stop
      endif
c
      return
c
  900 ieof = 1
      return
c
      end
