      subroutine inject(deltat)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: inject.f,v $
c  Revision 1.3  1999/07/07  16:26:22  tcm
c  Added tracking of changes due to pig injection for pa/irr.
c
c  Revision 1.2  1994/02/10  20:57:30  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c
c
c-----this subroutine injects mass from puffs carried by the pig submodel
c     ttraj into the coarse grid.  the amount of mass injected is considered to 
c     exist in an "onion-skin" of each puff that has grown to meet dimensional 
c     conditions based on grid size.  input parameters have been calculated by 
c     ttraj.
c
c     also adds concentration interval to puffs occupying the same cell
c     (cae, 1/13/91)
c
c     inputs:
c        idmp, jdmp, kdmp - cell indices to receive mass from puff
c        condmp  - mass to be added to cell, in terms of an emission rate 
c                  (mol/min)
c        tdmp    - total time to dump mass into grid (min).  if tdmp is less 
c                  than or equal to zero, no mass will be dumped; this 
c                  signifies an "empty" array location for future puff dumps. 
c        voldmp  - volume of puff onion-skin that is added to grid cell (km^3).
c        dtdmp   - initial coarse grid timestep "length" of puff; used to
c                  adjust voldmp in cases where current timestep is different
c                  from puff's initial timestep.
c                  
c
c     outputs:
c        conc    - altered coarse grid cell concentrations (umol/m^3)
c
c     called by:
c        aaamain
c
c     routines called:
c        none
c
c     c emery   11/12/91
c
c--- modified to treat condmp as a simple emissions rate and not
c    remove mass from grid.  the appropriate adjustments have 
c    already been made in puffdmp.  (tcm, 1/22/92)
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'pigdung.cmd'
      include 'balanc.cmd'
      include 'pufloc.cmd'
      include 'pigpen.cmd'
      include 'empuf.cmd'
      dimension delc(mxspec)
c
czswa--------------------------------------------------------------------
c  include irmb and do irpam - pig stuff
c
      include 'irmb.cmd'
c
c  if we need to do irpam, then call get_difc
c
      call get_difc(1,1,1)
c
czswz--------------------------------------------------------------------
c    

c-----loop to find array locations with valid dump information.

      do 10 m = 1,mxdmp
        if (tdmp(m,1).le.0.) goto 10
        ix = idmp(m,1)
        jy = jdmp(m,1)
        kz = kdmp(m,1)
        if (kz.eq.1) then
          delz = h(ix,jy,kz)
        else
          delz = h(ix,jy,kz) - h(ix,jy,kz-1)
        endif

c-----determine from tdmp the fraction of timestep over which dump is occuring;
c     the maximum is 1. also find volume adjustment factor.

        dtfrac = amin1(tdmp(m,1)/deltat,1.)

c-----loop over species, and perform the following:
c     (1) reduce the current cell concentration to remove cell mass entrained 
c         by the puff. 
c     (2) add puff onion-skin mass to current cell

        do 35 l = 1,nospec
c
          conc(ix,jy,l,kz) = conc(ix,jy,l,kz) + condmp(l,m)*
     &                deltat*dtfrac/(deltax(jy)*deltay*delz*1.e9)
c
          pgflow(l) = pgflow(l)
     &                  + condmp(l,m)*deltat*dtfrac/1.e6
c
          if (conc(ix,jy,l,kz).lt.bdmol(l)) then
            gpflow(l) = gpflow(l) + (bdmol(l) - conc(ix,jy,l,kz))
     &                  *deltax(jy)*deltay*delz*1.e3
            conc(ix,jy,l,kz) = bdmol(l)
          endif
c
   35   continue
c
c-----adjust tdmp down one timestep -- if it goes to zero, or below, this 
c     onion-skin is finished (empty), and the current array location is 
c     available to be loaded with new puff injection information.

        tdmp(m,1) = tdmp(m,1) - deltat

   10 continue
c
czswa-------------------------------------------------------------------
c  do the second part of ipram - pig stuff
c
      call get_difc(1,2,1)
c
czswz-------------------------------------------------------------------
c
      return
      end
