      subroutine injectf(nxf,nyf,nzf,nospec,concf,htf,ifine)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: injectf.f,v $
c  Revision 1.3  1999/07/07  16:27:01  tcm
c  Added tracking of changes due to pig injection for pa/irr.
c
c  Revision 1.2  1994/02/10  20:57:35  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c
c

c-----this subroutine injects mass from puffs carried by the pig submodel
c     ttraj into a fine grid.  the amount of mass injected is considered to 
c     exist in an "onion-skin" of each puff that has grown to meet dimensional 
c     conditions based on grid size.  input parameters have been calculated by 
c     ttraj.
c
c     inputs:
c        ifine   - grid identification
c        concf   - fine grid concentrations
c        idmp, jdmp, kdmp - cell indices to receive mass from puff
c        condmp  - mass to be added to cell, in terms of an emission rate 
c                  (mol/min)
c        tdmp    - total time to dump mass into grid (min).  if tdmp is less 
c                  than or equal to zero, no mass will be dumped; this 
c                  signifies an "empty" array location for future puff dumps. 
c        voldmp  - volume of puff onion-skin that is added to grid cell (km^3).
c        dtdmp   - initial coarse grid timestep "length" of puff; used to
c                  adjust voldmp in cases where current timestep is different
c                  from puff's initial timestep.
c
c     outputs:
c        concf    - altered fine grid cell concentrations (umol/m^3)
c
c     called by:
c        aaamain
c
c     routines called:
c        none
c
c     c emery  11/12/91
c
c--- modified to treat condmp as a simple emissions rate and not
c    remove mass from grid.  the appropriate adjustments have 
c    already been made in puffdmp.  (tcm, 1/22/92)
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'comchem.cmd'
      include 'pigdung.cmd'
      include 'pufloc.cmd'
      include 'pigpen.cmd'
      include 'balanc.cmd'
      dimension concf(nxf,nyf,nospec,nzf), htf(nxf,nyf,nzf)
      dimension delc(mxspec)
c
czswa-----------------------------------------------------------------
c  ipram added
c
      include 'irmb.cmd'
c
czswz-----------------------------------------------------------------

c-----loop to find array locations with valid dump information.

      ifn = ifine + 1
      do 10 m = 1,mxdmp
        if (tdmp(m,ifn).le.0.) goto 10
        ix = idmp(m,ifn)
        jy = jdmp(m,ifn)
        kz = kdmp(m,ifn)
        if (kz.eq.1) then
          delz = htf(ix,jy,kz)
        else
          delz = htf(ix,jy,kz) - htf(ix,jy,kz-1)
        endif

c-----determine from tdmp the fraction of timestep over which dump is occuring;
c     the maximum is 1. also figure the fraction of the dumped volume to apply
c     to this timestep.

        dtfrac = amin1(tdmp(m,ifn)/deltf(ifine),1.)

c-----loop over species, and perform the following:
c     (1) reduce the current cell concentration to remove cell mass entrained 
c         by the puff. 
c     (2) add puff onion-skin mass to current cell
c
czswa----------------------------------------------------------------------
c  the do-35 loop is modified to get pig contribution in fine grid cell
c  artiemis will be added later ..........
c
c
        do 35 l = 1,nospec
          aapig = condmp(l,m)*deltf(ifine)*dtfrac/(delfx(jy,ifine)*
     &            delfy(ifine)*delz*1.e9)
c
          concf(ix,jy,l,kz) = concf(ix,jy,l,kz) + aapig
c
          iflgf = iflagf(ix,jy,ifine)
          if (iflgf.gt.0) then
             pigf(iflgf,l,kz) = pigf(iflgf,l,kz) + aapig
          endif
c
            fpgflo(l,ifine) = fpgflo(l,ifine)
     &                  + condmp(l,m)*deltf(ifine)*dtfrac/1.e6
c
          if (concf(ix,jy,l,kz).lt.bdmol(l)) then
            fgpflo(l,ifine) = fgpflo(l,ifine) + (bdmol(l) - 
     &                concf(ix,jy,l,kz))*delfx(jy,ifine)*delfy(ifine)
     &                *delz*1.e3
            concf(ix,jy,l,kz) = bdmol(l)
          endif
   35   continue
c
czswz----------------------------------------------------------------------
c
c-----adjust tdmp down one timestep -- if it goes to zero, or below, this 
c     onion-skin is finished (empty), and the current array location is 
c     available to be loaded with new puff injection information.

        tdmp(m,ifn) = tdmp(m,ifn) - deltf(ifine)

   10 continue

      return
      end
