      subroutine pblmod(ws,ta,tsurf,pa,zs,z0,elinv,ustar,wref)

c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c  revised 940305 tcm
c  calculation of richardson number changed to represent the depth
c  of 5 m to middle of layer 1 rather than 0 to middle.
c
c$Log: pblmod.f,v $
c  Revision 1.4  2000/07/03  21:44:19  tcm
c  Impose minimum value check on ustar.
c
c  Revision 1.3  1998/11/04  01:46:38  tcm
c  Modified as documented in memo to LADCO dated March 98.
c
c  Revision 1.2  94/02/10  20:58:35  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c**********************************************************************
c  this subroutine calculates u*, monin-obhukov length, and 10-m windspeed
c  based on parameterizations of louis (bound. lay. met., vol. 17,
c  1979, 187-202).
c
c  c emery, 10/10/91
c
c  input:
c    ws                 wind magnitude at first layer midpoint (m/s)
c    ta                 temperature at first layer midpoint (k)
c    tsurf              surface temperature (k)
c    pa                 pressure at first layer midpoint (mb)
c    zs                 height at top of first layer (m)
c    z0                 surface roughness length (m)
c  output:
c    ustar              friction velocity (m/s)
c    elinv              inverse of m-o length (m^-1)
c    wref               windspeed at reference (10 m) height (m/s)
c**********************************************************************

      real lnzz
      data zr /10./, vk /0.4/, pi /3.1415927/, eps /1.e-10/, g /9.8/
      data dpdz /0.11/, gamma /0.286/

      za = zs/2.
      ps = pa + dpdz*za
      theta1 = ta*(1000./pa)**gamma
      thetas = tsurf*(1000./ps)**gamma

c-----calculate temperature gradients, a drag coefficient, and richardson #.

      lnzz = alog(za/z0)
      lnzzr = alog(zr/z0)
      dtheta = theta1 - thetas
c
c   extrapolate wind down to d1 assuming log profile
c
      d1 = 5.
      thetad = thetas + dtheta/za * d1
      w1 = ws*alog(d1/z0)/lnzz
      dwdz = (ws - w1) / (za - d1)
      thbar = (thetad + theta1)/2.
      richnum = g*(dtheta/za)/(thbar*dwdz**2.+eps)
      a2 = (vk/lnzz)**2.
      ar2 = (vk/lnzzr)**2.

c-----find richardson number-dependent stability functions fm and fh

      if (richnum.lt.0.) then
        cm = 7.4*a2*9.4*sqrt(za/z0)
        cmr = 7.4*ar2*9.4*sqrt(zr/z0)
        ch = 5.3*a2*9.4*sqrt(za/z0)
        fm = 1.-9.4*richnum/(1.+cm*sqrt(abs(richnum))+eps)
        fmr = 1.-9.4*richnum/(1.+cmr*sqrt(abs(richnum))+eps)
        fh = 1.-9.4*richnum/(1.+ch*sqrt(abs(richnum))+eps)
      else
        fm = 1./((1.+4.7*richnum)**2.+eps)
        fmr = fm
        fh = fm
      endif

c-----find u*, theta* as f(fm,fh,m,dtheta,a2)

      ustr2 = a2*fm*ws**2.
      if (ustr2 .le. 0.0625) then
      ustr2 = 0.0625
      end if
      ustar = sqrt(ustr2)
c      
c-----assign a minimum ustar
c    
      thstr = 1.35*a2*ws*dtheta*fh/ustar

c-----calculate 10-m wind

      wref = sqrt(ustr2/(ar2*fmr))

c-----find m-o length

      el = ustr2*ta/(vk*g*thstr+eps)
      elinv = 1./(el+eps)
       
      return
      end
