      subroutine phot(time,nyr,nmth,nday,ix,jy,kz,z,factor,lnight)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: phot.f,v $
c  Revision 1.8  1999/12/21  19:28:12  tcm
c  Renamed com3 include file to comd3.
c
c  Revision 1.7  1999/05/12  23:45:10  dfg
c  *** empty log message ***
c
c  Revision 1.6  1999/05/12  23:43:02  dfg
c  added one more photolysis rate and ispd is rxn108 and aldx is 93 now.
c
c  Revision 1.5  1998/11/11  17:04:01  ara
c  Include file errlog.cmd to introduce ierrlg variable
c  Directed error message to diag file and to log file (ierrlg)
c
c  Revision 1.4  1996/06/14  23:01:23  tcm
c  see comments below
c
*
*  modified to use acrolein photolysis rates for isoprod chem
*  1996/05/28 hpd
*
c  Revision 1.3  1995/04/26  00:42:09  tcm
c  	modified to use RADM cloud adjustment factors, but only when cloud cover is
c  	more than 70%
c
c  Revision 1.2  94/02/10  20:58:46  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c
c     determine photolysis rate constant factor (1/min)
c     for cell (i,j) at altitude z meters
c
c -- modified to correct zenith angle data statement.  see
c    explanation below (tcm, 6/13/91)
c
c -- modified to select table of photolysis rates based on
c    albedo, haze, and ozone column (tcm, 4/6/92)
c
c     called by : rchem
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'latlon.cmd'
      include 'comchem.cmd'
      include 'comd3.cmd'
      include 'photrk.cmd'
      include 'errlog.cmd'
c
      dimension zen(11),zht(11)
      real rkkk(7)
      dimension cldadj(11)
      logical lnight
c
c     cosine of zenith angles for k1 interpolation
c     (0,10,20,30,40,50,60,70,78,86,90)
c
      data zen/1.000,0.984,0.940,0.866,0.766
     &        ,0.643,0.500,0.342,0.208,0.070,0.000/
c
c     array zht is heights for interpolation  (in meters)
c
      data zht/0.0,150.0,360.0,640.0,980.0,1380.0,1840.0
     &        ,2350.0,2910.0,3530.0,4210.0/
c
c     cldadj(11) array of k1 adjustments for cloud cover
c     (0-10 clear to overcast)
c     source: maul,1980
c      data cldadj/1.0,0.91,0.84,0.79,0.75,0.72,0.68,0.62
c     &               ,0.53,0.41,0.23/
c
c     source: ilyas, 1986, atmos. env.
c     data cldadj/1.0,0.94,0.89,0.83,0.78,0.72,0.66,0.61
c    &               ,0.55,0.50,0.44/
c
c   cloud adjustment factors from RADM (entered, 3/13/95, tcm)
c     data cldadj/1.0,0.96,0.92,0.88,0.84,0.80,0.76,0.72
c    &               ,0.68,0.64,0.59/
c   cloud adjustment factors from RADM (entered, 3/13/95, tcm)
c   clouds adjustment only used if >= 70% cover
      data cldadj/1.0,1.00,1.00,1.00,1.00,1.00,1.00,0.72
     &               ,0.68,0.64,0.59/
c
c
c
c ************************************************
c   warning !!! -- this is cbm-iv hardwired !!!!
c ************************************************
c
c the id numbers of the photolysis reactions are .....
c
c          nrxn / 1,38,39,9,45,93 /
c
c and, they must be read in from the nphval file in the order
c  no2, hchor, hchos, o3o1d, alds,ispd
c
c
c     calculate cosine of solar zenith angle.  first, determine location
c
      lnight=.false.
      rllng=-ctlon(ix,jy)
      rllt=ctlat(ix,jy)
c
c   get class codes for albedo, ozone column, and haze
c
      ialb = jalb(ix,jy)
      iozc = jozc(ix,jy)
      ihaz = jhaz(ix,jy)
      if (ialb .lt. 1 .or. ialb .gt. 5 .or. 
     &    iozc .lt. 1 .or. iozc .gt. 5 .or.
     &    ihaz .lt. 1 .or. ihaz .gt. 3) then
        write(*,*) ' Invalid value for albedo, haze, or ',
     &      ' ozone column code at cell ',ix, jy
        write(*,*) ' ialb, iozc, ihaz: ',ialb, iozc, ihaz
        write(8,*) ' Invalid value for albedo, haze, or ',
     &      ' ozone column code at cell ',ix, jy
        write(8,*) ' ialb, iozc, ihaz: ',ialb, iozc, ihaz
        write(ierrlg,*) 1092, '  Invalid value for albedo, haze, or ',
     &      ' ozone column code at cell ',ix, jy
        write(ierrlg,*) ' ialb, iozc, ihaz: ',ialb, iozc, ihaz
        stop
      endif
c
c     note solar actually wants negative of w. longitude
c                               (i.e. east longitude)
c
      call solar(rllt,rllng,tz,nyr,nmth,nday,time,zcoef)
c
c     determine height factors for interpolation
c
      if (z.gt.0.0) then
        do 4 iz=2,11                                   
          if (z.le.zht(iz)) then
            iz1=iz-1                                       
            iz2=iz                                         
            ratz1=(zht(iz2)-z)/(zht(iz2)-zht(iz1))         
            ratz2=1.0-ratz1                                
            go to 8
          endif
    4   continue                                       
        iz1=11                                         
        iz2=11                                         
        ratz1=1.0                                      
        ratz2=0.0 
      else
        iz1=1            
        iz2=1            
        ratz1=1.0        
        ratz2=0.0        
      endif
c
    8 continue
c
c      determine zenith angle factors for interpolation
c
      if (zcoef.lt.zen(1))  then
        if (zcoef.le.zen(11)) then
          izn1 = 11                                                
          izn2 = 11                                                
          ratzn1 = 0.                                              
          ratzn2 = 1.                                              
        else
          do 10 k=2,11                                               
            if (zcoef.gt.zen(k)) then
              izn1 = k-1                                               
              izn2 = k                                                 
              ratzn2 = (zen(izn1)-zcoef)/(zen(izn1)-zen(izn2))         
              ratzn1 = (zcoef-zen(izn2))/(zen(izn1)-zen(izn2))   
              go to 20
            endif
   10     continue                                                   
        endif
      else
        izn1 = 1                          
        izn2 = 1                          
        ratzn1 = 1.                       
        ratzn2 = 0.                       
      endif  
c
   20 continue
c                                                                
c       calculate cloud transmission factors                     
c                                                                
      icld=int(cloud(ix,jy,kz)*10.0+1.5)                         
      if (icld.gt.11) icld=11
      if (icld.lt.1) icld=1
c
c      calculate rates for the six photolytic species
c
c
c      for each species ns, interpolate over zenith angles at high and 
c      low heights, then interpolate over height and apply cloud factor.
c
      do 600 ns=1,7
        prlow = prkn(ns,izn1,iz1,iozc,ialb,ihaz)*ratzn1 +
     &           prkn(ns,izn2,iz1,iozc,ialb,ihaz)*ratzn2
        prhigh= prkn(ns,izn1,iz2,iozc,ialb,ihaz)*ratzn1 +
     &           prkn(ns,izn2,iz2,iozc,ialb,ihaz)*ratzn2
        rkkk(ns)= (prlow*ratz1 + prhigh*ratz2)*cldadj(icld)
  600 continue
c
c     *** warning cbm-iv hard wired here ***
c
      factor = rkkk(1)
      lnight=.false.
      if (factor.le.0.0015) lnight=.true.
      if (lnight) factor=0.0
c
c     now do o3(rxn9), hcho-rads(rxn38), hcho-co(rxn39), and
c     acet(rxn45),ispd(rxn108), aldx(rxn93)
c
      if (lnight) return
c
      rk(1)  = rkkk(1)*rkn(1)
      rk(9)  = rkkk(4)*rkn(9)
      rk(38) = rkkk(2)*rkn(38)
      rk(39) = rkkk(3)*rkn(39)
      rk(45) = rkkk(5)*rkn(45)
      rk(108) = rkkk(6)*rkn(108)
      rk(93) = rkkk(7)*rkn(93)
c
c     o3 to o(rxn8), hno2(rxn23), no3(rxn14) ratioed off of no2

      rk(8)  = rk(1)*rkn(8)
      rk(14) = rk(1)*rkn(14)
      rk(23) = rk(1)*rkn(23)
c
c     h2o2(rxn34) ratioed off of hcho-co; mhp and paa equal to h2o2
c
      rk(34) = rk(39) * rkn(34)
*      rk(89) = rk(34)
*      rk(90) = rk(34)
c
c     open(rxn69) and mgly(rxn74) ratioed off of hcho-rads
c
      rk(69) = rk(38)*rkn(69)
*      rk(74) = rk(38)*rkn(74)
*
* changed mgly to be ratioed off of no2
*
      rk(74) = rk(1)*rkn(74)
c
c  do the rest even if laq is false as some is needed for deposition
c
c     aqueous photolysis rates
c
      atten = 0.85
c
c
  900 return
c
 2050 format (a80)
 2051 format (1x,10f12.0)
c
      end
