      subroutine rpm(nutrc,nnspec,dt0,time,idate,ientry,lerror)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: rpm.f,v $
c  Revision 1.7  1998/11/19  00:17:15  ara
c  Eliminated error message directed to unit 1
c
c  Revision 1.6  1998/11/18  16:52:59  ara
c  Directed warning message to screen,diag file and to log file (ierrlg)
c
c  Revision 1.5  1998/11/12  21:48:05  ara
c  Included errlog.cmd file to introduce ierrlg variable
c  Directed error message to screen, diag file and log file (ierrlg)
c
c  Revision 1.4  1998/11/04  01:52:39  tcm
c  Revised date treatment.
c
c  Revision 1.3  1995/04/26  00:44:42  tcm
c  	corrected subscript in check of plume reactivity flag
c
c  Revision 1.2  94/02/10  20:59:47  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
C     *** Drive for the reactive plume model (RPM) routines within
c     *** the PIG model.
c     called by : ttraj
c     calls : euler, dochem, switch, initlz
c
c     save
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'pigpen.cmd'
      include 'rpm.cmd'
      include 'pufloc.cmd'
      include 'comuam.cmd'
      include 'stack.cmd'
      include 'errlog.cmd'
      common /idcell/icell,jcell,kcell,tmchm
c
      logical lerror, lpreac
      defnw=4.0
      nspec=nnspec+1
      icell = -10
c
c     extract date
c
      if (idate .lt. 100000) then
        call jtocal(idate,icdate)
        nyr=icdate/10000
        nmth=icdate/100-nyr*100
        nday=icdate-nmth*100-nyr*10000
      else
        nyr=idate/10000
        nmth=idate/100-nyr*100
        nday=idate-nmth*100-nyr*10000
      endif
      if (nyr .lt. 100) then
        if (nyr .lt. 51) then
          nyr = nyr + 2000
        else
          nyr = nyr + 1900
        endif
      endif
c
c     this is the control subroutine for the rpm simulations within the
c     airshed
c
      if(ientry.ge.2) go to 50
      do 40 ipf=1,mxpfs
        if (nskins(ipf).eq.0) go to 40
        if (abs(nskins(ipf)).gt.nrclls) then
          write(*,3000) ipf,nskins(ipf),dt 
          write(8,3000) ipf,nskins(ipf),dt
          write(ierrlg,3001) 1104,ipf,nskins(ipf),dt
 3000     format(' bad nskins value for puff ',i6,' nskins= ',i6,
     &           ' dt= ',e12.4)
 3001     format(i4,1x,' bad nskins value for puff ',i6,'nskins= ',i6,
     &           ' dt= ',e12.4)
          stop
        endif
        ncells=nskins(ipf)
c
c     dump trajectory and ambient airshed data into local arrays for
c     plume simulation
c
        sigmy=sigma(ipf)
        osigmy=osgy(ipf)
        sigmz=sigmav(ipf)
        osigmz=osgz(ipf)
        sigy=osigmy
        sigz=osigmz
        jg = ifix(yloc(ipf)/deltay) + 1
        if (jg .lt. 1) jg = 1
        if (jg .gt. noy) jg = noy
        ig = ifix(xloc(ipf)/deltax(jg)) + 1
        if (ig .lt. 1) ig = 1
        if (ig .gt. nox) ig = nox
        do 30 k = 1,nlayer
          if (height(ipf).lt.h(ig,jg,k)) then
            kg = k
            goto 31
          endif
   30   continue
        kg = nlayer
   31   continue
        tcell = temper(ig,jg,kg)
        pcell = press(ig,jg,kg)
        zavght = height(ipf) + avterr(ig,jg)
c
        do 10 nsp=1,nspec
          do 11 nc=1,ncells
            if (ientry .eq. 1) pconc(nc,nsp)=concs(nsp,nc,ipf)
            if (ientry .eq. 0) pconc(nc,nsp)=connul(nsp,nc,ipf)
   11     continue
   10   continue
c
c     call subroutine euler to establish new concentrations resulting
c     from plume expansion. (i.e. entrainment and relative
c     diffusion)
c
        t = 0.
        dt = dt0
        iflag = 0
   15   continue
c
        call euler(ncells,nutrc,dt,dt0,ipf,iflag)
c
c     call subroutine dochem to perform chemical transformation
c     in all 15 species
c
        jcell = ipf
c              
        lpreac = prflg(nplume(ipf))
        do 25 npr=1,nsrcmbn(ipf)
          if (prflg(isrcmbn(ipf,npr))) lpreac = .true.
   25   continue
        call dochem(time,nyr,nmth,nday,nnspec,ncells,
     &  ig,jg,kg,dt,tcell,pcell,zavght,lpreac,lerror,nplume(ipf))
        if (lerror) then
          write(nutrc,2070) ipf
          write(*,2070) ipf
          write(8,2070) ipf
          write(ierrlg,2071) 282,ipf 
 2070     format( ' dochem reports a bad puff to rpm, #',i10)
 2071     format(i3,1x,' dochem reports a bad puff to rpm, #',i10)
          return
        endif
c
c     now either back to $15 to complete full time step
c     dt0 or to next puff
c
        if ( (dt0-t) .gt. 0.0001) go to 15
c
c     dump concentration array into concs for possible dumping
c
        do 20 nc=1,ncells
          do 21 nsp=1,nspec
            if (ientry .eq. 1) concs(nsp,nc,ipf)=pconc(nc,nsp)
            if (ientry .eq. 0) connul(nsp,nc,ipf)=pconc(nc,nsp)
   21     continue
   20   continue
c
c
c     now for the next puff
c
c
   40 continue
c
c     all plumes and puffs have advanced to new time step dt0
c     with the airshed
c     dump concentration array into concs and dump common block
c     /airrpm/ into /oldrpm/ for next time step
c
      if (ientry .eq. 0) call switch(nspec)
      return
c
c     initialize puffs corresponding to -nskins(ipf)=2
c
   50 do 70 ipf = 1, mxpfs
        if(nskins(ipf).ge.0) go to 70
        ncells = abs(nskins(ipf))
        call initlz(ncells,ipf,ientry)
c
c        load into old arrays
c
        osgy(ipf) = sigma(ipf)
        osgz(ipf) = sigmav(ipf)
        do 60 nsp = 1, nspec
          oaml(nsp,ipf) = aml(nsp,ipf)
          do 61 nc=1,ncells
            if (ientry .eq. 2) concs(nsp,nc,ipf) = pconc(nc,nsp)
            if (ientry .eq. 3) connul(nsp,nc,ipf) = pconc(nc,nsp)
   61     continue
   60   continue
   70 continue
      return
      end
