c----------------------------------------------------------------
c     subroutine rvmmap -- read variable-mesh map
c----------------------------------------------------------------
      subroutine rvmmap(nxx,nyy,nzz,iout,time,ndate,idebug)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: rvmmap.f,v $
c  Revision 1.11  1998/11/20  22:59:25  tcm
c  Corrected format of error message on line 606
c
c  Revision 1.10  1998/11/19  00:15:58  ara
c  Eliminated error messages directed to diag file (8)
c  as they were already directed by variable Iout
c
c  Revision 1.9  1998/11/18  17:33:03  ara
c  Directed warning messages to screen, diag file and log file
c
c  Revision 1.8  1998/11/16  21:01:44  ara
c  Corrected error in error message format
c
c  Revision 1.7  1998/11/12  23:06:41  ara
c  Included errlog.cmd to introduce ierrlg variable
c  Directd error messages to screen, diag and log file (ierrlg)
c  Some warnings may be errors: new revision needed
c
c  Revision 1.6  1998/11/06  00:16:43  tcm
c  Added file identifier to fine grid average and instant file headers.
c
c  Revision 1.5  1998/11/04  01:53:05  tcm
c  Revised date treatment.
c
c  Revision 1.4  1996/06/14  23:02:45  tcm
c  inactive code removed by gem
c
c  Revision 1.3  1995/04/26  00:45:02  tcm
c  	removed read of fine grid nest maps.  fine grid definitions are now read in
c  	setup.  added code to define igloc, jhloc, and fhloc, that define adjacent grids
c  	and factors for interpolation.  disable chemistry aggregation if end of file
c
c  Revision 1.2  94/02/10  20:59:52  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c
c
c  modified 2/5/92, tcm: accept times on file earlier than current
c  time and ndate, then go back to 451 to see if there is a time
c  closer than the last time read.  keep doing this until a tim
c  later than current time/ndate is found.  this will allow the
c  same agg file to be used for a day even if start time is not
c  exactly the same as beg time on file.  however, the program
c  will spend a little more time in rvmmap because it will go
c  through all the aggregate definition for each interval even
c  though it may not use it.
c
c  modified 7/3/91, tcm: fill in map for each grid that defines
c  locations of nest interfaces. (ingmap and ifngmp)
c
c  modified 2/11/91, tcm:  allow for multiple layers of nesting
c  in fine grids (i.e., fine grids nested within fine grids)
c
c  modified 9/20/90, tcm:  read definitions of fine grid areas in
c  addition to cell aggregation.
c  
c  Locations of fine grids are read from control file in setup.
c  
c  fine grid regions must be rectangles and must use the same
c  number of subdivisions throughout a given fine grid. 
c  fine grids are off limits to cell aggregation.
c
c  Three maps will be printed in the diagnostic file showing where
c  the fine grids are located.  In the example, there are 3 fine grids.
c  The first is at the upper left with two subdivisions
c  per coarse cell, another next to it also with two subdivisions
c  and a third with 5 subdivisions at the lower right. 
c  
c  Horizontal subdivisions:
c 
c     0000000000000000
c     0002222000000000
c     0002222bbbb00000
c     0002222bbbb00000
c     0000000000000550
c     0000000000000550
c     0000000000000000
c
c  Vertical subdivisions:
c
c     0000000000000000
c     0001111000000000
c     0001111111100000
c     0001111111100000
c     0000000000000110
c     0000000000000110
c     0000000000000000
c
c  Grid id numbers:
c
c     0000000000000000
c     0002222000000000
c     0002222333300000
c     0002222333300000
c     0000000000000110
c     0000000000000110
c     0000000000000000
c
c     subroutine reads a variable-mesh map consisting of a set of
c     two-dimensional integer arrays (each member of the set is one
c     layer) indicating clusters of grid cells to be treated as
c     aggregate cells for computing chemistry.  the three-dimen-
c     sional aggregate cells must be characterizable by three 
c     orthogonal lengths representing multiples of the fundamental
c     (i.e., smallest) grid dimensions, although the user need not
c     specify these.  they are determined automatically.  the user 
c     must privide one "map" for each layer.  each map consists of
c     aggregate cells suitably "colored" by one-character integers.
c     allowable "colors" are 1 through 9.  boundary cells must be
c     "colored" with the integer 0. boundaries may be irregular.
c     for example, the following map contains 32 aggregate cells in
c     a particular layer (5 of which are clusters of cells).
c
c     0000000000000000
c     0111113332212430
c     0111113331341210
c     0111112213141400
c     0444442231213000
c     0444441412120000
c     0000000000000000
c
c     the aggregates are a) one 5 x 3 cell in the upper left corner
c                        b) one 5 x 2 cell in the lower left corner
c                        c) one 3 x 2 cell adjacent to the cell
c                           identified in (a)
c                        d) one 2 x 1 cell to the right of the one
c                           identified in (c)
c                        e) one 2 x 2 cell to the right of the hori-
c                           zontal boundary separating the cells 
c                           identified in (a) and (b)
c                        f) 27 one-cell aggregates
c
c     note that all other fundamental cells (the one-cell aggregates) 
c     are identified with integers
c     unique with respect to adjacent cells.  (recall the four "color"
c     map problem).  all you need are four integers per layer.  when
c     "coloring" the adjacent layer, make sure no vertically adjacent
c     cells are colored the same unless they form part of an aggregate. 
c     for example, the
c     following map for layer 2 would aggregate the upper left-hand cluster
c     of cells in layer 1 with layer 2:
c
c     0000000000000000
c     0111114545454540
c     0111115454545450
c     0111114545454540
c     0454545454545450
c     0545454545454540
c     0000000000000000 
c
c     this subroutine checks that vertical aggregation is specified properly.
c     for example, identifying a 5 x 2 block of cells in the upper left
c     corner is invalid because the aggregate cannot be specified in terms
c     of three ranges of grid indices. 
c
c     one final note:  grids should be created with north along upper row
c     as on a map.  j indices are reversed in the subroutine.  enough text!
c
c    pertinent variables from common /cntrol/ 
c
c           ncm = number of aggregate cells
c       icmb(n) = beginning i-index of n-th aggregate cell 
c       icme(n) = ending    i-index of n-th aggregate cell
c       jcmb(n) = beginning j-index of n-th aggregate cell
c       jcme(n) = ending    j-index of n-th aggregate cell
c       kcmb(n) = beginning k-index of n-th aggregate cell
c       kcme(n) = ending    k-index of n-th aggregate cell
c
c       ivmm(i,j,k) = integer array of dimension nxx,nyy,nzz containing
c                     the "colored" map of aggregate cells.  this array is
c                     read from file vmeshm.in in an i1 format. the array
c                     elements are negated during the process of calculating
c                     aggregate cell bounds and used as a vertical alignment
c                     check,
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'ngmap.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'hedmsg.cmd'
      include 'errlog.cmd'
      save ifstrv, iaglod
c
      integer*2 ivmm(mxx,mxy,mxz)
      character*1 clast, vlast, cgrid
      character*35 cdivid
      common /fgsav/ mfhrz(mxy), mfvrt(mxy)
      character*120 mfhrz, mfvrt
      logical lerr
      character*10 mfavg, mfinst
c
      data cdivid/'123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      data ifstrv /0/, iaglod/0/
      data ione /1/
      data mfavg /'FAVERAGE  '/, mfinst /'FINSTANT  '/
c
c   fourth bit of idebug (i.e., 8's place) set will turn on
c   diagnostic writes in rvmmap
c
      idiag = and(idebug,8)
c
      lerr = .false.
c
c----------------------------
c     test dimensions
c----------------------------
      write(iout,2221)
 2221 format(' entered rvmmap')
      jtime=int(time)
      ntest=nxx*nyy*nzz
      if(ntest.gt.maxagg) then
        write(iout,1000) nxx,nyy,nzz,ntest,maxagg
        write(*,1000) nxx,nyy,nzz,ntest,maxagg        
        write(ierrlg,1101) 1105,nxx,nyy,nzz,ntest,maxagg
 1000   format(' abort in rvmmap:'
     &,    ' nox*noy*noz=',i2,'*',i2,'*',i2,'=',i4,'.gt.',i4,'(maxagg)')
 1101   format(i4,1x,' abort in rvmmap:'
     &,    ' nox*noy*noz=',i2,'*',i2,'*',i2,'=',i4,'.gt.',i4,'(maxagg)')
        stop
      endif
c
      if (idiag .ne. 0) then
        write(iout,*) 'ifstrv, iaglod = ',ifstrv, iaglod
      endif
c
c  fill in fine grid maps
c
      if (ifstrv .eq. 0) then
        ifstrv = 1
c
c  initialize ingmap and ifngmp to zeros
c  initialize numgrd to zero (i.e., entire domain is coarse grid)
c  initialize fine grid maps to indicate all coarse grid
c  initialize kvmap to indicate one-to-one correspondence of layers
c             between all grids
c
        do 10 ifine=1,numfin
          do 6 j=1,mxfy
            do 5 i=1,mxfx
              ifngmp(i,j,ifine) = 0
    5       continue
    6     continue
          do 8 k=1,mxfz
            kvmap(k,ifine) = k
    8     continue
   10   continue
        do 15 j=1,mxy
          do 14 i=1,mxx
            ingmap(i,j) = 0
            numgrd(i,j) = 0
            mfhrz(j)(i:i) = '0'
            mfvrt(j)(i:i) = '0'
   14     continue
   15   continue
c
c  set up remaining parameters for fine grid
c  (number and location read in the routine setup)
c
        do 420 ifine=1,numfin
          ifgptr(ifine) = 0
          ifglvl(ifine) = 1
c
c         check if this fine grid is within any other fine grids
c         defined so far
c
          do 640 if0 = 1,numfin
            if (if0 .eq. ifine) go to 640
            if (ixfb(ifine) .ge. ixfb(if0) .and.
     &          ixfb(ifine) .le. ixfe(if0) .and.
     &          jyfb(ifine) .ge. jyfb(if0) .and.
     &          jyfb(ifine) .le. jyfe(if0) ) then
              if (ixfe(ifine) .lt. ixfb(if0) .and.
     &            ixfe(ifine) .gt. ixfe(if0) .and.
     &            jyfe(ifine) .lt. jyfb(if0) .and.
     &            jyfe(ifine) .gt. jyfe(if0) ) then
                write(iout,*) ' ***error***',
     &              ' improper nesting at ',ixfb(ifine),
     &              jyfb(ifine)
                write(*,*) ' ***error***',
     &              ' improper nesting at ',ixfb(ifine),
     &              jyfb(ifine)
                write(ierrlg,*) 1106,'  ***error***',
     &              ' improper nesting at ',ixfb(ifine),
     &              jyfb(ifine)
                lerr = .true.
              endif
              if (if0 .gt. ifine) then
                write(iout,*) ' ******error******'
                write(iout,*) ' Grid nests defined in improper order.'
                write(iout,*) ' Grid ',ifine,' is inside grid ',if0
                write(iout,*) ' Definition order must be from outer'
                write(iout,*) ' grid to inner grids.'
                write(*,*) ' ******error******'
                write(*,*) ' Grid nests defined in improper order.'
                write(*,*) ' Grid ',ifine,' is inside grid ',if0
                write(*,*) ' Definition order must be from outer'
                write(*,*) ' grid to inner grids.'
                write(ierrlg,*) 1107,'  ******error******'
                write(ierrlg,*) ' Grid nests defined in improper order.'
                write(ierrlg,*) ' Grid ',ifine,' is inside grid ',if0
                write(ierrlg,*) ' Definition order must be from outer'
                write(ierrlg,*) ' grid to inner grids.'
                lerr = .true.
              endif
              ifgptr(ifine) = if0
              ifglvl(ifine) = ifglvl(if0) + 1
            endif
  640     continue
c
  642     nfx(ifine) = (ixfe(ifine)-ixfb(ifine)+1)*nhf(ifine)+2
          nfy(ifine) = (jyfe(ifine)-jyfb(ifine)+1)*nhf(ifine)+2
          nvf(ifine) = (nfz(ifine)-1)/nlayer+1
          if (idiag .ne. 0) then
            write(iout,'(4x,3i4,4x,2i4)')
     &          nfx(ifine), nfy(ifine), nfz(ifine),
     &          nhf(ifine), nvf(ifine)
          endif
c
c          check all these parameters against the maxima
c
          if (nfx(ifine) .gt. mxfx .or. 
     &        nfy(ifine) .gt. mxfy .or.
     &        nfz(ifine) .gt. mxfz) then
            write(iout,'(a,3i4,/,a,3i4)')
     &        ' maximum x y or z exceeded for fine grid',
     &        nfx(ifine), nfy(ifine), nfz(ifine),
     &        ' max values are:',mxfx,mxfy,mxfz
             write(*,'(a,3i4,/,a,3i4)')
     &        ' maximum x y or z exceeded for fine grid',
     &        nfx(ifine), nfy(ifine), nfz(ifine),
     &        ' max values are:',mxfx,mxfy,mxfz
             write(ierrlg,'(i4,1x,a,3i4,/,a,3i4)') 1108,
     &        ' maximum x y or z exceeded for fin grid',
     &        nfx(ifine), nfy(ifine), nfz(ifine),
     &        ' max values are:',mxfx,mxfy,mxfz
            lerr = .true.
          endif
          if (nfx(ifine) .gt. mxxx .or.
     &        nfy(ifine) .gt. mxxx) then
            write(iout,'(a,2i4,a,i4,/,a)') 
     &        'x or y fine grid size of ',nfx(ifine), nfy(ifine),
     &        ' exceeds maximum of ',mxxx,
     &        ' allowed in common shas'
            write(*,'(a,2i4,a,i4,/,a)')
     &        'x or y fine grid size of ',nfx(ifine), nfy(ifine),
     &        'exceeds maximum of ',mxxx,
     &        ' allowed in common shas'
            write(ierrlg,'(i4,1x,a,2i4,a,i4,/,a)') 1109,
     &        'x or y fine grid size of ',nfx(ifine), nfy(ifine),
     &        'exceeds maximum of ',mxxx,
     &        'allowed in common shas'
            lerr = .true.
          endif
          ixby = nfx(ifine)*nfy(ifine)
          if (ixby .gt. mxfxby) then
            write(iout,'(a,3i5,/,a,i5)') 
     &        ' max x by y dimension exceeded',
     &        nfx(ifine), nfy(ifine), ixby,
     &        ' max is ',mxfxby
            lerr = .true.
          endif
          if (nhf(ifine) .gt. maxdiv .or. 
     &        nvf(ifine) .gt. maxdiv ) then
            write(iout,'(a,2i4,/,a,i4)')
     &        ' maximum subdivisions exceeded for fine grid',
     &        nhf(ifine), nvf(ifine),
     &        ' max value is: ',maxdiv
            write(*,'(a,2i4,/,a,i4)')
     &        ' maximum subdivisions exceeded for fine grid',
     &        nhf(ifine), nvf(ifine),
     &        ' max value is: ',maxdiv
            write(ierrlg,'(i4,1x,a,2i4,/,a,i4)') 1110,
     &        ' maximum subdivisions exceeded for fine grid',
     &        nhf(ifine),nvf(ifine),
     &        ' max value is: ',maxdiv
            lerr = .true.
          endif
c
  420   continue
c
c   write out fine grid maps for user to check
c   also fill in map of nest interfaces
c
        write(iout,'(1x,i4,a)') numfin, ' fine grids defined.'
        write(iout,'(a)') ' the following definition of fine grids ',
     &                    ' will be used:',' '
        do 45 nn=1,numfin
          if (ifgptr(nn) .ne. 0) then
            if2 = ifgptr(nn)
            ixff0 = (ixfb(nn) - ixfb(if2))*nhf(if2) + 2
            ixff1 = (ixfe(nn) - ixfb(if2) + 1)*nhf(if2) + 1
            jyff0 = (jyfb(nn) - jyfb(if2))*nhf(if2) + 2
            jyff1 = (jyfe(nn) - jyfb(if2) + 1)*nhf(if2) + 1
            ii = ixff0 - 1
            ii2 = ixff1
            do 23 j=jyff0,jyff1
              ifngmp(ii,j,if2) = 1
              ifngmp(ii2,j,if2) = 1
   23       continue
            jj = jyff0 - 1
            jj2 = jyff1
            do 24 i=ixff0,ixff1
              ifngmp(i,jj,if2) = ifngmp(i,jj,if2) + 10
              ifngmp(i,jj2,if2) = ifngmp(i,jj2,if2) + 10
   24       continue
          else
            ii = ixfb(nn) - 1
            ii2 = ixfe(nn)
            do 25 j=jyfb(nn),jyfe(nn)
              ingmap(ii,j) = 1
              ingmap(ii2,j) = 1
   25       continue
            jj = jyfb(nn) - 1
            jj2 = jyfe(nn)
            do 26 i=ixfb(nn),ixfe(nn)
              ingmap(i,jj) = ingmap(i,jj) + 10
              ingmap(i,jj2) = ingmap(i,jj2) + 10
   26       continue
          endif
c
          if (nhf(nn) .lt. 35) then
            clast = cdivid(nhf(nn):nhf(nn))
          else
            clast = '*'
          endif
          if (nvf(nn) .lt. 35) then
            vlast = cdivid(nvf(nn):nvf(nn))
          else
            vlast = '*'
          endif
          do 40 j=jyfb(nn), jyfe(nn)
            do 35 i=ixfb(nn), ixfe(nn)
              mfhrz(j)(i:i) = clast
              mfvrt(j)(i:i) = vlast
              numgrd(i,j) = nn
   35       continue
   40     continue
   45   continue
c
        write(iout,'(/,a,/)') '     horizontal subdivisions:'
        do 47 j=nyy,1,-1
          write(iout,'(5x,120a1)') (mfhrz(j)(i:i),i=1,nxx)
   47   continue
        write(iout,'(/,a,/)') '     vertical subdivisions:'
        do 48 j=nyy,1,-1
          write(iout,'(5x,120a1)') (mfvrt(j)(i:i),i=1,nxx)
   48   continue
        write(iout,'(/,a,/)') '     map of grid id nos.:'
        do 49 j=nyy,1,-1
          write(iout,'(5x,120z1)') (numgrd(i,j),i=1,nxx)
   49   continue
c
c   look for fine grids that are adjacent to each other
c   set igloc and fhloc to point to row/col to use for
c   inflow from the adjacent grid
c   (igloc = 0 means no fine grid adjacent to this one,
c    otherwise the value is the fine grid that is adjacent)
c
        do 90 ifine=1,numfin
          do 53 iedg=1,2
            do 51 j=1,nfx(ifine)
              igloc(j,iedg,ifine) = 0
              fhloc(j,iedg,ifine) = 0.
   51       continue
            do 52 j=1,nfy(ifine)
              igloc(j,iedg+2,ifine) = 0
              fhloc(j,iedg+2,ifine) = 0.
   52       continue
   53     continue
c
c         --- check what sits on north edge of ifine
          iedg = 1
          do 56 ic=ixfb(ifine),ixfe(ifine)
            ic1 = (ic-ixfb(ifine))*nhf(ifine)+2
            ic2 = ic1 + nhf(ifine) - 1
            if0 = numgrd(ic,jyfe(ifine)+1)
            if (if0 .eq. 0) then
              fh1 = float(ic-1)
              hrat = 1./float(nhf(ifine))
              do 54 i=ic1,ic2
                igloc(i,iedg,ifine) = if0
                fhloc(i,iedg,ifine) = fh1 + float(i-ic1)*hrat
                jhloc(i,iedg,ifine) = jyfe(ifine)+1
   54         continue
            else
              fh1 = float((ic-ixfb(if0))*nhf(if0)+1)
              hrat = float(nhf(if0))/float(nhf(ifine))
              do 55 i=ic1,ic2
                igloc(i,iedg,ifine) = if0
                fhloc(i,iedg,ifine) = fh1 + float(i-ic1)*hrat
                jhloc(i,iedg,ifine) = (jyfe(ifine)+1-jyfb(if0))
     &                                 *nhf(if0)+2
   55         continue
            endif
   56     continue
c
c         --- check what sits on south edge of ifine
          iedg = 2
          do 66 ic=ixfb(ifine),ixfe(ifine)
            ic1 = (ic-ixfb(ifine))*nhf(ifine)+2
            ic2 = ic1 + nhf(ifine) - 1
            if0 = numgrd(ic,jyfb(ifine)-1)
            if (if0 .eq. 0) then
              fh1 = float(ic-1)
              hrat = 1./float(nhf(ifine))
              do 64 i=ic1,ic2
                igloc(i,iedg,ifine) = if0
                fhloc(i,iedg,ifine) = fh1 + float(i-ic1)*hrat
                jhloc(i,iedg,ifine) = jyfb(ifine)-1
   64         continue
            else
              fh1 = float((ic-ixfb(if0))*nhf(if0)+1)
              hrat = float(nhf(if0))/float(nhf(ifine))
              do 65 i=ic1,ic2
                igloc(i,iedg,ifine) = if0
                fhloc(i,iedg,ifine) = fh1 + float(i-ic1)*hrat
                jhloc(i,iedg,ifine) = (jyfb(ifine)-jyfb(if0))
     &                                 *nhf(if0)+1
   65         continue
            endif
   66     continue
c
c         --- check what sits on east edge of ifine
          iedg = 3
          do 76 jc=jyfb(ifine),jyfe(ifine)
            jc1 = (jc-jyfb(ifine))*nhf(ifine)+2
            jc2 = jc1 + nhf(ifine) - 1
            if0 = numgrd(ixfe(ifine)+1,jc)
            if (if0 .eq. 0) then
              fh1 = float(jc-1)
              hrat = 1./float(nhf(ifine))
              do 74 j=jc1,jc2
                igloc(j,iedg,ifine) = if0
                fhloc(j,iedg,ifine) = fh1 + float(j-jc1)*hrat
                jhloc(j,iedg,ifine) = ixfe(ifine)+1
   74         continue
            else
              fh1 = float((jc-jyfb(if0))*nhf(if0)+1)
              hrat = float(nhf(if0))/float(nhf(ifine))
              do 75 j=jc1,jc2
                igloc(j,iedg,ifine) = if0
                fhloc(j,iedg,ifine) = fh1 + float(j-jc1)*hrat
                jhloc(j,iedg,ifine) = (ixfe(ifine)+1-ixfb(if0))
     &                                 *nhf(if0)+2
   75         continue
            endif
   76     continue
c
c         --- check what sits on west edge of ifine
          iedg = 4
          do 86 jc=jyfb(ifine),jyfe(ifine)
            jc1 = (jc-jyfb(ifine))*nhf(ifine)+2
            jc2 = jc1 + nhf(ifine) - 1
            if0 = numgrd(ixfb(ifine)-1,jc)
            if (if0 .eq. 0) then
              fh1 = float(jc-1)
              hrat = 1./float(nhf(ifine))
              do 84 j=jc1,jc2
                igloc(j,iedg,ifine) = if0
                fhloc(j,iedg,ifine) = fh1 + float(j-jc1)*hrat
                jhloc(j,iedg,ifine) = ixfb(ifine)-1
   84         continue
            else
              fh1 = float((jc-jyfb(if0))*nhf(if0)+1)
              hrat = float(nhf(if0))/float(nhf(ifine))
              do 85 j=jc1,jc2
                igloc(j,iedg,ifine) = if0
                fhloc(j,iedg,ifine) = fh1 + float(j-jc1)*hrat
                jhloc(j,iedg,ifine) = (ixfb(ifine)-ixfb(if0))
     &                                 *nhf(if0)+1
   85         continue
            endif
   86     continue
c
c  diagnostics--- write out everything for this fine grid
c
          if (idiag .ne. 0) then
            write(iout,'(a,4i10)') 'Fine grid ',ifine, nfx(ifine), 
     &                         nfy(ifine), nfz(ifine)
            do 88 iedg=1,4
              nnn = nfx(ifine)
              if (iedg .gt. 2) nnn = nfy(ifine)
              write(iout,'(a,i3,a)') 'iedg = ',iedg,'  igloc:'
              write(iout,*) (igloc(j,iedg,ifine),j=1,nnn)
              write(iout,'(a)') '            fhloc:'
              write(iout,*) (fhloc(j,iedg,ifine),j=1,nnn)
              write(iout,'(a)') '            jhloc:'
              write(iout,*) (jhloc(j,iedg,ifine),j=1,nnn)
   88       continue
          endif
   90   continue
c
        if (lerr) then
          write(iout,*) ' Stopping due to errors in nest definition.'
          write(*,*) ' Stopping due to errors in nest definition.'
          write(ierrlg,*) 1111,'  Stopping due to errors in nest',
     &                'definition.'
          stop
        endif
c
c   write header info to fine grid average and inst files that
c   fully defines location and extent of each fine grid.
c   also write species simulated (in order).
c
        if (ifavg .gt. 0) then
          write(ifavg) mfavg, msg
          write(ifavg) numfin, navspc
          write(ifavg) (mavspc(l),l=1,navspc)
        endif
c
        if (ifinst .gt. 0) then
          write(ifinst) mfinst, msg
          write(ifinst) numfin, nospec
          write(ifinst)((mspec(i,l),i=1,10),l=1,nospec)
        endif
c
c   loop over fine grids, writing grid definition
c
        do 4881 ifine=1,numfin
          if (ifavg .gt. 0) write(ifavg) ixfb(ifine), jyfb(ifine),
     &                        ixfe(ifine), jyfe(ifine), nhf(ifine),
     &                        nvf(ifine), nfx(ifine), nfy(ifine), 
     &                        ione, ifgptr(ifine), ifglvl(ifine)
          if (ifinst .gt. 0) write(ifinst) ixfb(ifine), jyfb(ifine),
     &                        ixfe(ifine), jyfe(ifine), nhf(ifine),
     &                        nvf(ifine), nfx(ifine), nfy(ifine), 
     &                        nfz(ifine), ifgptr(ifine), ifglvl(ifine)
 4881   continue
c
      endif
c
      if (.not.lchmag) return
c----------------------------
c     read aggregate map
c----------------------------
      kntagg=0
c
  451 continue
c
      do 453 k=1,nzz
        read(iagg,1001,end=700)ivtime,ivday
        if (idiag .ne. 0) then
          write(iout,*) ' nzz, k, ivtime, ivday: ',nzz,k,ivtime,ivday
        endif
c
c  check here if next time is after current time.  i would just as
c  soon keep using the previous data rather than have to read in 
c  the same data hour after hour.  anyway i added these checks so
c  that if there is no new data, just go back to caller without
c  doing anything.  for this to work in general, the section below
c  that checks for synchronization was rewritten.  in 
c  particular, rewinding the file and rereading it would not work
c  anymore since i've stuck the fine grid definitions at the top
c  of the file.  i also changed the checks of time equal to 24
c  to checks of time equal 2400 since that seems to be the form
c  of time that is read in.  (tcm, 10/16/90)
c
        if (idiag .ne. 0) then
          write(iout,*) 'ivday, ndate = ',ivday, ndate
          write(iout,*) 'ivtime, jtime = ',ivtime, jtime
        endif
        if (iaglod .ne. 0) then
          ndays = intrvl(ivday,ndate)
          itchk = ivtime + 2400*ndays
          if (itchk .gt. jtime) then
            backspace iagg
            return
          endif
        endif
        if (idiag .ne. 0) then
          write(iout,*) 'searching for current date and time . . .'
        endif
c
        do 450 j=nyy,1,-1
          read(iagg,1002) (ivmm(i,j,k),i=1,nxx)
  450   continue
c
  453 continue
c
      ndays = intrvl(ivday,ndate)
      itchk = ivtime + 2400*ndays
      if(itchk .gt. jtime)then
        if(kntagg .le. 4)then
          rewind iagg
          kntagg=kntagg+1
          go to 451
        else
          write(iout,*)'error in aggregation file: cannot find data',
     &            ' for ndate;time;ivday;ivtime: ',ndate,time,
     &             ivday,ivtime
          write(*,*) 'error in aggregation file: cannot find data',
     &            ' for ndate;time;ivday;ivtime: ',ndate,time, 
     &             ivday,ivtime
          write(ierrlg,*) 1112,' error in aggregation file: cannot',
     &            ' find data for ndate;time;ivday;ivtime: ',ndate,time,
     &             ivday,ivtime
          stop
        endif
      endif
      iaglod = 1
 1001 format(i4,1x,i6,69a1)
 1002 format(80i1)
c-----------------------------------------------------------------------
c     loop through variable mesh map, identifying beginning and ending
c     indices of each aggregate cell
c-----------------------------------------------------------------------
      ncm=0
      do 502 k=1,nzz
        do 501 j=1,nyy
          do 500 i=1,nxx
            if(ivmm(i,j,k).le.0)then
              go to 500
            elseif(ivmm(i,j,k) .eq. 9)then
              ncm=ncm+1
              icmb(ncm)=i
              jcmb(ncm)=j
              kcmb(ncm)=k
              icme(ncm)=i
              jcme(ncm)=j
              kcme(ncm)=k
              ivmm(i,j,k)=-abs(ivmm(i,j,k))
            else
              ncm=ncm+1
              icmb(ncm)=i
              jcmb(ncm)=j
              kcmb(ncm)=k
              call endind(ncm,i,j,k,nxx,nyy,nzz,ivmm,mxx,mxy,mxz
     &,               icme,jcme,kcme,maxagg)
            endif
c----------------------------------------------
c    determine ending indices of aggregate cell
c----------------------------------------------
c
  500     continue
  501   continue
  502 continue
c
c-------------------------------------------------------------------------
c     now, double check for vertical aggregate-cell alignment errors.  
c     these could occur through user input errors and not be detected
c     by the above procedure.  if alignment errors exist, the beginning
c     indices of the erroneous aggregate cells will be identified.
c-------------------------------------------------------------------------
      icheck=0
      ivrfy=0
      do 660 n=1,ncm
        icnt = icme(n) - icmb(n)
        jcnt = jcme(n) - jcmb(n)
        kcnt = kcme(n) - kcmb(n)
        do 652 k=kcmb(n),kcme(n)
          do 651 j=jcmb(n),jcme(n)
            do 650 i=icmb(n),icme(n)
              if(abs(ivmm(i,j,k)).ne.
     &                    abs(ivmm(icmb(n),jcmb(n),kcmb(n)))) then
                write(iout,2000) n,icmb(n),jcmb(n),kcmb(n)
                write(*,2000) n,icmb(n),jcmb(n),kcmb(n)
                write(ierrlg,2010) 261,n,icmb(n),jcmb(n),kcmb(n)
                icheck=1
                go to 660
              endif
c
c          check that aggregate region does not overlap fine grid
c
              if (mfhrz(j)(i:i) .ne. '0') then
                if (icnt .gt. 0 .or. jcnt .gt. 0 .or.
     &              kcnt .gt. 0) then
                  icheck = 1
                  write(iout,2080) n,icmb(n),jcmb(n),kcmb(n),i,j,k
                  write(*,2080) n,icmb(n),jcmb(n),kcmb(n),i,j,k
               write(ierrlg,2081) 262,n,icmb(n),jcmb(n),kcmb(n),i,j,k
                endif
              endif
  650       continue
  651     continue
  652   continue
        ivrfy=ivrfy+1
  660 continue
      write(iout,2222) ivrfy,ncm
      write(*,2222) ivrfy,ncm 
      write(ierrlg,2223) 1114,ivrfy,ncm 
 2222 format(' from subroutine rvmmap (aggregate cell stuff),'
     &/      1x,i6,' aggregate cells verify out of ',i6,' possible'/) 
 2223 format(i4,1x,' from subroutine rmmap (aggregate cell stuff),'
     &/      1x,i6,' aggregate cells verigy out of ',i6,' possible'/)
      if(icheck.eq.1) stop
      go to 451
c
c  come here if end of file encountered
c  disable chem aggregation and inform user
c
  700 lchmag = .false.
      write(iout,2090) 
      write(*,2090)
      write(ierrlg,2091) 263
      return
c
 1050 format(a120) 
 2000 format(' aggregate cell ',i4,' erroneously defined. abort status.'
     &/      '  beginning indices are (',i2,',',i2,',',i2,')')
 2010 format(i3,1x,' aggregate cell ',i4,' erroneously difined. abort',
     &       ' status. beginning indices are (',i2,',',i2,',',i2,')')
 2001 format(' aggregate cell ',i4,' verifies. (i,j,k) index range='
     &,      ' (',i2,'-',i2,',',i2,'-',i2,',',i2,'-',i2,')')
 2050 format(' error - fine grid cell on edge of region at cell ',
     &       i2,',',i2,'.  aborting . . .',/)
 2080 format(' aggregate cell ',i4,' erroneously defined. ',
     &       ' abort status.',/,
     &       '  beginning indices are (',i2,',',i2,',',i2,')',/,
     &       '  overlaps fine grid at cell (',i2,',',i2,',',i2,')')
 2081 format(i3,1x,' aggregate cell ',i4,' erroneously defined. ',
     &       ' abort status.',/,
     &       '  beginning indices are (',i2,',',i2,',',i2,')',/,
     &       '  overlaps fine grid at cell (',i2,',',i2,',',i2,')')
 2090 format (' EOF on aggregation file.',/,
     &        ' Chemistry aggregation disabled.',/)
 2091 format (i3,1x,' EOF on aggregation file.',/,
     &        ' Chemistry aggregation disabled.',/)
c
c     close (iin)
      end
