c
c  common /scratch/ contains scratch vector from which memory
c  can be allocated at beginning of run
c
c$Log: scratch.cmd,v $
c  Revision 1.2  1998/11/04  02:12:53  tcm
c  Increased maximum allowed storage.
c
c  Revision 1.1  94/07/26  17:09:54  tcm
c  Initial revision
c  
c
      parameter (mxscrt=1200000)
      common /scratch/ xscrtc(mxscrt)
c
c  common /scrptr/ contains pointers to the beginning of arrays
c  in the overall scratch array.  The value of each pointer is
c  equal to the subscript of the array element in xscrtc that
c  is the same as the first element of the array to be allocated.
c  If a new array is needed from the scracth array, a pointer
c  must be added to this common block and the routine alloc must
c  be called with the appropriate size.  scrptr also includes the
c  original size of the scratch vector and the amount left.
c
      common /scrptr/ nscrtc, nsclft, kcncf(maxfin), kcoldf(maxfin),
     &                kavcnf(maxfin), ko3av, ko3avf(maxfin), 
     &                ko3tm, ko3pt, ko3tmf(maxfin), ko3ptf(maxfin)
c      
c      nscrtc  - total size of xscrtc
c
c      nsclft  - number of elements of xscrtc still available
c
c      kcncf   - pointer to beginning of cncf array for each fine grid,
c                array of fine grid concentrations, 
c                each dimensioned x,y,species,z  (NOT USED)
c
c      kcoldf  - pointer to beginning of coldf array for each fine grid,
c                array of fine grid surface concentrations at previous
c                step, each dimensioned x,y,species  (NOT USED)
c
c      kavcnf  - pointer to beginning of avconf array for each fine 
c                grid, array of fine grid average concentrations,
c                each dimensioned x,y,species  (NOT USED)
c
c      ko3av   - pointer to beginning of o3avg array for maintaining
c                running average of o3 concs on coarse grid
c                dimensioned x,y,intervals
c
c      ko3avf  - pointer to beginning of o3avgf array for maintaining
c                running average of o3 concs on fine grids
c                each dimensioned x,y,intervals
c
c      ko3tm   - pointer to tmo3 array, times of o3 concs for 
c                running hourly average
c
c      ko3pt   - pointer to array of pointers to locations of concs
c                to be used in running o3 averages
c
c      ko3tmf  - pointer to tmo3f array, times of o3 concs for 
c                running hourly average for fine grids
c
c      ko3ptf  - pointer to array of pointers to locations of concs
c                to be used in running o3 averages for fine grids
c
c      kvmt    - pointer to array of gridded vmt on coarse grid
c
c      kfvmt   - array of pointers to arrays of gridded vmt for 
c                fine grids
c
c
c
c  Example of the way memory might be allocated:
c
c  Suppose 21 species, 8 layers, fine grid 1 is 20 by 20
c                                fine grid 2 is 15 by 15:
c
c  Number of elements in cncf for fine grid 1 is 20*20*21*8 = 67200
c                             for fine grid 2 is 15*15*21*8 = 37800
c  Number of elements in coldf for fine grid 1 is 20*20*21 = 8400
c                              for fine grid 2 is 15*15*21 = 4725
c  Number of elements in avconf for fine grid 1 is 20*20*21 = 8400
c                               for fine grid 2 is 15*15*21 = 4725
c
c    xscrtc(1) = cncf(1,1,1,1) for fine grid 1, kcncf(1) = 1
c    xscrtc(2) = cncf(2,1,1,1) for fine grid 1, kcncf(1) = 1
c    xscrtc(3) = cncf(3,1,1,1) for fine grid 1, kcncf(1) = 1
c        .
c        .
c        .
c    xscrtc(67201) = cncf(1,1,1,1) for fine grid 2, kcncf(2) = 67201
c    xscrtc(67202) = cncf(2,1,1,1) for fine grid 2, kcncf(2) = 67201
c    xscrtc(67203) = cncf(3,1,1,1) for fine grid 2, kcncf(2) = 67201
c        .
c        .
c        .
c    xscrtc(105001) = coldf(1,1,1) for fine grid 1, kcoldf(1) = 105001
c    xscrtc(105002) = coldf(2,1,1) for fine grid 1, kcoldf(1) = 105001
c    xscrtc(105003) = coldf(3,1,1) for fine grid 1, kcoldf(1) = 105001
c        .
c        .
c        .
c    xscrtc(113401) = coldf(1,1,1) for fine grid 2, kcoldf(2) = 113401
c    xscrtc(113402) = coldf(2,1,1) for fine grid 2, kcoldf(2) = 113401
c    xscrtc(113403) = coldf(3,1,1) for fine grid 2, kcoldf(2) = 113401
c        .
c        .
c        .
c    xscrtc(118126) = avconf(1,1,1) for fine grid 1, kavcnf(1) = 118126
c    xscrtc(118127) = avconf(2,1,1) for fine grid 1, kavcnf(1) = 118126
c    xscrtc(118128) = avconf(3,1,1) for fine grid 1, kavcnf(1) = 118126
c        .
c        .
c        .
c    xscrtc(126526) = avconf(1,1,1) for fine grid 2, kavcnf(2) = 126526
c    xscrtc(126527) = avconf(2,1,1) for fine grid 2, kavcnf(2) = 126526
c    xscrtc(126528) = avconf(3,1,1) for fine grid 2, kavcnf(2) = 126526
c        .
c        .
c        .
