      subroutine setup(time,ndate,tend,idaynf,totime,tdatnx,
     &                 lrstrt,idebug)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: setup.f,v $
c  Revision 1.22  2000/07/03  21:42:37  tcm
c  Increased line length to 200 for reading albedo input data.
c
c  Revision 1.21  1999/12/21  19:28:28  tcm
c  Renamed com3 include file to comd3.
c
c  Revision 1.20  1999/07/07  16:36:48  tcm
c  Added input and output files for pa/irr treatment.
c
c  Revision 1.19  1999/05/14  17:09:26  gem
c  changed read of rates for version 1.17
c
c  Revision 1.18  1999/05/12  16:43:37  dfg
c  Added initialization of radical array to 1.0E-7.
c
c  Revision 1.17  1999/05/03  22:02:05  gem
c  fixed calculation of fine grid origin (xf,yf) before call to hdrchk
c
c  Revision 1.16  1999/05/03  18:29:15  gem
c  added capability to input and read fine grid data files for all met fields
c
c Revision 1.1  1999/05/03  17:55:36  gem
c Initial revision
c
c  Revision 1.15  1999/04/08  23:55:16  tcm
c  Read flag to determine if fast or standard chemistry solver should be used.
c
c  Revision 1.14  1998/11/20  23:29:13  tcm
c  Corrected spelling of iyfb to jyfb
c
c  Revision 1.13  1998/11/20  23:01:28  tcm
c  Corrected error messages and formats on lines 221, 1081, 1095, 1137,
c  1194, and 1200
c
c  Revision 1.12  1998/11/20  01:30:49  tcm
c  Read name and open error log file prior to other files.
c
c  Revision 1.11  1998/11/16  17:22:17  ara
c  Made small modifications to error messages format
c
c  Revision 1.10  1998/11/06  22:57:37  ara
c  Elimitated errors directed to file number 66
c  and made sure references to unit 8 (diag file) were always
c  included
c
c  Revision 1.9  1998/11/06  18:01:51  ara
c  Included errlg.cmd to direct logs to file ierrlg
c  Directed error messages in several lines to diag file, screen
c  and ierrlg file (this last one with the unique error code)
c  Total messages of error reviewed = 30
c  See associated database table for more details
c
c  Revision 1.8  1998/11/05  19:08:34  tcm
c  Added reads of file headers for met files
c  Eliminated references to receptors
c  Eliminated internal conversion of cell location to lat/lon.  Therefore
c  this conversion must now be done in processor that creates terrain file.
c
c  Revision 1.7  1998/11/04  01:53:25  tcm
c  Lengthened filename string.
c  Changed from max average o3 to step-by-step o3 output.
c  Fixed initialization of totime.
c  Fixed index in calculation of point source emissions totals.
c
c  Revision 1.6  1996/06/14  23:03:16  tcm
c  modified by gem to allow larger grid in albedo read
c  modified by hpd to read additional photolysis rate for
c  isop update
c  option to open pig dump file added by gem
c
c  Revision 1.5  1995/04/26  00:45:25  tcm
c  	added check of no. of levels against max array dimension.  moved initialization of
c  	rolling average o3 species above write of average file headers.  added reads of fine
c  	grid definitions.  removed reads of particulate bins (these are now in data statements
c  	in aqbdat).  read fine grid height/pressure files in case vertical nesting is
c  	being done.  added check that all average species have valid pointers.
c
c  Revision 1.4  94/10/18  17:48:15  tcm
c  Added print of plume-in-grid parameters to diag and out files
c  
c  Revision 1.3  94/07/27  09:22:06  tcm
c  Added code to set up for calculation of rolling o3 averages
c    - Check for species MXO3 in list of average species
c    - Add species TMO3 and set lmxo3 flag
c    - Allocate space for rolling average arrays from scratch array via
c      calls to alloc
c  Cleaned up some if statements and print of simulation options.
c  
c  Revision 1.2  94/02/10  21:00:08  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c
c
c...    this subroutine sets up input for model run
c
c    -- modified formatted read of albedo codes to allow larger 
c       grid size (gem, 4/1/96)
c
c    -- modified to set default value for difcof to 0.25/sqrt(2)
c       (tcm, 11/21/90)
c
c    -- modified to remove smolarkiewicz flag (shasta no longer
c       available) and replace with deposition flag.  maximum
c       advection step (dtstep) added. 
c
c    -- modified to allow for parameter inputs for the plume-in-grid
c       submodel (cae, 11/20/91)
c
c    -- modified to reorder read of coarse grid surface file and
c       initialization of flux arrays so that they will be properly
c       set when agg file is not being used. (tcm, 3/9/93)
c
c    -- Added initialization of radical for coarse grid for the
c       automatically generated chemistry solver.
c       (dfg, 05/11/99)
c
c
c...    files read from:
c...            7:              input file opened in 'aaamain.f'
c...            edf:            point source emission      (moles/hr)
c...            aem1:           area source emission for weekdays
c...            isurf:          surface cover type
c...            icbc:           initial concentration and boundary 
c...                            conditions (this variable is reused
c...                            for each of 3 files:  initial 
c...                            conditions, lateral boundaries, and
c...                            aloft concentrations)
c...            iterr:          terrain file
c...    files write to:
c...            stdout;     8;     66;
c...    routines called:
c...            update;     diagn;     (rvmmap: when lchmag = .true.);
c...            chread;     gunit;     (restrt: when lrstrt = .true.);
c...    called by:
c...            aaamain;
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'shrfin.cmd'
      include 'com1f.cmd'
      include 'comuam.cmd'
      include 'aerosl.cmd'
      include 'oneday.cmd'
      include 'latlon.cmd'
      include 'comchem.cmd'
      include 'comd3.cmd'
      include 'efcon.cmd'
      include 'ibccon.cmd'
      include 'hedmsg.cmd'
      include 'photrk.cmd'
      include 'adunit.cmd'
      include 'ppmcon.cmd'
      include 'advec.cmd'
      include 'iemcon.cmd'
      include 'ctlflg.cmd'
      include 'stack.cmd'
      include 'pigpen.cmd'
      include 'pigdmp.cmd'
      include 'idvcon.cmd'
      include 'scratch.cmd'
      include 'avo3con.cmd'
      include 'errlog.cmd' 
      include 'irmb.cmd'
c
      logical lerr
      dimension vcheck(5)
c
c
c
      integer aem
      logical lrstrt,lsuprs,liccon
c
      character*132 fn, fn2, fnerr
      character*10 mdum
      character*10 nams
      character*10 ifile
      character*10 ifili
      character*10 ifilo
      character*10 mwnd, mzp, mterr, mkv, mhum, mrain, mcld, mtmpr,
     &             msrf
      character*1 mblank
      integer utmzon
c
c     logical flag lsuprs suppress point source info on diag output file
c
      data lsuprs/.true./
      data liccon/.false./
      data cloud /maxagg*0.0/, rain/mxset*0.0/
      data ifile /'AVERAGE   '/, nuno /1/
      data ifili /'INSTANT   '/
      data ifilo /'O3STEPS   '/
      data mblank /' '/
      data izero /0/, fzero /0./
      data mwnd  /'WIND      '/
      data mzp   /'HEIGHT    '/
      data mterr /'TERRAIN   '/
      data mtmpr /'TEMPERATUR'/
      data mhum  /'H2O       '/
      data mkv   /'VDIFFUSION'/
      data mrain /'RAIN      '/
      data mcld  /'CLOUD     '/
      data msrf  /'SURFACE   '/
c
c  bit six of idebug on (i.e., the thirty-twos location) will turn
c  on debug writes in setup.
c
      idiag = and(idebug,32)
c
c  first action:  open error log file
c
      read(7,7001) ierrlg, fnerr
      open(ierrlg,file=fnerr,form='formatted')
c
      read(7,7001) iout,fn
      if (iout .ne. 66) then
        write(*,*) 'error: out file must be opened on unit 66'
        write (ierrlg,'(i4,1x,a,a)') 1045, 'error: out file must be', 
     &                                    ' opened on unit 66' 
        stop
      endif
      open(66,file=fn,form='formatted')
c***
      read(7,7001) iout,fn
      if (iout .ne. 8) then
        write(*,*) 'error: diag file must be opened on unit 8'
        write (8,*) 'error: diag file must be opened on unit 8'
        write (ierrlg,'(i4,1x,a,a)') 1046, 'error: diag file must be',   
     &                                    ' opened on unit 8' 
        stop
      endif
      open(8,file=fn,form='formatted')
      read(7,7001) iuavg,fn
      if (iuavg .gt. 0) then
        open(iuavg,file=fn,form='unformatted')
        write(8,7020) iuavg, fn
 7020   format(' open unit=',i2,'  file=',a,' for average conc output')
        read(7,7025) navspc, (mavspc(l),l=1,6)
        if (navspc.gt.6) read(7,70251) (mavspc(l),l=7,navspc)
 7025   format(i2,6a10)
70251   format(2x,6a10)
        do 15 l=1,navspc
          call lftpak(mavspc(l))
   15   continue
      else
        write(8,'(a)') ' no unit opened for average conc output'
      endif
      read(7,7001) iout,fn
      if (iout .ne. 51) then
        write(*,*) 'error: inst conc file must be opened on unit 51'
        write (8,*) 'error: inst conc file must be opened on unit 51'
        write (ierrlg,'(i4,1x,a,a)') 1047, 'error: inst conc file', 
     &                         ' must be opened on unit 51'
        stop
      endif
      open(51,file=fn,form='unformatted')
      write(8,7021) fn
 7021 format(' open unit=51= file=',a,' for dump (inst) conc out')
c
c   open fine grid output concentration files (standard units are
c   62 for average concs, 52 for inst concs)
c
      read(7,7001) ifavg,fn
      if (ifavg .gt. 0) then
        open(ifavg,file=fn,form='unformatted')
        write(8,7030) ifavg, fn
 7030   format(' open unit=',i2,' file=',a,' for fine grid ',
     &         'average conc output')
      else
        write(8,7031) 
 7031   format(' no file opened  for fine grid ',
     &         'average conc output')
      endif
      read(7,7001) ifinst,fn
      if (ifinst .gt. 0) then
        open(ifinst,file=fn,form='unformatted')
        write(8,7035) ifinst, fn
 7035   format(' open unit=',i2,'  file=',a,' for fine grid ',
     &         'inst conc output')
      else
        write(8,7036) 
 7036   format(' no file opened for fine grid ',
     &         'inst conc output')
      endif
c
c  open pig output restart files
c
      read(7,7001) iupigo,fn
      if (iupigo .gt. 0) then
        open(iupigo,file=fn,form='unformatted')
        write(8,7038) iupigo, fn
 7038   format(' open unit=',i2,'  file=',a,' for pig output restart',
     &         ' file')
      else
        write(8,'(a)') ' no file opened for pig output restart file'
      endif
      read(7,7001) iupign,fn
      if (iupigo .gt. 0) then
        open(iupign,file=fn,form='unformatted')
        write(8,7038) iupign, fn
      else
        write(8,'(a)') ' no file opened for pig output restart file'
      endif
c
c  open step by step ozone output file
c
      read(7,7001) iuo3,fn
      if (iuo3 .gt. 0) then
        open(iuo3,file=fn,form='unformatted')
        write(8,7044) iuo3, fn
 7044   format(' open unit=',i2,'  file=',a,' for step by step o3',
     &         ' file')
      else
        write(8,'(a)') ' no file opened for step by step o3 file'
      endif
c
czswa--------------------------------------------------------------
c
c  open out files names from param (unit 7) input file
c
      read(7,7001) irrcrs,fn
      if (irrcrs .ne. 03) then
        if (irrcrs .ne. 0) then
          write(*,*) 'error: out file must be opened on unit 03'
          stop
        endif
      endif
      if (irrcrs .gt. 0) open(irrcrs,file=fn,form='unformatted')
c
      read(7,7001) irrfine,fn
      if (irrfine .ne. 04) then
        if (irrfine .ne. 0) then
          write(*,*) 'error: out file must be opened on unit 04'
          stop
        endif
      endif
      if (irrfine .gt. 0) open(irrfine,file=fn,form='unformatted')
c
czswa--------------------------------------------------------------
c
      if (idiag .ne. 0) then
        write(*,3001) time,ndate
 3001   format(' enter setup  ',f10.1,i10)
      endif
      read(7,10) msg
   10 format(a80)
      write(8,20) msg
      write(66,20) msg
   20 format(1h1,t25,a80//)
c
c  read start/end time/date, data/print/time/averaging steps
c  then convert dates to julian
c  dates must be julian of the form YYDDD (1951 - 2050)
c             or calendar of the form YYYYMMDD (any year)
c             or calendar of the form YYMMDD (1910 - 1999)
c
      read(7,*) tstr,idystr,tend,idaynf,dtdata,dtprnt,dtstep,dtavg
      if (idystr .gt. 99999) then
        call caltoj(idystr,ijul)
        idystr = ijul
      endif
      if (idaynf .gt. 99999) then
        call caltoj(idaynf,ijul)
        idaynf = ijul
      endif
      if (dtavg .le. 0.) dtavg = 10.
      read(7,31) dlong,dlat,nox,noy,nlayer,nospec
      datadt=0.6*dtdata
   31 format(2f7.0,4i5)
      nospc1=nospec
c
c     make sure nox<=mxx and noy<=mxy otherwise errors will result
c
      if (nox.gt.mxx .or. noy.gt.mxy) then
        write(8,33) nox,noy
        write(*,33) nox,noy
        write (ierrlg,344) 1001, nox,noy
   33   format(' $$error parameters mxx,mxy not consistent with input'
     &  ,' nox,noy ',2i5)
  344   format (i4,1x,' $$error parameters mxx, mxy not consistent',
     &                           ' with input nox, noy',2i5)
        stop1
      endif
c
c     make sure nlayer < mxz otherwise errors will result
c
      if (nlayer.gt.mxz) then
        write(8,34) mxz, nlayer
        write(*,34) mxz, nlayer
        write (ierrlg,345) 1002,mxz,nlayer 
   34   format(' $$error parameter mxz not consistent with input'
     &  ,' nlayer ',2i5)
  345   format (i4,1x,' $$error parameter mxz not consistent with', 
     &                  ' input nlayer ',2i5)
        stop2
      endif
c
c   check for special output file for O3 and if it's there,
c   set flag
c
      lmxo3 = .false.
      if (iuo3 .ne. 0) lmxo3 = .true.
c
c   read in fine grid definitions
c
      read(7,'(i10)') numfin
      if (numfin .gt. maxfin) then
        write(8,'(a,i4,/,a,i4)')
     &           ' Max number of fine grids exceeded: ',numfin,
     &           ' Max is ',maxfin
        write(*,'(a,i4,/,a,i4)')
     &           ' Max number of fine grids exceeded: ',numfin,
     &           ' Max is ',maxfin
        write (ierrlg,'(i4,1x,a,i4,/,a,i4)') 1048, 
     &            ' Max number of fine grids exceeded: ',numfin, 
     &            ' Max is ', maxfin 
        stop
      endif
      do 35 ifine=1,numfin
        read(7,'(6i5)') ixfb(ifine), jyfb(ifine), 
     &                  ixfe(ifine), jyfe(ifine), 
     &                  nhf(ifine), nfz(ifine)
	if ( nfz(ifine) .lt. nlayer ) then
	  write(8,721) nfz,nlayer
          write (*,721) nfz,nlayer
          write (ierrlg,821) 1003,nfz,nlayer 
  721     format(' error in vertical nesting',/,
     &           ' number of fine grid vertical layers ',i3,
     &           'is less than coarse grid vertical layers ',i3 )
  821     format (i4,1x,' error in vertical nesting',/,
     &           ' number of fine grid vertical layers ', i3, 
     &           'is less than coarse grid vertical layers ',i3 )
	  stop
	endif
   35 continue
c
      read(7,23) lwet,lrstrt,ldepn,lpig,lcart,linert
c  23 format(8l10)
   23 format(6l10)
      read(7,23) larea, lpts, l1eday, lchfast
      read(7,2301) l1day,lchmag,ismth,lstagr
 2301 format(2l10,i10,l10)
c
c  allowable values of ismth are 1, 2 or 3.  default is 2.
c
      if (ismth .lt. 1 .or. ismth .gt. 3) ismth = 2
c
c  read plume-in-grid parameters, and initialize the submodel
c
      if (lpig) then
        read(7,2302) nrclls, wdump, xcmbn, delpuf, delout, mxage
 2302   format(i2,2f10.2,3f10.0)
        call piginit(tstr,idystr)
      endif
c
c  horizontal diffusion coefficient parameters 
c
      read(7,24) difcof,difmax,difmin,chmint
   24 format(4f10.0)
c
c     if not entered use default of 0.1767767 (=0.25/sqrt(2)) 
c     for difcof
c
      if (difcof.le.0.0) difcof=0.1767767
      if (difmax.le.0.0) difmax=6.0
c     if (difmin.le.0.0) difmin=0.0006
c
      read(7,50) wlong,elong,slat,nlat,tz,utmzon
      read(7,118)xorg,yorg
  118 format(2f10.0)
c
      write(8,40) tstr,idystr,tend,idaynf,dtdata,dtprnt,dtstep,dtavg,
     & dlong,dlat,nox,noy,nlayer,nospec
      write(66,40) tstr,idystr,tend,idaynf,dtdata,dtprnt,dtstep,dtavg,
     & dlong,dlat,nox,noy,nlayer,nospec
   40 format(/,2x,'Simulation start time/date:    ',f10.0,2x,i10,/,
     &       2x,'Simulation end time/date:      ',f10.0,2x,i10,/,
     &       2x,'Data input interval (min):               ',f10.0,/,
     &       2x,'Output interval (min):                   ',f10.0,/,
     &       2x,'Maximum coarse grid time step (min):     ',f10.0,/,
     &       2x,'Minimum hrly avg ozone interval (min):   ',f10.0,//,
     &       2x,'Longitude grid spacing:                  ',f10.3,/,
     &       2x,'Latitude grid spacing:                   ',f10.3,/,
     &       2x,'Grid dimensions (x-, y-, z-cells):  ',3i5,/,
     &       2x,'Number of species:                       ',i10,/)
      write(8,26) lwet,lrstrt,ldepn,lpig,lcart,linert,
     &            larea, lpts, l1eday, l1day,
     &            lchmag,ismth,lstagr,chmint
      write(66,26) lwet,lrstrt,ldepn,lpig,lcart,linert,
     &            larea, lpts, l1eday, l1day,
     &            lchmag,ismth,lstagr,chmint
   26 format(' wet deposition option = ',l10,/,
     &       '       restart option  = ',l10,/,
     &       ' dry deposition option = ',l10,/,
     &       '           pig option  = ',l10,/,
     &       '          cart option  = ',l10,/,
     &       '         inert option  = ',l10,/,
     &       '   area source option  = ',l10,/,
     &       '  point source option  = ',l10,/,
     &       ' one day emiss option  = ',l10,/,
     &       ' one day inputs option = ',l10,/,
     &       ' aggregate-cell option = ',l10,/,
     &       ' dissaggregation method= ',i10,/,
     &       ' staggered u-v grid op = ',l10,/,
     &       ' chm time step interval= ',f10.1)
      if (lchfast) then
        write(*,*) ' '
        write(*,*) '*****  FAST CHEMISTRY SOLVER ENABLED. *****'
        write(*,*) ' '
        write(66,*) ' '
        write(66,*) '*****  FAST CHEMISTRY SOLVER ENABLED. *****'
        write(66,*) ' '
        write(8,*) ' '
        write(8,*) '*****  FAST CHEMISTRY SOLVER ENABLED. *****'
        write(8,*) ' '
      endif
   50 format(4f10.2,f10.0,i10)
      write(8,55) wlong,elong,slat,nlat,tz
   55 format(5x,'wlong=',f8.2,5x,'elong=',f8.2,5x,'slat=',
     &  f8.2,5x,'nlat=',f8.2,5x,'tzone=',f4.1)
      read(7,60) pkrchm,crchem,lchem
   60 format(2e10.2,l10)
      write(8,61) pkrchm,crchem,lchem
   61 format(5x,'pkrchm=',e12.4,10x,'crchem=',e12.4,/
     &,/,5x,'chemistry option=',l10,' true=oxidant chemistry, false'
     &,'=linear so2/so4 chemistry (nospec=1)')
      write(8,69)
   69 format('  advection/diffusion is done using smolarkiewicz')
      if (.not.lcart) write(8,168)
      if (lcart) write(8,170)
  168 format('  long/latitude coordinateosystem')
  170 format('  cartesian coordinate system; dlong/dlat  are in km')
      write(8,71) difcof,difmax,difmin
   71 format(' horizontal diffusion parameter difcof= ',f15.1
     &,' where kh = difcof * deltay**2 * def ',/
     &,' maximum and minimum diffusivity = ',2f15.5,' km2/min')
      if (lpig) then
        write(8,711) nrclls, wdump, xcmbn, delpuf, delout, mxage
        write(66,711) nrclls, wdump, xcmbn, delpuf, delout, mxage
  711   format(' Plume-in-Grid parameters:',/,
     &         '    Number of skins:                 ',i2,/,
     &         '    Maximum ratio of plume to grid:  ',f10.2,/,
     &         '    Plume combination distance (km): ',f10.2,/,
     &         '    Puff release interval (min):     ',f10.0,/,
     &         '    Puff print interval (min):       ',f10.0,/,
     &         '    Maximum puff age (hours):        ',f10.0)
      endif
c
c  removed reads of particulate bins, default values in data 
c  statements in aqbdat
c
c     read(7,774)(rsize(iptcl),iptcl=1,nbin)
c     read(7,774)(rmass1(iptcl),iptcl=1,nbin)
c     read(7,774)(rmass2(iptcl),iptcl=1,nbin)
c     read(7,774)(rmass3(iptcl),iptcl=1,nbin)
c 774 format(8f10.3)
c
c read in unit numbers and file names for input and output
c open files
c read headers for files with headers
c
      read(7,7001) irrin,fn
      if (irrin .ne. 02) then
        if (irrin .ne. 0) then
          write(*,*) 'error: in file must be opened on unit 02'
          stop
        endif
      else
        write(*,'(a,i2,3a)') ' open unit = ', irrin, ' file = ',fn,
     &            ' for irr control file.'
      endif
      if (irrin .gt. 0) then
        open(irrin,file=fn,form='formatted',status='old')
      endif
c
      read(7,7001) jchem,fn
 7001 format(i2,a)
      open(jchem,file=fn,form='formatted',status='old')
      write(8,7002) jchem,fn
 7002 format(' open unit=',i2,'= file=',a,' for chem file')
c
      read(7,7001) mdf,fn
      open(mdf,file=fn,form='unformatted',status='old')
      write(8,7003) mdf,fn
 7003 format(' open unit=',i2,'= file=',a,' for mdf file')
      call hdrchk(mdf, mwnd, xorg, yorg, nox, noy, nlayer)
c
      read(7,7001) mixht,fn
      open(mixht,file=fn,form='unformatted',status='old')
      write(8,7004) mixht,fn
 7004 format(' open unit=',i2,'= file=',a,' for mixht file')
      call hdrchk(mixht, mzp, xorg, yorg, nox, noy, nlayer)
c
      read(7,7001) itemp,fn
      open(itemp,file=fn,form='unformatted',status='old')
      write(8,7005) itemp,fn
 7005 format(' open unit=',i2,'= file=',a,' for temper file')
      call hdrchk(itemp, mtmpr, xorg, yorg, nox, noy, nlayer)
c
      read(7,7001) iuh2o,fn
      open(iuh2o,file=fn,form='unformatted',status='old')
      write(8,7006) iuh2o,fn
 7006 format(' open unit=',i2,'= file=',a,' for water file')
      call hdrchk(iuh2o, mhum, xorg, yorg, nox, noy, nlayer)
c
      read(7,7001) isurf,fn
      open(isurf,file=fn,form='unformatted',status='old')
      write(8,7007) isurf,fn
 7007 format(' open unit=',i2,'= file=',a,' for file surf')
      call hdrchk(isurf, msrf, xorg, yorg, nox, noy, nuno)
c
      read(7,7001) edf,fn
      open(edf,file=fn,form='unformatted',status='old')
      write(8,7008) edf,fn
 7008 format(' open unit=',i2,'= file=',a,' for edf file')
c
      read(7,7001) aem1,fn
      open(aem1,file=fn,form='unformatted',status='old')
      write(8,7010) aem1,fn
 7010 format(' open unit=',i2,'= file=',a,' for aem weekday file')
      read(7,7001) aem2,fn
      if (aem2.gt.0) then
        open(aem2,file=fn,form='unformatted',status='old')
        write(8,7011) aem2,fn
 7011   format(' open unit=',i2,'= file=',a,' for aem saturday file')
      else
        aem2=aem1
      endif
      read(7,7001) aem3,fn
      if (aem3.gt.0) then
        open(aem3,file=fn,form='unformatted',status='old')
        write(8,7012) aem3,fn
 7012   format(' open unit=',i2,'= file=',a,'for aem sunday file')
      else
        aem3=aem1
      endif
c
c  open initial concentration file
c
      read(7,7001) icbc,fn
      if (icbc .gt. 0) then
        open(icbc,file=fn,form='unformatted',status='old')
        write(8,7013) icbc,fn
 7013   format(' open unit=',i2,'= file=',a,'for initial conditions')
      endif
c
      read(7,7001) icloud,fn
      if (icloud.gt.0) then
        open(icloud,file=fn,form='unformatted',status='old') 
        write(8,7014) icloud,fn
 7014   format(' open unit=',i2,'= file=',a,' for cloud file')
        call hdrchk(icloud, mcld, xorg, yorg, nox, noy, nlayer)
      endif
c
      read(7,7001) irain,fn
      if (irain.gt.0) then
        open(irain,file=fn,form='unformatted',status='old')
        write(8,7015) irain,fn
 7015   format(' open unit=',i2,'= file=',a,' for rain file')
        call hdrchk(irain, mrain, xorg, yorg, nox, noy, nlayer)
      endif
c
      read(7,7001) iterr,fn
      open(iterr,file=fn,form='unformatted',status='old')
      write(8,7016) iterr,fn
 7016 format(' open unit=',i2,' file=',a,' for terrain file')
      call hdrchk(iterr, mterr, xorg, yorg, nox, noy, nuno)
c
      read(7,7001) ikv,fn
      open(ikv,file=fn,form='unformatted',status='old')
      write(8,7017) ikv,fn
 7017 format(' open unit=',i2,' file=',a,' for kv dif file')
      call hdrchk(ikv, mkv, xorg, yorg, nox, noy, nlayer)
c
c  read in terrain data and centroids of modeling cells
c  if using cartesian coordinate system
c
      read(iterr) ((avterr(i,j),i=1,nox),j=1,noy)
      if (lcart) then
        read(iterr) ((ctlon(i,j),i=1,nox),j=1,noy)
        read(iterr) ((ctlat(i,j),i=1,nox),j=1,noy)
      else
        do 189 j=1,noy
          do 186 i=1,nox
            ctlon(i,j)=wlong+(float(i)-0.5)*dlong
            ctlat(i,j)=slat+(float(j)-0.5)*dlat
  186     continue
  189   continue
      endif
c
c     make sure number of species is consistent with lchem option
c
      if (.not.lchem) then
        nospc1=nospec
        kso2=1
        kso4=2
        if (nospec.gt.mxspec) then
          write(8,72) nospec
          write(*,72) nospec
          write (ierrlg,712) 1004,nospec
   72     format(' error in setup nospec exceeds dim ',i5)
  712     format (i4,1x,' error in setup nospec exceeds dim ',i5) 
          stop1
        endif
      endif
c  
      read(7,73) dtmin,maxitr,rerror
      write(66,74) dtmin,maxitr,rerror
   73 format(f10.0,i10,f10.0)
   74 format(' dtmin= ',f10.6,' maxitr= ',i5,' rerror= ',f10.6)
c
c     input chemical rate constants
c
      if(idiag.ne.0)write(8,*)'entering chread...'
      call chread(lrstrt,jchem,nn1,nn2,idebug)
      if(idiag.ne.0)write(8,*)'returned from chread...'
      if(linert)then
        if (nn2 .ne. nospec) then
          write(*,*) 'inconsistent number of species ',nn2,'<>',nospec
          write(8,*) 'inconsistent number of species ',nn2,'<>', 
     &                 nospec
          write (ierrlg,'(i4,1x,a,a,i5,a,i5)') 1005,'inconsistent', 
     &                    ' number of species ',nn2, '<>',nospec
          stop
        endif
        nn1=nospec
      endif
c
c calculate the look-up table for the temperature dependent rate constants:
c
      call tirate
c
c     read in fractional land use for each cell
c
      write(*,3024) nox,noy,nospc1
 3024 format(' read surf file',3i10)
      read(isurf) (((fland(i,j,k),i=1,nox),j=1,noy),k=1,mlus)
      close(isurf)
c
c------------------------------------
c   open file containing aggregate-
c   cell map and process aggregate-
c   cell index information
c------------------------------------
      if(lchmag) then
        if(idiag.ne.0)write(8,*)'reading aggregation file name...'
        read(7,7001) iagg,fn
        open(iagg,file=fn,form='formatted', status='old')
        write(8,4567) iagg,fn
 4567   format(' open file=',i2,' file-',a,' for variable-mesh map')
      endif
      if(idiag.ne.0)write(8,*)'entering rvmmap...'
      call rvmmap(nox,noy,nlayer,8,tstr,idystr,idebug)
c
c  optionally open mdf files for any fine grid
c
      do 4560 ifine=1,numfin
        read(7,7001) ifmdf(ifine),fn
        if (ifmdf(ifine) .gt. 0) then
          open(ifmdf(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifmdf(ifine),
     &         ' file ',fn,' for mdf for fine grid ',
     &         ifine
          xf = xorg + (ixfb(ifine)-1)*dlong - dlong/float(nhf(ifine))
          yf = yorg + (jyfb(ifine)-1)*dlat - dlat/float(nhf(ifine))
          call hdrchk(ifmdf(ifine), mwnd, xf, yf, 
     &                nfx(ifine), nfy(ifine), nfz(ifine))
        else
          write(8,'(a,a,i2)') ' no file opened for mdf ',
     &         'for fine grid ',ifine
        endif
 4560 continue
c
c  optionally open vert diffusion files for any fine grid
c
      do 4570 ifine=1,numfin
        read(7,7001) ifkv(ifine),fn
        if (ifkv(ifine) .gt. 0) then
          open(ifkv(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifkv(ifine),
     &         ' file ',fn,' for kv for fine grid ',
     &         ifine
          xf = xorg + (ixfb(ifine)-1)*dlong - dlong/float(nhf(ifine))
          yf = yorg + (jyfb(ifine)-1)*dlat - dlat/float(nhf(ifine))
          call hdrchk(ifkv(ifine), mkv, xf, yf, 
     &                nfx(ifine), nfy(ifine), nfz(ifine))
        else
          write(8,'(a,a,i2)') ' no file opened for kv ',
     &         'for fine grid ',ifine
        endif
 4570 continue
c
c  optionally open height/pressure files for any fine grid
c
      do 4575 ifine=1,numfin
        read(7,7001) ifzp(ifine),fn
        if (ifzp(ifine) .gt. 0) then
          open(ifzp(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifzp(ifine),
     &         ' file ',fn,' for height/pressure for fine grid ',
     &         ifine
          xf = xorg + (ixfb(ifine)-1)*dlong - dlong/float(nhf(ifine))
          yf = yorg + (jyfb(ifine)-1)*dlat - dlat/float(nhf(ifine))
          call hdrchk(ifzp(ifine), mzp, xf, yf, 
     &                nfx(ifine), nfy(ifine), nfz(ifine))
        else
          if (nfz(ifine) .gt. nlayer) then
            write(8,'(a)') ' ****error****'
            write(8,'(a,a,i2)') ' No file opened for height/pressure ',
     &         'for fine grid ',ifine
            write(8,'(a)') ' Fine grid heights must be provided when'
            write(8,'(a)') ' number of fine grid layers exceeds number'
            write(8,'(a)') ' of coarse grid layers.'
            write(*,'(a)') ' ****error***'
            write(*,'(a,a,i2)') ' No file opened for height/pressure ', 
     &         'for fine grid ',ifine 
            write(*,'(a)') ' Fine grid heights must be provided when'
            write(*,'(a)') ' number of fine grid layers exceeds number'
            write(*,'(a)') ' of coarse grid layers.'
            write(ierrlg,'(i4,1x,a)') 1006, ' ****error****'
            write(ierrlg,'(a,a,i2)') 'No file opened for height/',
     &                   'pressure for fine grid ',ifine
            write(ierrlg,'(a)') ' Fine grid heights must be provided'
            write(ierrlg,'(a)') ' when number of fine grid layers' 
            write(ierrlg,'(a)') ' exceeds number of coarse grid layers'
            stop
          endif
          write(8,'(a,a,i2)') ' No file opened for height/pressure ',
     &         'for fine grid ',ifine
        endif
 4575 continue
c
c  optionally open temperature files for any fine grid
c
      do 4576 ifine=1,numfin
        read(7,7001) iftemp(ifine),fn
        if (iftemp(ifine) .gt. 0) then
          open(iftemp(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',iftemp(ifine),
     &         ' file ',fn,' for temperature for fine grid ',
     &         ifine
          xf = xorg + (ixfb(ifine)-1)*dlong - dlong/float(nhf(ifine))
          yf = yorg + (jyfb(ifine)-1)*dlat - dlat/float(nhf(ifine))
          call hdrchk(iftemp(ifine), mtmpr, xf, yf, 
     &                nfx(ifine), nfy(ifine), nfz(ifine))
        else
          write(8,'(a,a,i2)') ' No file opened for temperature ',
     &         'for fine grid ',ifine
        endif
 4576 continue
c
c  optionally open h2o files for any fine grid
c
      do 4577 ifine=1,numfin
        read(7,7001) ifuh2o(ifine),fn
        if (ifuh2o(ifine) .gt. 0) then
          open(ifuh2o(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifuh2o(ifine),
     &         ' file ',fn,' for h2o for fine grid ',
     &         ifine
          xf = xorg + (ixfb(ifine)-1)*dlong - dlong/float(nhf(ifine))
          yf = yorg + (jyfb(ifine)-1)*dlat - dlat/float(nhf(ifine))
          call hdrchk(ifuh2o(ifine), mhum, xf, yf, 
     &                nfx(ifine), nfy(ifine), nfz(ifine))
        else
          write(8,'(a,a,i2)') ' No file opened for h2o ',
     &         'for fine grid ',ifine
        endif
 4577 continue
c
c  optionally open rain files for any fine grid
c
      do 4578 ifine=1,numfin
        read(7,7001) ifrain(ifine),fn
        if (ifrain(ifine) .gt. 0) then
          open(ifrain(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifrain(ifine),
     &         ' file ',fn,' for rain for fine grid ',
     &         ifine
          xf = xorg + (ixfb(ifine)-1)*dlong - dlong/float(nhf(ifine))
          yf = yorg + (jyfb(ifine)-1)*dlat - dlat/float(nhf(ifine))
          call hdrchk(ifrain(ifine), mrain, xf, yf, 
     &                nfx(ifine), nfy(ifine), nfz(ifine))
        else
          write(8,'(a,a,i2)') ' No file opened for rain ',
     &         'for fine grid ',ifine
        endif
 4578 continue
c
c  optionally open cloud files for any fine grid
c
      do 4579 ifine=1,numfin
        read(7,7001) ifcld(ifine),fn
        if (ifcld(ifine) .gt. 0) then
          open(ifcld(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifcld(ifine),
     &         ' file ',fn,' for cloud for fine grid ',
     &         ifine
          xf = xorg + (ixfb(ifine)-1)*dlong - dlong/float(nhf(ifine))
          yf = yorg + (jyfb(ifine)-1)*dlat - dlat/float(nhf(ifine))
          call hdrchk(ifcld(ifine), mcld, xf, yf, 
     &                nfx(ifine), nfy(ifine), nfz(ifine))
        else
          write(8,'(a,a,i2)') ' No file opened for cloud ',
     &         'for fine grid ',ifine
        endif
 4579 continue
c
c  optionally open and read surface files for any fine grid
c
      do 4590 ifine=1,numfin
        read(7,7001) ifsrf(ifine),fn
        if (ifsrf(ifine) .gt. 0) then
          open(ifsrf(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifsrf(ifine),
     &         ' file ',fn,' for surface char. for fine grid ',
     &         ifine
          xf = xorg + (ixfb(ifine)-1)*dlong - dlong/float(nhf(ifine))
          yf = yorg + (jyfb(ifine)-1)*dlat - dlat/float(nhf(ifine))
          call hdrchk(ifsrf(ifine), msrf, xf, yf, 
     &                nfx(ifine), nfy(ifine), nuno)
          call rdfsrf(nfx(ifine),nfy(ifine),mlus,ifsrf(ifine),
     &                flandf(1,1,ifine))
          close(ifsrf(ifine))
        else
          write(8,'(a,a,i2)') ' no file opened for surface char. ',
     &         'for fine grid ',ifine
        endif
 4590 continue
c
c  open emissions files for fine grids and initialize ahprcf
c
      do 4595 ifine=1,numfin
        call ahintf(nfx(ifine),nfy(ifine),nfz(ifine),ahprcf(1,1,ifine))
        read(7,7001) ifaem1(ifine),fn
        if (ifaem1(ifine) .gt. 0) then
          open(ifaem1(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifaem1(ifine),
     &         ' file ',fn,' for weekday emissions for fine grid ',
     &         ifine
        else
          write(8,'(a,a,i2)') ' no file opened for weekday emissions ',
     &         'for fine grid ',ifine
        endif
        read(7,7001) ifaem2(ifine),fn
        if (ifaem2(ifine) .gt. 0) then
          open(ifaem2(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifaem2(ifine),
     &         ' file ',fn,' for saturday emissions for fine grid ',
     &         ifine
        else
          write(8,'(a,a,i2)') ' no file opened for saturday emissions ',
     &         'for fine grid ',ifine
        endif
        read(7,7001) ifaem3(ifine),fn
        if (ifaem3(ifine) .gt. 0) then
          open(ifaem3(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifaem3(ifine),
     &         ' file ',fn,' for sunday emissions for fine grid ',
     &         ifine
        else
          write(8,'(a,a,i2)') ' no file opened for sunday emissions ',
     &         'for fine grid ',ifine
        endif
 4595 continue
c-------------------------------------
c
c  initialize all flux tracking arrays to zero
c
      call flxnit(nospec,numfin)
c
c  write instantaneous file headers
c
      tb = tstr/100.
      te = tend/100.
      if (lcart) then
        dx = dlong*1000.
        dy = dlat*1000.
        xu = xorg*1000.
        yu = yorg*1000.
      else
        dx = dlong
        dy = dlat
        xu = wlong
        yu = slat
	utmzon = -9
      endif
c
      write(51) (ifili(i:i),mblank,mblank,mblank,i=1,10),
     &             (msg(i:i),mblank,mblank,mblank,i=1,60),nuno,
     &             nospc1,idystr,tb,idaynf,te
      write(51) fzero,fzero, utmzon, xu, yu, dx, dy,
     &          nox, noy, nlayer, izero, izero, fzero, fzero, fzero
      write (51)  izero,izero,nox,noy
      write (51) ((mspec(m,l),m=1,10),l=1,nospc1)   
c
c  identify species for avg output file
c
      if (iuavg .gt. 0) then
        if (navspc .le. 0) then
          navspc = nospec
          do 76 l=1,nospec
            kavspc(l) = l
            write(mavspc(l),'(10a1)') (mspec(i,l),i=1,10)
   76     continue
        else
          do 78 l=1,nospec
            write(nams,'(10a1)') (mspec(i,l),i=1,10)
            do 77 lav=1,navspc
              if (nams .eq. mavspc(lav)) then
                kavspc(lav) = l
              endif
   77       continue
   78     continue
c
c  allocate space for each step to be included in o3 average
c
          if (lmxo3) then
            no3stp = 0
            write(*,'(a)') ' Allocate space for step by step o3.'
            write(8,'(a)') ' Allocate space for step by step o3.'
            nsteps = int(60./dtavg + .98)
            nvals = nox*noy*nsteps
            call alloc(nscrtc, nsclft, xscrtc, nvals, ko3av, lerr)
            if (lerr) then
              write(*,'(a)') 'ALLOC reports error for coarse grid.'
              write(8,'(a)') 'ALLOC reports error for coarse grid.'
              write(ierrlg,'(i4,1x,a,a)') 1007,'ALLOC reports error',
     &                                ' for coarse grid'
              stop
            endif
            do 80 ifine=1,numfin
              nvals = nfx(ifine)*nfy(ifine)*nsteps
              call alloc(nscrtc, nsclft, xscrtc, nvals,
     &                   ko3avf(ifine), lerr)
              if (lerr) then
                write(*,'(a,i4,a)') 
     &               'ALLOC reports error for fine grid ',ifine,'.'
                write(8,'(a,i4,a)') 
     &               'ALLOC reports error for fine grid ',ifine,'.'
                write(ierrlg,'(i4,1x,a,i4,a)') 1008,
     &                'ALLOC reports error for fine grid ',ifine,'.' 
                stop
              endif
              nvals = nsteps
              call alloc(nscrtc, nsclft, xscrtc, nvals, ko3tmf(ifine),
     &                   lerr)
              if (lerr) then
                write(*,'(a)') 'ALLOC reports error for time array.'
                write(8,'(a)') 'ALLOC reports error for time array.'
                write(ierrlg,'(i4,1x,a,a)') 1009,'ALLOC reports error',
     &                                    ' for time array.'
                write(*,'(a,i3)') 'Fine grid = ', ifine
                write(8,'(a,i3)') 'Fine grid = ', ifine
                write(ierrlg,'(a,i3)') 'Fine grid = ', ifine
                stop
              endif
c             call alloc(nscrtc, nsclft, xscrtc, nvals, ko3ptf(ifine),
c    &                   lerr)
c             if (lerr) then
c               write(*,'(a)') 'ALLOC reports error for pointer array.'
c               write(8,'(a)') 'ALLOC reports error for pointer array.'
c               write(ierrlg,'(i4,1x,a,a)') 1010,'ALLOC reports error'
c    &                          ,' for pointer array.'
c               write(*,'(a,i3)') 'Fine grid = ', ifine
c               write(8,'(a,i3)') 'Fine grid = ', ifine
c               write(ierrlg,'(a,i3) 'Fine grid = ', ifine
c               stop
c             endif
   80       continue
            nvals = nsteps
            call alloc(nscrtc, nsclft, xscrtc, nvals, ko3tm, lerr)
            if (lerr) then
              write(*,'(a)') 'ALLOC reports error for time array.'
              write(8,'(a)') 'ALLOC reports error for time array.'
              write(ierrlg,'(i4,1x,a,a)') 1011, 'ALLOC reports error',
     &                        ' for time array'
              stop
            endif
c           call alloc(nscrtc, nsclft, xscrtc, nvals, ko3pt, lerr)
c           if (lerr) then
c             write(*,'(a)') 'ALLOC reports error for pointer array.'
c             write(8,'(a)') 'ALLOC reports error for pointer array.'
c             write(ierrlg,'(i4,1x,a,a)') 1012,'ALLOC reports error for',
c     &                             ' pointer array'

c             stop
c           endif
          endif
        endif
c
c  check that all average species have a valid pointer
c
        do 85 l=1,navspc
          if (kavspc(l) .le. 0 .or. kavspc(l) .gt. nospec+2) then
            write(*,'(a)') '***ERROR IN SETUP ***'
            write(*,'(a,a)') 'Invalid average species: ', mavspc(l)
            write(8,'(a)') '***ERROR IN SETUP ***'
            write(8,'(a,a)') 'Invalid average species: ', mavspc(l)
            write(ierrlg,'(i4,1x,a)') 1013,'***ERROR IN SETUP***'
            write(ierrlg,'(a,a)') 'Invalid average species: ',
     &                          mavspc(l)
            stop
          endif
   85   continue
c
c  write average file headers
c
        write(iuavg) (ifile(i:i),mblank,mblank,mblank,i=1,10),
     &               (msg(i:i),mblank,mblank,mblank,i=1,60),nuno,
     &               navspc,idystr,tb,idaynf,te
        write(iuavg) fzero,fzero, utmzon, xu, yu, dx, dy,
     &            nox, noy, nuno, izero, izero, fzero, fzero, fzero
        write (iuavg)  izero,izero,nox,noy
        write (iuavg) ((mavspc(l)(m:m),mblank,mblank,mblank,m=1,10),
     &                l=1,navspc)   
      endif
c
c  write step by step o3 file headers
c
      if (iuo3 .gt. 0) then
        nstmp = 1
        write(iuo3) (ifilo(i:i),mblank,mblank,mblank,i=1,10),
     &               (msg(i:i),mblank,mblank,mblank,i=1,60),nuno,
     &               nstmp,idystr,tb,idaynf,te
        write(iuo3) fzero,fzero, utmzon, xu, yu, dx, dy,
     &            nox, noy, nuno, izero, izero, fzero, fzero, fzero
        write (iuo3)  izero,izero,nox,noy
        write (iuo3) (mspint(m,3),m=1,10)
c
c   loop over fine grids, writing grid definition
c
        write(iuo3) numfin
        do 89 ifine=1,numfin
          write(iuo3) ixfb(ifine), jyfb(ifine),
     &              ixfe(ifine), jyfe(ifine), nhf(ifine),
     &              nvf(ifine), nfx(ifine), nfy(ifine),
     &              nfz(ifine), ifgptr(ifine), ifglvl(ifine)
 89     continue
      endif
c
c     check for bad dimensions
c
      write(*,7776) kind
 7776 format('kind=',30i3)
c
      if (.not.linert) then
c
        if (nn1.gt.mxspec .or. nn2.gt.mxspec) then
          write(*,7777) nn1,nn2
          write(8,7777) nn1,nn2
          write(ierrlg,7799) 1014,nn1,nn2
 7777      format(' error in number of chemical species input'
     &    ,' from chemparam file',/,' nors= ',i5,'  nospec= ',i5
     &    ,' must be equal to mxspec')
 7799      format(i4,1x,' error in number of chemical species input'
     &    ,' from chemparam file',/,' nors= ',i5,'  nospec= ',i5
     &    ,' must be equal to mxspec')
          stop2
        endif
c
        if (nospec.ne.nn2) then
          write(*,1010) nn2,nospec
          write(8,1010) nn2,nospec
          write(ierrlg,1019) 1015,nn2,nospec
 1010     format(' error in setup lchem=true then nospec = ',i5
     &    ,' but input value nospec= ',i5)
 1019     format(i4,1x,' error in setup lchem=true then nospec = ',i5
     &    ,' but input value nospec= ',i5)
          stop1
        endif
c
      endif
c
c read photolysis rates if not inert run
c
c the id numbers of the photolysis reactions are .....
c
c      nrxn / 1,38,39,9,45,90 /
c
c and, they must be read in from the nphval file in the order
c  no2, hchor, hchos, o3o1d, alds,ispd
c
      if (.not.linert) then
        read(7,7001) nphval, fn
        open(unit=nphval,file=fn,form='formatted',status='old')
        write(8,'(a,i2,a,a,/,a)') ' open unit ',nphval,
     &         ' file ',fn,' for photolysis rates table.'
c
c      read the species x zenith angle x altitude file for all
c         ozone column, haze and albedo classes
c
        do 28 nozc=1,5
          do 27 nalb=1,5
            do 25 nhaz=1,3
              read (nphval,2050) fn
              write(8,2020) fn
              read (fn,'(12x,f7.3,8x,f7.3,11x,f7.3)')
     &             ozcl(nozc), albcl(nalb), hazcl(nhaz)
              do 22 i=1,11
                read (nphval,2050) fn
                if (nozc .eq. 1 .and. nalb .eq. 1 .and. nhaz .eq. 1)
     &                                           write(8,2021) i,fn
* changed to 6 for ispd rxn
c  changed to 7 for V1.17
c
                do 21 ii=1,7
                  read (nphval,2051) (prkn(ii,j,i,nozc,nalb,nhaz),
     &                                                     j=1,10)
                  prkn(ii,11,i,nozc,nalb,nhaz) = 0.
   21           continue
   22         continue
   25       continue
   27     continue
   28   continue
        close (nphval)
 2020   format(' read phot values    ',a80)
 2021   format(' values for height',i2,1x,a80)
 2050   format (a80)
 2051   format (1x,10f12.0)
c
c   open file of albedo, haze, and ozone column codes and read albedo
c
        read (7,7001) nphval, fn
        open(unit=nphval,file=fn,form='formatted',status='old')
        write(8,'(a,i2,a,a,/,a)') ' open unit ',nphval,
     &     ' file ',fn,' for gridded albedo-haze-ozone column data.'
        read (nphval,2050) fn
        write(8,'(a80)') fn
        read(nphval,'(a10,5f10.0)') mdum, (vcheck(n),n=1,5)
        write(8,'(a10,5f10.4)') mdum, (vcheck(n),n=1,5)
        do 310 n=1,5
          if (vcheck(n) .ne. albcl(n)) then
            write(*,*) ' albedo classes in gridded codes file',
     &            ' do not match values in photolysis rates table.'
            write(8,*) ' albedo classes in gridded codes file',
     &            ' do not match values in photolysis rates table.'
            write(ierrlg,'(i4,1x,a,a,a)') 
     &             1018,' albedo classes in gridded',
     &             ' codes file do not match values in photolysis',
     &             ' rates table'
            stop
          endif
  310   continue
        read(nphval,'(a10,5f10.0)') mdum, (vcheck(n),n=1,3)
        write(8,'(a10,5f10.4)') mdum, (vcheck(n),n=1,3)
        do 315 n=1,3
          if (vcheck(n) .ne. hazcl(n)) then
            write(*,*) ' haze classes in gridded codes file',
     &            ' do not match values in photolysis rates table.'
            write(8,*) ' haze classes in gridded codes file',
     &            ' do not match values in photolysis rates table.'
            write(ierrlg,'(i4,1x,a,a,a)') 
     &            1019, 'haze classes in gridded',
     &            ' codes file do not match values in photolysis',
     &            ' rates table.'

            stop
          endif
  315   continue
        read(nphval,'(a10,5f10.0)') mdum, (vcheck(n),n=1,5)
        write(8,'(a10,5f10.4)') mdum, (vcheck(n),n=1,5)
        do 320 n=1,5
          if (vcheck(n) .ne. ozcl(n)) then
            write(*,*) ' ozone column classes in gridded codes file',
     &            ' do not match values in photolysis rates table.'
            write(8,*) ' ozone column classes in gridded codes file',
     &            ' do not match values in photolysis rates table.'
            write(ierrlg,'(i4,1x,a,a,a)') 1020,' ozone column classes',
     &            ' in gridded codes file do not match values in', 
     &            ' photolysis rates table.'

            stop
          endif
  320   continue
c
c   read gridded albedo classes for coarse grid
c
        read(nphval,'(a10,3i10)') mdum,idd,nxalb,nyalb
        call lftpak(mdum)
        write (8,'(a10,3i10)') mdum,idd,nxalb,nyalb
        if (mdum(1:6) .ne. 'ALBEDO') then
          write(*,*) ' Incorrect data label ', mdum, 
     &      ' data label must be ALBEDO'
          write(8,*) ' Incorrect data label ', mdum, 
     &      ' data label must be ALBEDO'
          write(ierrlg,'(i4,1x,a,a,a)') 1022, 'Incorrect data label ',
     &      mdum,' data label must be ALBEDO'
          stop
        endif
        if (nxalb .ne. nox .or. nyalb .ne. noy) then
          write(*,*) ' coarse grid dimensions on albedo file ',
     &          ' do not match nox, noy: ', nox, noy
          write(8,*) ' coarse grid dimensions on albedo file ',
     &          ' do not match nox, noy: ', nox, noy
          write(ierrlg,'(i4,1x,a,a,2i5)') 
     &          1023,' coarse grid dimensions',
     &          ' do not match nox, noy: ', nox, noy
          stop
        endif
        do 330 j=noy,1,-1
          read(nphval,'(200i1)') (jalb(i,j),i=1,nox)
  330   continue
c
c  read in gridded albedo classes for any fine grids that
c  may be provided.  fine grids must be in order, within range
c  of those defined, and correct dimensions.
c
        ifsav = 0
        do 340 ifine=1,numfin
          read(nphval,'(a10,3i10)') mdum,iff,nxalb,nyalb
          call lftpak(mdum)
          write (8,'(a10,3i10)') mdum,iff,nxalb,nyalb
          if (iff .le. 0) go to 350
          if (mdum(1:6) .ne. 'ALBEDO') then
            write(*,*) ' Incorrect data label ', mdum, 
     &      ' data label must be ALBEDO'
            write(8,*) ' Incorrect data label ', mdum, 
     &      ' data label must be ALBEDO'
            write(ierrlg,'(i4,1x,a,a,a)') 1024, ' Incorrect data label',
     &      mdum, ' data label must be ALBEDO'
            stop
          endif
          if (iff .ne. ifine) then
            write(*,*) ' fine grid albedo data out of order.'
            write(8,*) ' fine grid albedo data out of order.'
            write(ierrlg,'(i4,1x,a,a)') 1025,'fine grid albedo out',
     &                  ' of order'
            stop
          endif
          if (nxalb.ne.nfx(ifine) .or. nyalb.ne.nfy(ifine)) then
            write(*,*) ' fine grid dimensions on albedo file ',
     &          ' do not match nfx, nfy: ', nfx(ifine), nfy(ifine)
            write(8,*) ' fine grid dimensions on albedo file ',
     &          ' do not match nfx, nfy: ', nfx(ifine), nfy(ifine)
            write(ierrlg,'(i4,1x,a,a,2i1)') 1026,' fine grid dimensions'
     &          ,' on albedo file do not match nfx, nfy: ',  
     &            nfx(ifine),nfy(ifine)
            stop
          endif
          do 335 j=nfy(ifine),1,-1
            read(nphval,'(200i1)') (jfalb(i,j,ifine),i=1,nfx(ifine))
  335     continue
          ifsav = ifine
  340   continue
        read(nphval,'(a10,3i10)') mdum,iff,nxalb,nyalb
        call lftpak(mdum)
        write (8,'(a10,3i10)') mdum,iff,nxalb,nyalb
        if (mdum(1:6) .ne. 'ALBEDO') then
          write(*,*) ' Incorrect data label ', mdum, 
     &      ' data label must be ALBEDO'
          write(8,*) ' Incorrect data label ', mdum, 
     &      ' data label must be ALBEDO'
          write(ierrlg,'(i4,1x,a,a,a)') 
     &      1027,' Incorrect data label ', mdum,
     &      ' data label must be ALBEDO'
          stop
        endif
        if (iff .ne. 0) then
          write(*,*) ' Unidentifiable fine grid, #',iff
          write(8,*) ' Unidentifiable fine grid, #',iff
          write(ierrlg,'(i4,1x,a,a,i10)') 1028,' Unidentifiable ',
     &      'fine grid, #',iff
          stop
        endif
c
  350   continue
c
c   if jfalb has not been set for any of the fine grids, 
c   initialize it to value in next coarser grid
c
        if (ifsav .lt. numfin) then
          do 390 ifine=ifsav+1,numfin
            if (ifgptr(ifine) .eq. 0) then
              do 365 j=2,nfy(ifine)-1
                jc = jyfb(ifine) + (j-2)/nhf(ifine)
                do 360 i=2,nfx(ifine)-1
                  ic = ixfb(ifine) + (i-2)/nhf(ifine)
                  jfalb(i,j,ifine) = jalb(ic,jc)
  360           continue
  365         continue
            else
              ifset = ifgptr(ifine)
              nhr = nhf(ifine)/nhf(ifset)
              ioff = 2 + (ixfb(ifine)-ixfb(ifset))*nhf(ifset)
              joff = 2 + (jyfb(ifine)-jyfb(ifset))*nhf(ifset)
              do 375 j=2,nfy(ifine)-1
                jf2 = joff + (j-2)/nhr
                do 370 i=2,nfx(ifine)-1
                  if2 = ioff + (i-2)/nhr
                  jfalb(i,j,ifine) = jfalb(if2,jf2,ifset)
  370           continue
  375         continue
            endif
  390     continue
        endif
c
c  initialize dates and times for haze and ozone column
c
        oztim1 = 0.
        iozdt1 = 0
        oztim2 = 0.
        iozdt2 = 0
        hztim1 = 0.
        ihzdt1 = 0
        hztim2 = 0.
        ihzdt2 = 0
c
      endif
c
      noxy=nox*noy
      write(*,3004)
 3004 format(' read all of input file ')
      time=tstr
      ndate=idystr
      totime=tstr
c
c  read point source emissions data
c
      if (lpts) then
        call hedpts(nospec, nox, noy, edf, nemit, kemit)
        write(*,3013) nopts,edf
 3013   format(' reading point source data nopts=',2i10)
   70   format(i10,l10,f10.0,l10)
        write(8,1080) nopts, larea
 1080   format(//i10,' point sources are involved in this simulation; ',
     &   ' area source emissions= ',l5,5x)
c
c
c     check if suppressing output
c
        do 140 i=1,nopts
          if (lsuprs.and.(i.gt.5)) go   to 140
c
          write(8,150) i
          write(8,152) plong(i),plat(i),stkht(i),
     &                 stkd(i),temps(i),vexit(i),pflag(i)
  150     format(' source number ',i5)
  152     format(' stack parameters   ',/,6f10.2,l10)
  140   continue
c
        write(*,3014) nplm
 3014   format(' finish reading points nplm =',i10)
c
        if (nplm.gt.mxplms) then
          write(8,153) nplm
          write(*,153) nplm
          write(ierrlg,155)1029,nplm
  153     format('$error parameter mxplms is not big enough, nplm =',i5)
  155     format(i4,1x,'$error parameter mxplms is not big enough, ',
     &       'nplm =',i5)
          stop3
        endif
c
        write(8,226) nopts
        write(*,226) nopts
  226   format(' in setup nopts = ', i10)
      endif
c
c
c
c  if area sources are present, read headers
c
      if (larea) then
        call gunit(aem,time,ndate)
        do 1901 ifine=1,numfin
          if (aem .eq. aem1) then 
            ifaem(ifine) = ifaem1(ifine)
          else if (aem .eq. aem2) then
            ifaem(ifine) = ifaem2(ifine)
          else if (aem .eq. aem3) then
            ifaem(ifine) = ifaem3(ifine)
          endif
 1901   continue
        if (idiag .ne. 0) then
          write(*,3019)time,ndate,aem
 3019     format(' after call gunit ',f10.1,i10,i10)
        endif
        call hedqar(nospec,nox,noy,aem,naemit,kaemit)
        if (idiag .ne. 0) then
          write(*,'(a)') ' back from hedqar'
        endif
        do 1955 ifine=1,numfin
          if (ifaem(ifine) .gt. 0) then
            call hedqar(nospec,nfx(ifine),nfy(ifine),ifaem(ifine),
     &                  nfemit(ifine), kfemit(1,ifine))
            if (idiag .ne. 0) then
              write(*,'(a)') ' back from hedqar'
            endif
          endif
 1955   continue
        kntaem=0
      endif
c
c  if area sources included, skip to correct time
c
      if (larea) then
        call getqar(mxx,mxy,mxspec,nox,noy,naemit,aem,time,ndate,
     &              emtim1,iemdy1,emtim2,iemdy2,qarea,iqeof)
        if (iqeof .ne. 0) then
          write(*,'(a,i4,a,f10.2,i10)') ' end of file on unit',
     &         aem,' at ',time,ndate
          stop
        endif
        if (idiag .ne. 0) then
          write(*,'(a)') ' back from getqar'
        endif
        do 1902 ifine=1,numfin
          if (ifaem(ifine) .gt. 0) then
            call getqar(nfx(ifine),nfy(ifine),nfemit(ifine),
     &                  nfx(ifine),nfy(ifine),nfemit(ifine),
     &                  ifaem(ifine),time,ndate,
     &                  emtm1f(ifine),iemd1f(ifine),
     &                  emtm2f(ifine),iemd2f(ifine),
     &                  qfarea(1,1,ifine),iqeof)
            if (iqeof .ne. 0) then
              write(*,'(a,i4,a,f10.2,i10)') ' end of file on unit',
     &              ifaem(ifine),' at ',time,ndate
              stop
            endif
            if (idiag .ne. 0) then
              write(*,'(a)') ' back from getqar'
            endif
          endif
 1902   continue
      endif
c
      if (lpts) then
        call getpt(time,ndate,edf,nemit,iqeof)
        if (iqeof .ne. 0) then
          write(*,'(a,i4,a,f10.2,i10)') ' end of file on unit',
     &         edf,' at ',time,ndate
          stop
        endif
      endif
c
c...    add up total area emissions
      do 278 i=1,mxspec
        tarea(i,1)=0.0
        tarea(i,2)=0.0
  278 continue
      do 175 ll=1,naemit
        l = kaemit(ll)
	if ( l .le. 0 ) goto 175
        do 177 j=1,noy
          do 173 i=1,nox
            tarea(l,1)=tarea(l,1)+qarea(i,j,ll)
  173     continue
  177   continue
  175 continue
      do 178 ll=1,nemit
        l=kemit(ll)
	if ( l .le. 0 ) goto 178
        do 176 i=1,nopts
          tarea(l,2)=tarea(l,2)+ptemis(ll,i)
  176   continue
  178 continue
      write(8,301)
      write(8,300) (tarea(l,1),l=1,nospec)
      write(8,302) (tarea(l,2),l=1,nospec)
  301 format(' total area and point source emissions (moles/min)')
  300 format(' area ',8(1x,e12.4))
  302 format('point ',8(1x,e12.4))
c
c  total area source emissions on fine grids
c
      do 98 ifine=1,numfin
        call eftot(nfx(ifine),nfy(ifine),nospec,
     &                  qfarea(1,1,ifine),ifine,idebug)
   98 continue
c
c     calculate grid spacing in km
c
      write(*,3021)
 3021 format(' call grdsp')
      call grdsp
c
c  *** set initial radical concentration is 1.e-7 (dfg).
c
      do 881 i=1,mxrads
        do 881 j=1,nox*noy*nlayer
        radcon(i,j)=1.0e-7
 881  continue
c
c  ***read initial concentrations in ppm (lchem=.true.) or ug/m3
c
      if (.not.lrstrt) then
        call hedibc(nospec,nox,noy,icbc,nicsp,kicsp,mdum)
        if (mdum .ne. ifili) liccon = .true.
        call getic(time,ndate,icbc,iceof)
        if (iceof .ne. 0) then
          write(*,*) ' end of file on ic file'
          stop
        endif
      endif
      if (icbc .gt. 0) then
        close (icbc)
        write(8,'(a,i3,a)') ' closed ic file (unit ',icbc,')'
      endif
c
c   open and read aloft concentration file
c
      read(7,7001) icbc,fn
      open(icbc,file=fn,form='formatted',status='old')
      write(8,7026) icbc,fn
 7026 format(' open unit=',i2,'= file=',a,'for aloft concentrations')
c
c   set all caloft initially to bdnl
c
      do 102 l=1,nospec
        caloft(l) = bdnl(l)
  102 continue
c
c   read a line, identify a species, and go back for more
c
  105 read(icbc,171,end=108) mdum, ctest
  171 format(a10,f10.7)
      do 107 l=1,nospec
        write(nams,'(10a1)') (mspec(i,l),i=1,10)
        if (nams .eq. mdum) then
          caloft(l) = ctest
        endif
  107 continue
      go to 105
c
  108 write(8,201)
      write(8,202) ((mspec(n,i),n=1,10),caloft(i),i=1,nospec)
  201 format(' assumed concentrations above top layer= ')
  202 format(5x,10a1,2x,e12.4)
      close (icbc)
      write(8,'(a,i3,a)') 
     &           ' closed aloft concentration file (unit ',icbc,')'
c
c read boundary file unit
c
      read(7,7001) icbc,fn2
c
c    open deposition velocity file
c
      read(7,7001) ibeta,fn
      if ( ibeta .gt. 0 ) then
         open(ibeta,file=fn,form='unformatted',status='old')
         write(8,7028) ibeta,fn
	 ldepn=.false.
         call heddv(nospec)
      else
         if (ldepn) then
           write(8,7029)
         else
           write(8,7040)
         endif
      endif
 7028 format(' open unit=',i2,'= file=',a,'for deposition velocities',
     &        /,'  ldepn will be set to .false.')
 7029 format(' no file opened for deposition velocities...',/,
     &    ' dry deposition will be calculated internally.')
 7040 format(' no file opened for deposition velocities...',/,
     &    ' no dry deposition will be calculated.')
c
c  optionally open P-i-G dump file
c
      if ( lpig ) then
        read(7,7001) ipig,fn
        if ( ipig .gt. 0 ) then
  	   open(ipig,file=fn,form='unformatted')
	   write(8,7041) ipig,fn
        else
	   write(8,7042)
        endif
      endif
 7041 format(' open unit=',i2,' file=',a,' for P-i-G dump ')
 7042 format(' no file opened for P-i-G dump ')
c
c  before we get too far, check unit numbers for conflicts
c
      call unchek
c
c  read the boundary condition headers
c  concentrations are read in update
c
      open(icbc,file=fn2,form='unformatted',status='old')
      write(8,7027) icbc,fn2
 7027 format(' open unit=',i2,'= file=',a,'for boundary concentrations')
      call hedibc(nospec,nox,noy,icbc,nbcsp,kbcsp,mdum)
      call bnddef(icbc,nox,noy,istr,iend,jstr,jend)
c
c     write out boundary definition to unit 8
c
      write(8,2080)
      write(8,2081) (istr(j),iend(j),j=1,noy)
      write(8,2081) (jstr(i),jend(i),i=1,nox)
 2080 format(' boundary definition')
 2081 format(10('(',i3,',',i3,')'))
c
c     if restarting read old concentrations
c
      if (lrstrt) then
        call restrt(time,ndate)
        if (lpig) call pgread(time,ndate)
      endif
      if ( lpig .and. .not. lrstrt .and. ipig .gt. 0 ) then
	 npuffs=0
	 call pigdmp(time,ndate)
      endif
c
      call diagn(time,ndate,1,tend,tstr,idystr
     &,idaynf)
c
c  factor to convert micromoles/m3 to ppm
c  1293 is density of air in gm/m3 at stp
c  28.8 is average molecular weight of air
c
      cpfac = 28.8/1293.
c 
c  use t = 298 and p = 1 atm to convert lower bound and caloft
c
      ff = 273./(298.*cpfac)
      do 200 l=1,nospec
        bdmol(l) = bdnl(l)*ff
        caloft(l) = caloft(l)*ff
  200 continue
c
c  update reads the meteorological data
c
      call update(time,ndate,tdatnx,totime,idebug)
c
c
c  variables lsnow and lstres are included for future development
c  for now, we just initialize them to zeros
c
      do 206 j=1,noy
        do 205 i=1,nox
          lsnow(i,j) = 0
          lstres(i,j) = 0
  205   continue
  206 continue
      do 207 j=1,mxfxby
        lsnowf(j) = 0
        lstrsf(j) = 0
  207 continue
c
c   convert ic in ppm to micromoles/m3 if necessary
c
      if (liccon) then
        do 240 k=1,nlayer
          do 230 l=1,nospec
            do 220 j=1,noy
              do 210 i=1,nox
                conc(i,j,l,k) = conc(i,j,l,k)*press(i,j,k)*
     &                          273./(temper(i,j,k)*cpfac)
  210         continue
  220       continue
  230     continue
  240   continue
      endif
c
      if (idiag .ne. 0) then
        write(*,3015)
 3015   format(' leave setup ',f10.1,i10)
      endif
      return
      end
