      subroutine smolar (dt,deltax)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: smolar.f,v $
c  Revision 1.5  1999/07/07  16:37:47  tcm
c  Modified to keep track of horizontal fluxes for pa/irr.
c
c  Revision 1.4  1996/06/14  23:04:28  tcm
c  corrected antidiffusion calculation for south and
c  west edges
c
c  Revision 1.3  1995/04/26  00:46:02  tcm
c  	removed superfluous diagnostic on d(j)
c
c  Revision 1.2  94/02/10  21:00:27  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c
c     ***  perform advection using scheme of smolarkiewicz (1983)
c
c     -- changed to deal with optional staggered u/v grids
c        (tcm, 7/15/91)
c
      include 'parm.cmd'
      include 'advec.cmd'
c
      common /shas/ w(mxxx),c(mxxx),d(mxxx),cb(mxxx),
     &              jadf(mxxx),nm1,b1,b2
c
      dimension a(mxxpl2)
c
      dimension wm(mxxx),wmd(mxxx)
c
czswa---------------------------------------------------------------
c
c  add two dimensions to tally the fluxes on the boundaries
c
      common /irrshas/ flux1(mxxx), flux2(mxxx)
c
c  initialize the fluxes to zero
c
      do i = 1, mxxx
         flux1(i) = 0.0
         flux2(i) = 0.0
      enddo
c
czswz---------------------------------------------------------------
c
c
c     adjust for deposition
c
      do 5 j=2,nm1
        c(j)=c(j)+d(j)*c(j)*dt
    5 continue
c
c
c     *** check beg boundary
c
      if (lstagr) then
        wchk = w(1)
      else
        wchk = .5*(w(1) + w(2))
      endif
      if (wchk.lt.0.) then
c
c        --wind is blowing out.  redefine boundary concentration
c
         c(1) = c(2)
c
      else
c
c        --wind is blowing in.  use inflow boundary conditions
c
         c(1)=b1
c
      endif
c****************************
c
c
      a(1)=c(1)
c
c     *** check end boundary
c
      n=nm1+1
c
      if (lstagr) then
        wchk = w(nm1)
      else
        wchk = .5*(w(nm1) + w(n))
      endif
c
      if (wchk.gt.0.) then
c
c        --wind is blowing out.  redefine boundary concentration
         c(n) = c(nm1)
c
      else
c
c        --wind is blowing in.  use inflow boundary conditions
         c(n)=b2
c
      endif
c********************
c
c
      a(n)=c(n)
c
c  corrected below -- factor of ca does not belong in wq until
c  after wmq has been calculated.  (tcm, 8/25/95)
      ca=dt/deltax
      if (lstagr) then
Ctcm    wq = w(1)*ca
        wq = w(1)
      else
Ctcm    wq=0.5*(w(2)+w(2-1))*ca
        wq=0.5*(w(2)+w(2-1))
      endif
      wmq=(abs(wq)*deltax-wq*wq*dt)/deltax
      wq = wq*ca
      f2=0.5*((wq+abs(wq))*c(1)+(wq-abs(wq))*c(2))
      do 90 i=2,nm1
        if (lstagr) then
          wm(i) = w(i)
        else
          wm(i)=0.5*(w(i)+w(i+1))
        endif
        wmd(i)=(abs(wm(i))*deltax-wm(i)*wm(i)*dt)/deltax
   90 continue
c
c     advection loop
c
      do 100 i=2,nm1
        f1=f2
        wp=wm(i)*ca
        abswp=abs(wp)
        f2=0.5*((wp+abswp)*c(i)+(wp-abswp)*c(i+1))
c
czswa----------------------------------------------------------
c  keep track of fluxes at the boundaries
c
        flux1(i) = flux1(i) + f1
        flux2(i) = flux2(i) - f2
c
czswz-------------------------------------------------------------
c
  100 a(i)=c(i)-(f2-f1)
c
c
      do 299 kk=1,2
        wadq=wmq*(a(2)-a(1))/(a(1)+a(2)
     &  +1.0e-15)
        wmq=(abs(wadq)*deltax-dt*wadq*wadq)/deltax
        abswdq=abs(wadq)
        f2=0.5*ca*((wadq+abs(wadq))*a(1)+(wadq-abs(wadq))
     &  *a(2))
c
c  don't do anti-diffusion adjustment on boundary
c
        f2 = 0.
c
c**************calculate anti-diffusion velocity
c
        do 200 i=2,nm1
          f1=f2
          wadm=wmd(i)*(a(i+1)-a(i))/(a(i)+a(i+1)+1.0e-15)
          wmd(i)=(abs(wadm)*deltax-dt*wadm*wadm)/deltax
          f2=0.5*ca*((wadm+abs(wadm))*a(i)+(wadm-abs(wadm))*a(i+1))
c
c  don't do anti-diffusion adjustment on boundary or on nested grid
c  interfaces
c
          if (i .eq. nm1) f2 = 0.
          if (jadf(i) .ne. 0) f2 = 0.
          a(i)=a(i)-(f2-f1)
c
czswa----------------------------------------------------------
c  keep track of fluxes at the boundaries
c
        flux1(i) = flux1(i) + f1
        flux2(i) = flux2(i) - f2
c
czswz-------------------------------------------------------------
c
  200   continue
  299 continue
c 
czswa-------------------------------------------------------------
c
c  the old code is replaced/rewritten with IRR included
c
c     horizontal diffusion--cb(i)=kh(i)*deltat/(deltax)**2
c
      do 301 i=2,nm1
        t1 = cb(i-1)*(a(i)-a(i-1))
        t2 = cb(i)*(a(i+1)-a(i))
        if (i .eq. 2 .or. jadf(i-1) .ne. 0) then
          c(i) = a(i) + t2
          t1 = 0.0
        else if (i .eq. nm1 .or. jadf(i) .ne. 0) then
          c(i) = a(i) - t1
          t2 = 0.0
        else
          c(i) = a(i) + t2 - t1
        endif
c
czswa----------------------------------------------------------
c  keep track of fluxes at the boundaries
c
        flux1(i) = flux1(i) - t1
        flux2(i) = flux2(i) + t2
c
czswz-------------------------------------------------------------
c
  301 continue
 9999 return
      end
