      subroutine step4 (nspec,nrs,dt,con,lerr,tcll,rcon,llpt,
     &                  icf,iflg, k, aac2m)
c
c  new routine for step4 to incorporate the automatically generated 
c  chem routines.
c  Dongfen Gao -- Standard Solver
c
c$Log: step4_std.f,v $
c Revision 1.5  2002/06/26  20:44:13  tcm
c Added check of lnitr flag before calculating con(knitr)
c
c Revision 1.4  1999/12/21  21:09:26  tcm
c Corrected use of lnitr to knitr at about line 282.
c
c Revision 1.2  1999/07/07  16:40:10  tcm
c Modified argument list, added code for irr tracking.  This code
c must still be updated for the CB-IV tox mechanism.
c
c Revision 1.1  1999/06/23  23:45:39  dfg
c Initial revision
c
c
c
c--input variables
c         nspec     i  no of species
c         nrs       i  no of reactive species
c         dt        r  time step (min)
c         con       r  concentrations (ppm)
c        rcon       r  radical concentrations (ppm)
c
c--output variables
c         con       r  new concentrations (ppm)
c        rcon       r  new radical concentrations (ppm)
c
c--local variables
c         hold      r  temporary conc vector
c        rhold      r  temporary radical conc array
c         rate
c
c         iconv     i  =1 means failure to converge
c
c--routines called
c         chem
c         chem2a1
c         chem2a2
c         chem2b
c
      include  'parchm.cmd'
      include  'comchem.cmd'
      include  'comd3.cmd'

      dimension  con(nspec), hold(mxspec),ctmp(mxspec)
      dimension  rcon(mxrads),rhold(mxrads)
      dimension  rate(mxspec), rp(mxspec,mxspec) 
      logical lerr,lfirst,llpt
      common /idcell/icell,jcell,kcell,tmchm
c
czswa---------------------------------------------------------------
c  again, put in the proper dimensions for the job
c
      include  'parm.cmd'
      include  'parfin.cmd'
      include 'irmb.cmd'
      dimension r_old(irrxy,mxgrx,mxz), r_irr(mxgrx)
      dimension r_inst(mxgrx)
      logical ltimeloop
c
c  initialize some of the variables
c
      if (iflg .gt. 0) then
         ltimeloop = .true.
         do irs = 1, nreact
            r_old(iflg,irs,k) = 0.0
            r_irr(irs) = 0.0
         enddo
      endif
c
czswz---------------------------------------------------------------
c

      data dchmin /1.0e-7/
      itmax = 0
      lss  = .true.
c
c     *** set up time step parameters
c
      nitmax=10
      dttot = 0.
      dtmax=6.    ! 6 minutes
      dtuse = amin1(dt,dtmax)
      hdt = 0.5*dtuse
      tdt = 0.5*dtuse
c
c     write(*,*) 'rk1= ', rk(1)
  50  continue
c     write(*,*) 'itcnt,dtuse,dttot= ',itcnt,dtuse,dttot
c--- save the total nitrogrn for balance check
c
        tnoy = con(kno)+con(kno2)+con(kpan)+con(khno3)+con(khno2)
     &     + con(kpanx)+con(kpna)
        if (lnitr) tnoy=tnoy+con(knitr)
c
        if (ldark) then
          call titrat(con(kno),con(kno2),rcon(kno3),con(ko3),tcll)
        endif
c
        do 100 l=1,nspec
          hold(l) = con(l)
  100   continue
c
        do 110 l=1,mxrads
          rhold(l) = rcon(l)
  110   continue
c
czswa-------------------------------------------------------------------
c  keep track of all species (ss species + explicit species)
c  this subroutine is called if we need the change in concentration
c  before and after the titrate subroutine
c
czswz-------------------------------------------------------------------
c
c
        iter=0
  115   iter=iter+1
        iconv=0
c
        do l=1,nspec
          ctmp(l)=(hold(l)+con(l))/2.
        end do
c***
        if (ldark) then
c
c  nighttime, assume no,o3 and no3 react immediately
c      O and O1D are zero.
c
          rhold(ko)=1.0E-30
          rhold(ko1d)=1.0E-30
c
          if (hold(kno).lt.1.e-4.and.hold(ko3).lt.1.e-4) then
             call chem2b(nspec,nrs,ctmp,rhold,rate,rp,lerr)
          elseif (hold(ko3).lt.1.0e-6) then
c
c  When chem2a1 is called, this means that NO has won the titrations.
c  O3 concentration is set to zero.  So does for NO3 and N2O5.
c
             rhold(kno3)=1.0e-19
             rhold(kn2o5)=1.0e-19
             ctmp(ko3)=1.0e-12
             hold(ko3)=1.0e-12
             call chem2a1(nspec,nrs,ctmp,rhold,rate,rp,lerr)
          elseif (hold(kno).lt.1.0e-6) then
c
c  When chem2a2 is called, this means that O3 has won the titrations.
c  NO concentration is set to zero.
c
             ctmp(kno)=1.0e-12
             hold(kno)=1.0e-12
             rhold(kto2)=1.0E-19
             call chem2a2(nspec,nrs,ctmp,rhold,rate,rp,lerr)
          endif
        else
          call chem(nspec,nrs,ctmp,rhold,rate,rp,lerr) 
        endif

        if (lerr)  then
          write(*,*) 'error in chem -- return true for lerr'
          write(*,*) 'ier,iter,nitmax= ',ier,iter,nitmax
          write(*,1200) (n,rate(n),n=1,nrs)
          write(*,1300) (n,hold(n),n=1,nrs)
          write(*,1400) (n,rk(n),n=1,94)
          call flush (6)
          stop
        endif
c
czswa-------------------------------------------------------------------
c  get the starting point to apply the trapazoidal rule
c
      if (ltimeloop) then
         ltimeloop = .false.
         if (iflg .gt. 0) then
            call irr_rate(iflg,k,nrs,hold,r_irr,aac2m,r_inst)
            do irs = 1, nreact
               r_old(iflg,irs,k) = r_irr(irs)
            enddo
         endif
      endif
c
czswz-------------------------------------------------------------------
c
c
        do 130 i=1,nrs
          do 120 j=1,nrs
            rp(i,j)=-0.5*hdt*rp(i,j)
  120     continue
  130   continue
c
        do 140 i=1,nrs
          rp(i,i)=1.+rp(i,i)
  140   continue
c
        do 150 l=1,nrs
          rate(l) = -(hold(l) - con(l) - hdt*rate(l))
  150   continue
c
        if (ldark) then
          if (hold(kno).lt.1.e-4.and.hold(ko3).lt.1.e-4) then
            call colpivot(1,nrs,mxspec,rp,rate,ier)
          elseif (hold(kno).lt.1.0e-6) then
            call colpivot(2,nrs,mxspec,rp,rate,ier)
            rate(1)=0.
          elseif (hold(ko3).lt.1.0e-6) then
            rate(3)=rate(1)
            do ll=1,nrs
              tmp = rp(1,ll)
              rp(1,ll) = rp(3,ll) 
              rp(3,ll) = tmp 
            end do
            do ll=1,nrs
              tmp = rp(ll,1)
              rp(ll,1) = rp(ll,3) 
              rp(ll,3) = tmp 
            end do

            call colpivot(2,nrs,mxspec,rp,rate,ier)
            rate(1)=rate(3)
            rate(3)=0.
          endif
        else
          call colpivot(1,nrs,mxspec,rp,rate,ier)
        endif
c
        if (ier.eq.0) then
          lerr=.true.
          write(*,*) 'error in concentration calculation'
          write(*,*) 'in colpivot, matrix of equations is singular'
          write(*,*) 'ier,iter,nitmax= ',ier,iter,nitmax
          write(*,1200) (n,rate(n),n=1,nrs)
          write(*,1300) (n,hold(n),n=1,nrs)
          write(*,1400) (n,rk(n),n=1,94)
          call flush (6)
          stop
        endif
c
        tmpmax=0.
        do 160 l = 1,nrs
          tmphh=hold(l)
          if (hold(l).gt.1.0e-6) then
            err=abs(rate(l)/hold(l))
          else
            err = abs(rate(l))
          endif
          hold(l)=tmphh + rate(l)
          if (err.gt.tmpmax) then
            tmpmax=err
          endif
          if (hold(l).le. 0.0) hold(l)=tmphh/10.
  160   continue
c
        if (tmpmax.le. rerror) then
c
c  now values converged, save concentrations, and check if we finished
c  the original step.
c
          if (itmax.lt.iter) itmax = iter
          do 200 l=1,nrs
            if (hold(l).lt.bdnl(l)) hold(l)=bdnl(l)
            con(l) = hold(l)
  200     continue
          do 210 l=1,mxrads
            rcon(l) = rhold(l)
  210     continue
c
c--- calculate total nitrogen after concentration converged.
c    if total nitrogen is changes above 1%, we will force
c    the total nitrogen to be balance.
c
          tnoy1 = con(kno)+con(kno2)+con(kpan)+con(khno3)+con(khno2)
     &          + con(kpanx)+con(kpna)
          if (lnitr) tnoy1=tnoy1+con(knitr)

          difnoy=(tnoy1-tnoy)/tnoy
          if ( abs(difnoy) .gt. 0.01) then
            con(kno)=con(kno)*tnoy/tnoy1
            con(kno2)=con(kno2)*tnoy/tnoy1
            con(khno3)=con(khno3)*tnoy/tnoy1
            con(khno2)=con(khno2)*tnoy/tnoy1
            con(kpan)=con(kpan)*tnoy/tnoy1
            if (lnitr) con(knitr)=con(knitr)*tnoy/tnoy1
            con(kpanx)=con(kpanx)*tnoy/tnoy1
            con(kpna)=con(kpna)*tnoy/tnoy1

c           write(*,*) 'hdt,i,j,k= ',hdt,icell,jcell,kcell
c           write(*,*) 'Step4-N balance:tnoy,tnoy1= ',
c    &                  tnoy,tnoy1,difnoy
          endif
c
c
czswa-------------------------------------------------------------------
c  this part of the routine is added to calculate
c  the integrated reaction rate
c  question: should we get off easy (use R) or
c            should we do it the right way
c            (re-calculate with updated conc with the unit in mass)
c  I think I'll do it the right way, not that the integrated rate will
c  change that much (I think) ..............
c
        if (iflg .gt. 0) then
c
c  do coarse grid cells
c
           if (icf .eq. 1) then
              call irr_rate(iflg,k,nrs,hold,r_irr,aac2m,r_inst)
              adt = 0.5*dtuse
              do irs = 1, nreact
                 r_tot(iflg,irs,k) = r_tot(iflg,irs,k) +
     &                adt*(r_old(iflg,irs,k)+r_irr(irs))
                 r_old(iflg,irs,k) = r_irr(irs)
              enddo
c
           elseif (icf .eq. 2) then
c
c  now fine grid cells
c
              call irr_rate(iflg,k,nrs,hold,r_irr,aac2m,r_inst)
              adt = 0.5*dtuse
              do irs = 1, nreact
                 r_totf(iflg,irs,k) = r_totf(iflg,irs,k) +
     &                adt*(r_old(iflg,irs,k)+r_irr(irs))
                 r_old(iflg,irs,k) = r_irr(irs)
              enddo
c
          endif
c
        endif
c
czswz-------------------------------------------------------------------
c
          dttot = dttot + hdt 
          if (dttot .ge. dt*0.9999) go to 9999
c
c  for next time step, use a little big value than the current values,
c      but less than the original time step.
c
          if (1.5*hdt .lt. tdt) then
            hdt=1.5*hdt 
          endif
          if (dttot + hdt .gt. dt) then
            hdt = dt-dttot
          endif
          go to 50
c
        else
c
c  fail to converge, next iteration ?
c
          if (iter .le. nitmax) then
            iconv=1
            go to 115
          else
c
c  still fail to converge after nitmax times, time step must be cut
c
            hdt=hdt/2.0

            if (hdt .lt. dtmin) then
              write(*,*) 'time step is samller than input from',
     &                    'the user: hdt, dtmin= ', hdt,dtmin
              write (*,2900) icell, jcell, kcell, lcell, iiseg
              write (*,2901) hdt
              call flush (6)
              lerr=.true.
              stop
            endif

            if (hdt .lt. dchmin) then
              write(*,*) 'time step is samller than dchmin: ',
     &                    'hdt, dchmin= ', hdt,dtmin
              write (*,2900) icell, jcell, kcell, lcell, iiseg
              write (*,2901) hdt
              call flush (6)
              lerr=.true.
              stop
            endif
            go to 50
          endif
        endif
c
c---------------------------------------------------------------
 1200 format(1x,'   rate  ',/,(1x,i4,e12.5))
 1300 format(1x,'   hold  ',/,(1x,i4,e12.5))
 1400 format(1x,'     rk  ',/,(1x,i4,e12.5))
 2900 format (23h0error in -step4- cell , 3(i4, 2h, ), 10hspecies = ,
     &  i2, 10h  segment=, i2)
 2901 format (34h could not reduce step size below , f8.3)
c---------------------------------------------------------------------
c
 9999 return
      end
