      subroutine tdrate (nreacx, temp,pres)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: tdrate.f,v $
c  Revision 1.3  1999/12/21  19:28:43  tcm
c  Renamed com3 include file to comd3.
c
c  Revision 1.2  1994/02/10  21:01:46  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c
c
c
c  lookup table version 
c
c--input variables
c         nreacx    i  no of gas phase reactions
c         temp      r  temp
c         pres      r  pressure
c
c--output variables
c         rk        r  calculated gas phase rate constants
c         aqrk      r  calculated aqueous phase rate constants
c         kt        r  calculated equilibrium constants
c
c--local variables
c         tterm     r  -(1/tcell - 1/tref)
c
c--routines called
c         none
c
c--called by
c         rchem
c         dochem
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comchem.cmd'
      include 'comd3.cmd'
      include 'comtdr.cmd'
c
c***********************************************************************
c
c     ***  adjust temperature-dependent rate constants
c
c            rk = rkn * exp(-(1/t - 1/298)*acten)
c
c     *** adjust for pressure
c
c         gas-phase rate constants are input in ppm-hr units
c         where the conversion factor from the physical units of molec-cm3-s
c         was calculated at 1 atm.
c         for use in the updated cb-iv, the input rate constants must be
c         multiplied by pressure to the power of the # of reactants - 1
c         (note that this is the actual # of reactants, and not just those
c         which are cbm species -- must count o2, ch4, etc)
c         these are now included in the input file newrate.in as the
c         variable norder
c
c         some reactions are pressure-dependent, that is, they have
c         components of both 2-nd order and 3-rd order reactions.
c         the rate constants for these reactions are calculated from the
c         formula (demore et al., 1987):
c
c                     k3          {1+[log(k3/k2)]**2}**-1
c         k(p) = ----------- * 0.6
c                 1 + k3/k2
c
c                where k3 = k3n(t)*pres**2
c                      k2 = k2n(t)*pres
c
      dimension rk3(mxgrx)
c
c
c     *** adjust rate constants for temperature and pressure
c
      temp = amax1(250.,temp)
      it = aint(temp)
      tcol = float(it) + 1.
      it = it -248
c
      factor = (pres * 298./temp)**2.
      do 250 i=1,nreacx
        delk = rkt(it,i) - rkt(it-1,i)
        delt = tcol - temp
        rk(i) = rkt(it,i) - delt*delk
  250 continue
c
c  don't skip as some is needed for deposition
c  if no aqueous chemistry, skip aqueous rate calculation
c
c      if (.not.laq) go to 450
c  calculate new reaction rate for aqueous reactions
c
c      do 400 i=1,naqrat
c        if (laphot(i)) go to 390
c  340   delk = aqrkt(it,i) - aqrkt(it-1,i)
c        delt = tcol - temp
c        aqrk(i) = aqrkt(it,i) - delt*delk
c  390 continue
c  400 continue
c
c  calculate new equilibrium constant values
c
  450 do 500 i= 1, naqeq
        delk = ktt(it,i) - ktt(it-1,i)
        delt = tcol - temp 
        kt(i) = ktt(it,i) - delt*delk
  500 continue
c
c alter equilibrium constants for h law to yield m/ppm
c
      do 600 i= 1, nhen
        hheff(i) = kt(i)
        kt(i) = kt(i) * 1.e-6 * pres
  600 continue
c
c  now estimate some effective henry's law coefficients to be used
c  in dry deposition calculation
c  wesely recommends ph 7 for leaf surface (c(ih) = 1.e-7)
c
      cih = 1.0e-07
      hheff(1) = hheff(1)*(1.+ kt(24)/cih + kt(24)*kt(25)/(cih*cih))
      hheff(3) = hheff(3)*(1.+ kt(34)/cih)
      hheff(5) = hheff(5)*(1.+ kt(26)/cih)
c  nh3 is included in case we add it as a state species
c     hheff(9) = hheff(9)*(1. + kt(30)*cih/kt(29))
      hheff(12) = hheff(12)*(1. + kt(33)/cih)
      hheff(18) = hheff(18)*(1. + kt(36)/cih)
      hheff(19) = hheff(19)*(1. + kt(45))
c
      return
      end
c
