      subroutine wetdep(ix,jy,rdecay,deltat)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: wetdep.f,v $
c  Revision 1.5  1999/12/21  19:28:51  tcm
c  Renamed com3 include file to comd3.
c
c  Revision 1.4  1999/05/14  17:25:40  dfg
c  Change ald2 to acet due to TOX chemistry.
c  Set kbsulf=ksulf at the very begining.
c
c  Revision 1.3  1998/11/04  02:06:00  tcm
c  Fixed so2 and so4 pointers.
c
c  Revision 1.2  94/02/10  21:02:49  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comuam.cmd'
      include 'comd3.cmd'
      include 'comchem.cmd'
c
c
      dimension rdecay(mxspec,mxz)
c
c     calcualtes a decay rate (1/min) for all species
c     due to precipitation scavenging
c
c        gasseous removal based on equilibrium scavenging assumption
c         (hales and sutter, 1979)
c
c        so4 removal based on parameterization of scott, 1978.
c
c     current implementation:  - bergeron type clouds only
c
c
c
c
      real ma
      real lwc
c
c     equilibrium constants ekh, ek1d, ek2d
c     for 8 species in order:
c                             so2 nox hno3 ox co pan eth ald
c     note: nox has equilibrium constants of 2/3 no2 and 1/3 no
c           ox has equilibrium constant as o3
c
c*****altered to include no & no2 separately; cae 10/92
c                             so2 no no2 hno3 ox co pan eth ald
c
      dimension ekh(9),ek1d(9),ek2d(9),sol(9)
c
      logical lfirst
      save lfirst, sol
      data lfirst/.true./
c
      data ekh/1.23,1.93e-3,7.0e-3,1.983e+5,1.14e-2,1.14e-3,5.0
     &        ,4.87e-3,7.04e3/
      data ek1d/0.016,0.0,0.0,15.4,5*0.0/
      data ek2d/6.24e-8,8*0.0/
c
c     assume ph of 4.5 for now
c
      data ph/4.5/
c
      data  c1/0.0052/
      data ma/0.1/, fr/0.75/
      data roh2o/1.e12/
c
      kbsulf=ksulf
c     first time through define solubility parameter sol
c
      if (.not.lfirst) go to 4
c
      hplus=10.0**(-ph)
      hplus2=hplus*hplus
      do 3 k=1,9
        sol(k)=1.0/(ekh(k)+ekh(k)*ek1d(k)/hplus+ekh(k)*ek1d(k)*ek2d(k)
     &             /hplus2)
    3 continue
    4 continue
      lfirst=.false.
c
c     zero out decay terms
c
      do 15 j=1,nlayer
        do 5 i=1,nospec
          rdecay(i,j)=0.0
    5   continue
   15 continue
      ran=rain(ix,jy)
      if (ran.lt.0.005) return
      rt=h(ix,jy,nlayer)*1000.0
c
c     so4 removal
c
c     terminal velocity of rain drops followed by
c     washout term
c
      rainm=25.4*ran
      d=8.95e-4 * (rainm)**0.21
      vv=130.*d**0.5
      ddtt=rt/vv
c
      if (lso2) then
c
c     weighted average concentration c
c
c     ma is the fraction of subcloud aerosol > 1 micron
c     and hence is washed out (assumed = 1 micron)
c
        htemp=0.
        cx=0.
        do 6 k=1,nlayer
          cx=cx+(h(ix,jy,k)-htemp)*conc(ix,jy,kbsulf,k)
          htemp=h(ix,jy,k)
    6   continue
        cx=cx/h(ix,jy,nlayer)
c
        r1=c1*(ma*cx*1.e-6)*0.3*ddtt
        r2=(fr*cx*1.e-6)*(1.-4.41e-2*(rainm)**(-0.88))/
     &   (1.56 + 0.44*alog(rainm))
c
        rdecay(kbsulf,1)=(r1+r2)*roh2o*(ran/39.37)/(h(ix,jy,nlayer))
     &               *conc(ix,jy,kbsulf,1)*(1./60.)
c
        do 7 k=2,nlayer
          rdecay(kbsulf,k) = rdecay(kbsulf,1) 
     &                * conc(ix,jy,kbsulf,1)/conc(ix,jy,kbsulf,k)
    7   continue
      endif
c
c     so2,nox,hnno3,ox,co,pan,eth removal
c
      ranm=ran/39.37
      do 50 k=1,nlayer
        tempa=temper(ix,jy,k)
        lwc=2.278e-5*tempa*ranm/vv
        top=1.367e-3*ranm*tempa
c
c     so2 removal
c
        if (lso2) rdecay(ksiv,k)=top/(rt*(lwc+sol(1)))
c
c     nox removal
c
        if (.not.lchem) go to 50
        rdecay(kno,k)=top/(rt*(lwc+sol(2)))
        rdecay(kno2,k)=top/(rt*(lwc+sol(3)))
c
c     hno3 removal (cbm-ii has no hno3)
c
        rdecay(khno3,k)=top/(rt*(lwc+sol(4)))
c
c     o3 removal
c
        rdecay(ko3,k)=top/(rt*(lwc+sol(5)))
c
c     co removal
c
        rdecay(kco,k)=top/(rt*(lwc+sol(6)))
c
c     pan removal
c
        rdecay(kpan,k)=top/(rt*(lwc+sol(7)))
c
c     eth removal
c
        rdecay(keth,k)=top/(rt*(lwc+sol(8)))
c
c     acet removal
c
        rdecay(kacet,k)=top/(rt*(lwc+sol(9)))
c
   50 continue
c
c     restrict maximum removal rate due to wet and dry deposition to
c     90% per time step (note: dry deposition takes precedent over wet)
c
      ratmax=0.9/deltat
      do 100 isp=1,nospec
        rdecay(isp,1)=amin1(rdecay(isp,1),ratmax)
        do 95 k=2,nlayer
   95   rdecay(isp,k)=amin1(rdecay(isp,k),ratmax)
  100 continue
c
      return
      end
